# This script demonstrates the use of spatial analysis by distance (SADIE) to interrogate the most appropriate spatial
# resolution for the subsequent analysis and modelling of the Maori colonisation of New Zealand. Essentially, this 
# script provides the spatial template for everything that comes after.

# All the analyses are currently coded to write output to the DEMO_DIR folder to avoid writing over the existing data.
# All the plots and analyses are conducted using these existing data files.


#### Loading Libraries ####
library(SpatialPack)
library(readxl)
library(epiphy)
library(raster)
library(spdep)

#### Establishing source files ####
DATA_DIR <- "./data"
DEMO_DIR <- "./demonstration"
OUTPUT_DIR <- "./models"
PLOT_DIR <- "./figures"

if (file.exists(file.path(DEMO_DIR, "/sadie"))) {
  cat("The folder already exists")
} else {
  dir.create(file.path(DEMO_DIR, "/sadie"))
}

#### Load data sources ####
read_excel_allsheets <- function(filename, tibble = FALSE) {
  sheets <- readxl::excel_sheets(filename)
  x <- lapply(sheets, function(X) readxl::read_excel(filename, sheet = X))
  if(!tibble) x <- lapply(x, as.data.frame)
  names(x) <- sheets
  x
}

NZarch<-read_excel_allsheets(file.path(DATA_DIR, "ArchSitevs14C.xlsx"))

tenKM<-as.data.frame(NZarch$`10KM`)
twentyfiveKM<-as.data.frame(NZarch$`25KM`)
fiftyKM<-as.data.frame(NZarch$`50KM`)

z10 <- cor.spatial(tenKM$`14C`, tenKM$ArchSites, tenKM[c("longitude","latitude")])
z25 <- cor.spatial(twentyfiveKM$`14C`, twentyfiveKM$ArchSites, twentyfiveKM[c("longitude","latitude")])
z50 <- cor.spatial(fiftyKM$`14C`, fiftyKM$ArchSites, fiftyKM[c("longitude","latitude")])

z10
z25
z50

## Step 1: Calculate Lee's I ####
# 50 km resolution
plot(fiftyKM$ArchSites, fiftyKM$`14C`)
cor(fiftyKM$ArchSites, fiftyKM$`14C`)

fifty.sp<-fiftyKM
coordinates(fifty.sp) <- ~ longitude + latitude
grph <- relativeneigh(coordinates(fifty.sp))
nb.50 <- graph2nb(grph)

w.50 <- nb2listw(nb.50, style="W", zero.policy=TRUE)

lee.test(fiftyKM$`14C`, fiftyKM$ArchSites, w.50, zero.policy=TRUE, alternative="less")

# 25 km resolution
plot( twentyfiveKM$ArchSites, twentyfiveKM$`14C`)
mod<-lm(twentyfiveKM$`14C`~twentyfiveKM$ArchSites)
summary(mod)
abline(mod)

twentyfive.sp<-twentyfiveKM
coordinates(twentyfive.sp) <- ~ longitude + latitude
coords.25 <- coordinates(twentyfive.sp)
grph <- relativeneigh(coords.25)
nb.25 <- graph2nb(grph)

w.25 <- nb2listw(nb.25, style="W", zero.policy=TRUE)

lee.test(twentyfiveKM$`14C`, twentyfiveKM$ArchSites, w.25, zero.policy=TRUE, alternative="less")

# 10 km resolution
plot( tenKM$ArchSites, tenKM$`14C`)
mod<-lm(tenKM$`14C`~tenKM$ArchSites)
summary(mod)
abline(mod)

ten.sp<-tenKM
coordinates(ten.sp) <- ~ longitude + latitude
grph <- relativeneigh(coordinates(ten.sp))
nb.10 <- graph2nb(grph)

w.10 <- nb2listw(nb.10, style="W", zero.policy=TRUE)

lee.test(tenKM$`14C`, tenKM$ArchSites, w.10, zero.policy=TRUE, alternative="less")


## Step 2: SADIE at 50km resolution ####

dated.50<-as.data.frame(cbind(fiftyKM$longitude,fiftyKM$latitude,fiftyKM$`14C`))
Archsites.50<-as.data.frame(cbind(fiftyKM$longitude,fiftyKM$latitude,fiftyKM$ArchSites))


SADIE.dated<-sadie(dated.50, index = "Perry",
      nperm = 100, seed = NULL, threads = 1,method = "shortsimplex",
      verbose = TRUE)
SADIE.ArchSites<-sadie(Archsites.50, index = "Perry",
                   nperm = 100, seed = NULL, threads = 1,method = "shortsimplex",
                   verbose = TRUE)
plot(SADIE.dated)
plot(SADIE.ArchSites)

archout_fifty<-as.data.frame(SADIE.ArchSites$info_clust)
archout_fifty$res <- 50

datedout_fifty<-as.data.frame(SADIE.dated$info_clust)
datedout_fifty$res <- 50

dated.cluster<-SADIE.dated$info_clust
ArchSites.cluster<-SADIE.ArchSites$info_clust

cor(dated.cluster$idx_P,ArchSites.cluster$idx_P)
cor.test(dated.cluster$idx_P,ArchSites.cluster$idx_P) 

ID<-list(row.names(ArchSites.cluster))
Correlation<-as.data.frame(cbind(ID,ArchSites.cluster$idx_P,dated.cluster$idx_P))
Correlation$ID<-row.names(Correlation)
names(Correlation)<-c("ID","ArchSites.idx","dated.idx")
Correlation$ArchSites.idx<-as.numeric(Correlation$ArchSites.idx)
Correlation$dated.idx<-as.numeric(Correlation$dated.idx)

r.obt <- cor(Correlation$ArchSites.idx, Correlation$dated.idx)
cat("The obtained correlation is ",r.obt,'\n')

nreps = 10000000 # 10000000 iterations
r.random <- numeric(nreps)
for (i in 1:nreps) {
  Y <- Correlation$ArchSites.idx
  X <- sample(Correlation$dated.idx, nrow(Correlation), replace = FALSE)
  r.random[i] <- cor(X,Y)
}
min(r.random)
max(r.random)
prob <- length(r.random[r.random >= r.obt])/nreps
cat("Probability randomized r >= r.obt",prob)


## Step 3: SADIE at 25km resolution ####

dated.25<-as.data.frame(cbind(twentyfiveKM$longitude,twentyfiveKM$latitude,twentyfiveKM$`14C`))
Archsites.25<-as.data.frame(cbind(twentyfiveKM$longitude,twentyfiveKM$latitude,twentyfiveKM$ArchSites))


SADIE.dated<-sadie(dated.25, index = "Perry",
                   nperm = 100, seed = NULL, threads = 1,method = "shortsimplex",
                   verbose = TRUE)
SADIE.ArchSites<-sadie(Archsites.25, index = "Perry",
                       nperm = 100, seed = NULL, threads = 1,method = "shortsimplex",
                       verbose = TRUE)
plot(SADIE.dated)
plot(SADIE.ArchSites)

archout_twenty5<-as.data.frame(SADIE.ArchSites$info_clust)
archout_twenty5$res <- 25

datedout_twenty5<-as.data.frame(SADIE.dated$info_clust)
datedout_twenty5$res <- 25

dated.cluster<-SADIE.dated$info_clust
ArchSites.cluster<-SADIE.ArchSites$info_clust

cor(dated.cluster$idx_P,ArchSites.cluster$idx_P)
cor.test(dated.cluster$idx_P,ArchSites.cluster$idx_P)

ID<-list(row.names(ArchSites.cluster))
Correlation<-as.data.frame(cbind(ID,ArchSites.cluster$idx_P,dated.cluster$idx_P))
Correlation$ID<-row.names(Correlation)
names(Correlation)<-c("ID","ArchSites.idx","dated.idx")
Correlation$ArchSites.idx<-as.numeric(Correlation$ArchSites.idx)
Correlation$dated.idx<-as.numeric(Correlation$dated.idx)

r.obt <- cor(Correlation$ArchSites.idx, Correlation$dated.idx)
cat("The obtained correlation is ",r.obt,'\n')

nreps = 10000000 # 10000000 iterations
r.random <- numeric(nreps)
for (i in 1:nreps) {
  Y <- Correlation$ArchSites.idx
  X <- sample(Correlation$dated.idx, nrow(Correlation), replace = FALSE)
  r.random[i] <- cor(X,Y)
}
min(r.random)
max(r.random)
prob <- length(r.random[r.random >= r.obt])/nreps
cat("Probability randomized r >= r.obt",prob)


## Step 4: SADIE at 10km resolution ####

dated.10<-as.data.frame(cbind(tenKM$longitude,tenKM$latitude,tenKM$`14C`))
Archsites.10<-as.data.frame(cbind(tenKM$longitude,tenKM$latitude,tenKM$ArchSites))


SADIE.dated<-sadie(dated.10, index = "Perry",
                   nperm = 100, seed = NULL, threads = 1,method = "shortsimplex",
                   verbose = TRUE)
SADIE.ArchSites<-sadie(Archsites.10, index = "Perry",
                       nperm = 100, seed = NULL, threads = 1,method = "shortsimplex",
                       verbose = TRUE)
plot(SADIE.dated)
plot(SADIE.ArchSites)

archout_ten<-as.data.frame(SADIE.ArchSites$info_clust)
archout_ten$res <- 10

datedout_ten<-as.data.frame(SADIE.dated$info_clust)
datedout_ten$res <- 10

dated.cluster<-SADIE.dated$info_clust
ArchSites.cluster<-SADIE.ArchSites$info_clust

cor(dated.cluster$idx_P,ArchSites.cluster$idx_P)
cor.test(dated.cluster$idx_P,ArchSites.cluster$idx_P)

ID<-list(row.names(ArchSites.cluster))
Correlation<-as.data.frame(cbind(ID,ArchSites.cluster$idx_P,dated.cluster$idx_P))
Correlation$ID<-row.names(Correlation)
names(Correlation)<-c("ID","ArchSites.idx","dated.idx")
Correlation$ArchSites.idx<-as.numeric(Correlation$ArchSites.idx)
Correlation$dated.idx<-as.numeric(Correlation$dated.idx)

r.obt <- cor(Correlation$ArchSites.idx, Correlation$dated.idx)
cat("The obtained correlation is ",r.obt,'\n')

nreps = 10000000 # 10000000 iterations
r.random <- numeric(nreps)
for (i in 1:nreps) {
  Y <- Correlation$ArchSites.idx
  X <- sample(Correlation$dated.idx, nrow(Correlation), replace = FALSE)
  r.random[i] <- cor(X,Y)
}
min(r.random)
max(r.random)
prob <- length(r.random[r.random >= r.obt])/nreps
cat("Probability randomized r >= r.obt",prob)


# summary data outputs

rasterA50 <- archout_fifty[, c(1,2,5)]
rasterA50 <- rasterFromXYZ(rasterA50)

rasterD50 <- datedout_fifty[, c(1,2,5)]
rasterD50 <- rasterFromXYZ(rasterD50)

rasterdelta50 <- rasterA50-rasterD50

sadie50 <- stack(rasterA50, rasterD50, rasterdelta50)

# Writing the raster from memory to disk (as a new file)
writeRaster(sadie50, file.path(DEMO_DIR, "/sadie/SADIE_50km_stack.grd"),overwrite=TRUE)

rasterA25 <- archout_twenty5[, c(1,2,5)]
rasterA25 <- rasterFromXYZ(rasterA25)

rasterD25 <- datedout_twenty5[, c(1,2,5)]
rasterD25 <- rasterFromXYZ(rasterD25)

rasterdelta25 <- rasterA25-rasterD25

sadie25 <- stack(rasterA25, rasterD25, rasterdelta25)

# Writing the raster from memory to disk (as a new file)
writeRaster(sadie25, file.path(DEMO_DIR, "/sadie/SADIE_25km_stack.grd"),overwrite=TRUE)

rasterA10 <- archout_ten[, c(1,2,5)]
rasterA10 <- rasterFromXYZ(rasterA10)

rasterD10 <- datedout_ten[, c(1,2,5)]
rasterD10 <- rasterFromXYZ(rasterD10)

rasterdelta10 <- rasterA10-rasterD10

sadie10 <- stack(rasterA10, rasterD10, rasterdelta10)

# Writing the raster from memory to disk (as a new file)
writeRaster(sadie10, file.path(DEMO_DIR, "/sadie/SADIE_10km_stack.grd"),overwrite=TRUE)

