# This script uses a hurdled boosted regression model to generate plausible layers of habitat suitability for Maori in
# New Zealand.  This script begins with a dataframe of presence and absence data, and runs through the development of
# boosted regression trees (BRT), and their validation, to develop appropriate spatial templates for the subsequent modelling
# in later scripts.

# All the analyses are currently coded to write output to the DEMO_DIR folder to avoid writing over the existing data.
# All the plots and analyses are conducted using these existing data files.


#### Loading Libraries ####
library(biomod2)
library(ecospat)
library(gbm)
library(matrixStats)
library(raster)
library(sampling)
library(TSS)

# load Elith functions for BRT
source("scripts/brt.functions.R")

#### Establishing source files ####
DATA_DIR <- "data"
DEMO_DIR <- "demonstration"
OUTPUT_DIR <- "models"
PLOT_DIR <- "figures"

if (file.exists(file.path(DEMO_DIR, "/hurdles"))) {
  cat("The folder already exists")
} else {
  dir.create(file.path(DEMO_DIR, "/hurdles"))
}

#### Load data sources ####
Maori<-read.csv(file.path(DATA_DIR, "NZ25km_current_v2.csv"))
Maori$X=NULL
Maori$X.1=NULL
Maori$field_1=NULL
Maori<- na.omit(Maori) 
Maori$ArchSites_count <- as.numeric(gsub(",","",Maori$ArchSites_count))
Maori$ArchSites.C14 <- as.numeric(gsub(",","",Maori$ArchSites.C14))

dated.true<-subset(Maori,Maori$FALSE_C14 == "TRUE") # subsets to only known locations for Maori C14 dates


#### Step 1: Binomial of absence data ####
Moari.bin <- gbm.step(data = dated.true,
                        gbm.x = c(10:18),
                        gbm.y = 6,
                        family = "bernoulli",
                        tree.complexity = 2,
                        learning.rate = 0.001,
                        bag.fraction = 0.75)
  
  gbm.plot(Moari.bin, n.plots=8, write.title=F)
  
  summary(Moari.bin)
  
  preds <- predict.gbm(Moari.bin, dated.true,n.trees=Moari.bin$gbm.call$best.trees, type="response")
  calc.deviance(dated.true[,6],preds, calc.mean=T, family="bernoulli")

# Simplifying the model
  
  Moari.bin.simp <- gbm.simplify(Moari.bin, n.drops = 8) #tests drops of n-1 factors to optimise simplification
  # optimal at 2
  # dropping:  
  # 1 - NPP_annual [18]
  # 2 - Coldest_temp [11]
  

# 10-fold cross-validation
  
  n <- dim(dated.true)[1]
  MAE <- rep(0,1000)
  RMSE <- rep(0,1000)
  auc.set <- rep(0,1000)
  
  for (i in 1:1000) {
    random=srswor(round(n/10),n)
    bootstrap= subset(cbind(random,dated.true), random==0)[-1]
    new.data= subset(cbind(random,dated.true), random==1)[-1]
    Moari.bin.simp <- gbm.fixed(data=bootstrap, gbm.x = c(11,14:17), gbm.y = 6,family = "bernoulli", learning.rate = 0.001, tree.complexity = 2, n.trees = 6000)
    pred <- predict.gbm(Moari.bin.simp, new.data, n.trees=5000, type="response")
    MAE[i] <- mean(abs(pred-new.data$binary_C14))
    RMSE[i] <- sqrt(mean((new.data$binary_C14-pred)^2))
    auc.set[i] <- roc(new.data[[6]],pred)
  }
  mean(MAE) # Mean prediction error (MAE) = 0.3067724
  mean(RMSE) # root mean square error (RMSE) = 0.3977831
  mean(auc.set) # auc = 0.8281493

  Moari.bin.simp <- gbm.step(data = dated.true,
                        gbm.x = c(11, 14:17),
                        gbm.y = 6,
                        family = "bernoulli",
                        tree.complexity = 2,
                        learning.rate = 0.001,
                        bag.fraction = 0.75)
  
  summary(Moari.bin.simp)
  gbm.plot(Moari.bin.simp, n.plots=4, write.title=F) #plots the partial effects of contributors
  
  dated.true$habitat.fit<-predict.gbm(Moari.bin.simp, dated.true, n.trees=Moari.bin.simp$gbm.call$best.trees, type="response")
    
# Calculate Boyce index
  obs <- (dated.true$habitat.fit
          [which(dated.true$binary_ArchSites==1)])
  ecospat.boyce (fit = dated.true$habitat.fit , obs, nclass=0, 
                 window.w="default", res=100, PEplot = TRUE)  

  #boyce = 0.963  

# Calculate TRUE SKILL SCORE against Archsites

  n <- dim(dated.true)[1]
  TSS<-data.frame("0.1"=numeric(0),
                    "0.2"=numeric(0),
                    "0.3"=numeric(0),
                    "0.4"=numeric(0),
                    "0.5"=numeric(0),
                    "0.6"=numeric(0),
                    "0.7"=numeric(0),
                    "0.8"=numeric(0),
                    "0.9"=numeric(0))
  for (i in 1:1000) {
    random=srswor(round(n/10),n)
    bootstrap= subset(cbind(random,dated.true), random==0)[-1]
    new.data= subset(cbind(random,dated.true), random==1)[-1]
    score<-TSS(new.data$habitat.fit, cont2 = NULL, new.data$binary_ArchSites, thresh1=c(0.1,0.2,0.3,0.4,0.5,0.6,0.7,0.8,0.9), largerPres1 = TRUE,
        largerPres2 = FALSE, link = `&`)
    TSS<-rbind(TSS,score$tss)
    names(TSS)<-c("0.1","0.2","0.3","0.4","0.5","0.6","0.7","0.8","0.9")
  }
  colMeans(TSS) # optimal threshold = 0.7

# 2nd run to further refine
  
  TSS<-data.frame("0.65"=numeric(0),
                  "0.66"=numeric(0),
                  "0.67"=numeric(0),
                  "0.68"=numeric(0),
                  "0.69"=numeric(0),
                  "0.70"=numeric(0),
                  "0.71"=numeric(0),
                  "0.72"=numeric(0),
                  "0.73"=numeric(0),
                  "0.74"=numeric(0),
                  "0.75"=numeric(0),
                  "0.76"=numeric(0),
                  "0.77"=numeric(0),
                  "0.78"=numeric(0),
                  "0.79"=numeric(0),
                  "0.80"=numeric(0))
  for (i in 1:1000) {
    random=srswor(round(n/10),n)
    bootstrap= subset(cbind(random,dated.true), random==0)[-1]
    new.data= subset(cbind(random,dated.true), random==1)[-1]
    score<-TSS(new.data$habitat.fit, cont2 = NULL, new.data$binary_ArchSites, thresh1=c(0.65,0.66,0.67,0.68,0.69,0.70,0.71,0.72,0.73,0.74,0.75,0.76,0.77,0.78,0.79,0.80), largerPres1 = TRUE,
               largerPres2 = FALSE, link = `&`)
    TSS<-rbind(TSS,score$tss)
    names(TSS)<-c("0.65","0.66","0.67","0.68","0.69","0.70","0.71","0.72","0.73","0.74","0.75","0.76","0.77","0.78","0.79","0.80")
  }
  colMeans(TSS) # optimal threshold = 0.66

## calculate binary presence/absence of Maori
  Maori$habitat.fit<-predict.gbm(Moari.bin.simp, Maori, n.trees=Moari.bin.simp$gbm.call$best.trees, type="response")
  Maori$pres.abs<- ifelse(Maori$habitat.fit > 0.66, 1, 0)
  level.plot(Maori$pres.abs, XY = Maori[,c("X_long","Y_lat")], color.gradient = "grey", cex = 2, show.scale = F)

write.csv(Maori, file.path(DEMO_DIR, "/hurdles/NZ25km_current_v2.csv"))

Maori<-read.csv(file.path(DEMO_DIR, "/hurdles/NZ25km_current_v2.csv"))

#### Step 2: Poisson of count data ####
Maori.pos = subset(Maori,Maori$pres.abs == "1") #subsets to only suitable habitat cells from previous fits
levels(as.factor(Maori.pos$C14_count))
Maori.pos<-subset(Maori.pos,Maori.pos$C14_count>0)

  Maori.pos.brt <- gbm.step(data=Maori.pos,
                            gbm.x = c(11:19),
                            gbm.y = 6,
                            family = "poisson",
                            tree.complexity = 2,
                            learning.rate = 0.001,
                            bag.fraction = 0.75)
                          
  summary(Maori.pos.brt)
  summary(Moari.bin.simp)
  
  #try running with ET[14], NPP[19], Warmest[11], coldest[12] +/-River[16]
  
  Maori.pos.brt.1 <- gbm.step(data=Maori.pos,
                            gbm.x = c(11:12,14,19,16),
                            gbm.y = 6,
                            family = "poisson",
                            tree.complexity = 2,
                            learning.rate = 0.001,
                            bag.fraction = 0.75)
  
  summary(Maori.pos.brt.1)
  
  Maori.pos.brt.2 <- gbm.step(data=Maori.pos,
                              gbm.x = c(11:12,14,19),
                              gbm.y = 6,
                              family = "poisson",
                              tree.complexity = 2,
                              learning.rate = 0.001,
                              bag.fraction = 0.75)
  
  summary(Maori.pos.brt.2)
  
  Maori.pos.brt.3 <- gbm.step(data=Maori.pos,
                              gbm.x = c(11:14,19),
                              gbm.y = 6,
                              family = "poisson",
                              tree.complexity = 5,
                              learning.rate = 0.0001,
                              bag.fraction = 0.8)
  
  summary(Maori.pos.brt.3)
  
  gbm.plot(Maori.pos.brt.3, n.plots=8, write.title = F)
  gbm.plot.fits(Maori.pos.brt)

  preds1 <- predict.gbm(Maori.pos.brt.1, Maori.pos, n.trees=Maori.pos.brt.1$gbm.call$best.trees, type="response")
  preds2 <- predict.gbm(Maori.pos.brt.2, Maori.pos, n.trees=Maori.pos.brt.2$gbm.call$best.trees, type="response")
  preds3 <- predict.gbm(Maori.pos.brt.3, Maori.pos, n.trees=Maori.pos.brt.3$gbm.call$best.trees, type="response")
  
  max(preds3)
  calc.deviance(Maori.pos[,6],preds3, calc.mean=T, family="poisson") # residual deviance=4.359261
  
  #QQ plot of prediction against observations (C14 dated sites)
  plot(Maori.pos[,6], preds1)
  plot(Maori.pos[,6], preds2)
  plot(Maori.pos[,6], preds3)
  
  lm1 <- lm(preds1 ~ Maori.pos[,6])
  summary(lm1)
  lm2 <- lm(preds2 ~ Maori.pos[,6])
  summary(lm2)
  lm3 <- lm(preds3 ~ Maori.pos[,6])
  summary(lm3)# R2 explained = 0.4305
  abline(lm3$coefficients[1], lm3$coefficients[2])
  
  ## 10-fold cross-validation
  
  n <- dim(Maori.pos)[1]
  MAE <- rep(0,1000)
  M.err <- rep(0,1000)
  RMSE <- rep(0,1000)
  pred <- rep(0,1000)
  predSD <- rep(0,1000)
  counts <- rep(0,1000)
  countsSD <- rep(0,1000)
  
  for (i in 1:1000) {
    random=srswor(round(n/10),n)
    bootstrap= subset(cbind(random,Maori.pos), random==0)[-1]
    new.data= subset(cbind(random,Maori.pos), random==1)[-1]
    Maori.pos.brt <- gbm.fixed(data=bootstrap, gbm.x = c(11:14,19), gbm.y = 6,family = "poisson", learning.rate = 0.0001, bag.fraction = 0.8, tree.complexity = 5, n.trees = 5000)
    preds <- predict.gbm(Maori.pos.brt, new.data, n.trees=Maori.pos.brt$gbm.call$best.trees, type="response")
    MAE[i] <- mean(abs(preds-new.data$C14_count))
    M.err[i] <- mean((preds-new.data$C14_count))
    RMSE[i] <- sqrt(mean((preds-new.data$C14_count)^2))
    pred[i] <- mean(preds)
    predSD[i] <- sd(preds)
    counts[i] <- mean(new.data$C14_count)
    countsSD[i] <- sd(new.data$C14_count)
  }
  mean(MAE) # mean(MAE) = 3.64
  abs(mean(M.err)) # abs(mean(M.err)) = 0.22
  mean(RMSE) # mean(RMSE) = 5.13
  mean(pred) # mean(pred) = 4.47
  mean(predSD) # mean(predSD) = 0.81
  mean(counts) # mean(counts) = 4.72
  mean(countsSD) # mean(countsSD) = 5.15
  
  mean(predSD)/(sqrt(n))
  mean(countsSD)/(sqrt(n))
  
  
# 33% bootstrap
  
  n <- dim(Maori.pos)[1]
  MAE <- rep(0,1000)
  RMSE <- rep(0,1000)
  pred <- rep(0,1000)
  counts <- rep(0,1000)
  
  for (i in 1:1000) {
    random=srswor((round(n/3)),n)
    bootstrap= subset(cbind(random,Maori.pos), random==0)[-1]
    new.data= subset(cbind(random,Maori.pos), random==1)[-1]
    Maori.pos.brt <- gbm.fixed(data=bootstrap, gbm.x = c(11:14,19), gbm.y = 6,family = "poisson", learning.rate = 0.0005, bag.fraction = 0.8, tree.complexity = 5, n.trees = 1000)
    preds <- predict.gbm(Maori.pos.brt, new.data, n.trees=Maori.pos.brt$gbm.call$best.trees, type="response")
    MAE[i] <- mean(abs(preds-new.data$C14_count))
    RMSE[i] <- sqrt(mean((new.data$C14_count)^2))
    pred[i] <- mean(preds)
    counts[i] <- mean(new.data$C14_count)
  }
  
  mean(MAE) # Mean prediction error (MAE) = 3.379241
  mean(RMSE) # root mean square error (RMSE) = 6.731769
  mean(pred) # mean prediction estimated = 4.293006
  mean(counts) # mean counts sampled = 4.4585
  sd(counts) # sd of counts sampled = 0.6308963
  
# 33% bootstrap against Archsites
  
  n <- dim(Maori.pos)[1]
  MAE <- rep(0,1000)
  RMSE <- rep(0,1000)
  pred <- rep(0,1000)
  counts <- rep(0,1000)
  
  for (i in 1:1000) {
    random=srswor((round(n/3)),n)
    bootstrap= subset(cbind(random,Maori.pos), random==0)[-1]
    new.data= subset(cbind(random,Maori.pos), random==1)[-1]
    Maori.pos.brt <- gbm.fixed(data=bootstrap, gbm.x = c(11:14,19), gbm.y = 6,family = "poisson", learning.rate = 0.0005, bag.fraction = 0.8, tree.complexity = 5, n.trees = 1000)
    preds <- predict.gbm(Maori.pos.brt, new.data, n.trees=Maori.pos.brt$gbm.call$best.trees, type="response")
    MAE[i] <- mean(abs(preds-new.data$ArchSites.))
    RMSE[i] <- sqrt(mean((new.data$ArchSites.)^2))
    pred[i] <- mean(preds)
    counts[i] <- mean(new.data$ArchSites.)
  }
  
  mean(MAE) # Mean prediction error (MAE) = 115.5099
  mean(RMSE) # root mean square error (RMSE) = 213.623
  mean(pred) # mean prediction estimated = 4.272482
  mean(counts) # mean counts (archsites) = 119.1428
  sd(counts) # sd of counts sampled = 22.41338
  
  preds <- predict.gbm(Maori.pos.brt.3, Maori.pos, n.trees=Maori.pos.brt.3$gbm.call$best.trees, type="response")
  
  #QQ plot of prediction against observations (ArchSites)
  Maori.pos[,9]<-as.numeric(Maori.pos[,9])
  plot(Maori.pos[,9], preds)
  lm <- lm(preds ~ Maori.pos[,9])
  summary(lm)
  abline(lm$coefficients[1], lm$coefficients[2])
  
  #(Intercept)    3.9434111  0.0907608  43.448  < 2e-16 ***
  #Maori.pos[, 9] 0.0021272  0.0004207   5.057 1.38e-06 ***
  #F-statistic: 25.57 on 1 and 133 DF,  p-value: 1.384e-06
  #R2 explained=0.3093
  
  
  ## calculate abundance of Maori
  Maori$est.abund<-predict.gbm(Maori.pos.brt.3, Maori, n.trees=Maori.pos.brt.3$gbm.call$best.trees, type="response")
  
  level.plot(Maori$est.abund, XY = Maori[,c("X_long","Y_lat")], color.gradient = "grey", cex = 2, show.scale = F)

write.csv(Maori, file.path(DEMO_DIR, "/hurdles/NZ25km_current_v2.csv"))

  
#### Step 3: Combine hurdle analyses ####

# 5000 replicate bootstrap to predict abundance and combine with habitat suitability (zero hurdle)
# results in mean grids of counts and abundance +/- SD

M2<-Maori
n <- dim(Maori.pos)[1]
#n <- dim(M2)[1]
pred <- matrix(nrow = nrow(M2), ncol = 5000)
combo <- matrix(nrow = nrow(M2), ncol = 5000)
combo.s <- matrix(nrow = nrow(M2), ncol = 5000)

#30% bootstrap
for (i in 1:5000) {
  random=srswor(3*(round(n/10)),n)
  bootstrap= subset(cbind(random,Maori.pos), random==0)[-1]
  #bootstrap= subset(cbind(random,M2), random==0)[-1]
  new.data= subset(cbind(random,Maori.pos), random==1)[-1]
  #new.data= subset(cbind(random,M2), random==1)[-1]
  #Maori.pos.brt <- gbm.step(data=Maori.pos, gbm.x = 11:19, gbm.y = 6, family = "poisson", tree.complexity = 2, learning.rate = 0.001,bag.fraction = 0.75,verbose = FALSE)
  Maori.pos.brt <- gbm.fixed(data=bootstrap, gbm.x = c(11:14,19), gbm.y = 6,family = "poisson", learning.rate = 0.0001, bag.fraction = 0.8, tree.complexity = 5, n.trees = 9000)
  pred[,i] <- predict.gbm(Maori.pos.brt, M2, n.trees=Maori.pos.brt$gbm.call$best.trees, type="response")
  combo[,i] <- (predict.gbm(Maori.pos.brt, M2, n.trees=Maori.pos.brt$gbm.call$best.trees, type="response"))*M2$habitat.fit
  combo.s[,i] <- combo[,i]/max(combo[,i])
  }

sd(rowMeans(combo))

M2$mean.est.count<-rowMeans(pred)
M2$est.count.SD<-rowSds(pred)
M2$mean.est.abund<-rowMeans(combo)
M2$est.abund.SD<-rowSds(combo)
M2$mean.est.StdAbund<-rowMeans(combo.s)
M2$est.StdAbund.SD<-rowSds(combo.s)

level.plot(M2$mean.est.count, XY = M2[,c("X_long","Y_lat")], color.gradient = "grey", cex = 1.4, show.scale = F)
level.plot(M2$est.count.SD, XY = M2[,c("X_long","Y_lat")], color.gradient = "grey", cex = 1.4, show.scale = F)
level.plot(M2$mean.est.abund, XY = M2[,c("X_long","Y_lat")], color.gradient = "grey", cex = 1.4, show.scale = F)
level.plot(M2$est.abund.SD, XY = M2[,c("X_long","Y_lat")], color.gradient = "grey", cex = 1.4, show.scale = F)
level.plot(M2$mean.est.StdAbund, XY = M2[,c("X_long","Y_lat")], color.gradient = "grey", cex = 1.4, show.scale = F)
level.plot(M2$est.StdAbund.SD, XY = M2[,c("X_long","Y_lat")], color.gradient = "grey", cex = 1.4, show.scale = F)

mean(M2$est.count.SD)

min(M2$mean.est.StdAbund)
max(M2$mean.est.StdAbund)
min(M2$est.StdAbund.SD)
max(M2$est.StdAbund.SD)

min(M2$mean.est.abund)
max(M2$mean.est.abund)
min(M2$est.abund.SD)
max(M2$est.abund.SD)

Maori$standard.abundance<-M2$mean.est.StdAbund

write.csv(M2, file.path(DEMO_DIR, "/hurdles/M2.csv"))

M2<-read.csv(file.path(DEMO_DIR, "/hurdles/M2.csv"))

gridshp<-shapefile(file.path(DATA_DIR, "25km_current.shp"))
template <- raster(ncol=length(unique(gridshp$X_long)), nrow=length(unique(gridshp$Y_lat)))
extent(template) <- extent(gridshp)
values(template) <- 1

xy <- cbind(M2$X_long,M2$Y_lat)
rasterDF <- rasterize(xy, template, M2$habitat.fit)
# Writing the raster from memory to disk (as a new file)
writeRaster(rasterDF, file.path(DEMO_DIR, "/hurdles/Maori_likelihood.asc"), format="ascii",overwrite=TRUE)

rasterDF <- rasterize(xy, template, M2$mean.est.abund)
# Writing the raster from memory to disk (as a new file)
writeRaster(rasterDF, file.path(DEMO_DIR, "/hurdles/Maori_EstabundMean.asc"), format="ascii",overwrite=TRUE)

rasterDF <- rasterize(xy, template, M2$est.abund.SD)
# Writing the raster from memory to disk (as a new file)
writeRaster(rasterDF, file.path(DEMO_DIR, "/hurdles/Maori_EstabundSD.asc"), format="ascii",overwrite=TRUE)

rasterDF <- rasterize(xy, template, M2$mean.est.StdAbund)
# Writing the raster from memory to disk (as a new file)
writeRaster(rasterDF, file.path(DEMO_DIR, "/hurdles/Maori_StdabundMean.asc"), format="ascii",overwrite=TRUE)

rasterDF <- rasterize(xy, template, M2$est.StdAbund.SD)
# Writing the raster from memory to disk (as a new file)
writeRaster(rasterDF, file.path(DEMO_DIR, "/hurdles/Maori_StdabundSD.asc"), format="ascii",overwrite=TRUE)


M3 = subset(M2,M2$pres.abs == "1") #subsets to only suitable habitat cells from previous fits
levels(as.factor(M3$C14_count))
M3<-subset(M3,M3$C14_count>0)

xy <- cbind(M3$X_long,M3$Y_lat)
rasterDF <- rasterize(xy, template, M3$mean.est.count)
# Writing the raster from memory to disk (as a new file)
writeRaster(rasterDF, file.path(DEMO_DIR, "/hurdles/Maori_EstCountsMean.asc"), format="ascii",overwrite=TRUE)

rasterDF <- rasterize(xy, template, M3$est.count.SD)
# Writing the raster from memory to disk (as a new file)
writeRaster(rasterDF, file.path(DEMO_DIR, "/hurdles/Maori_EstCountsSD.asc"), format="ascii",overwrite=TRUE)


#### Step 4: Interpolate to 20km resolution for SEPMS modelling ####
# This step is required to guarantee that the population modelling undertaken in the script '03_Maori_Simulations' can differentiate
# between the North Island and the South Island, and will not spread communities across shared dispersal neighbourhoods that bridge
# the Cook Strait. This is done by simple bilinear interpolation of the model projections.

NZ_raster <- raster(file.path(DATA_DIR, "nz_region_mask.grd"))
plot(NZ_raster , main = "New Zealand raster",
             xlab = "Longitude (degrees)", ylab = "Latitude (degrees)",
             colNA = "blue")

rawmeangrid <- raster (file.path(DEMO_DIR, "/hurdles/Maori_StdabundMean.asc"))
crs(rawmeangrid) <- crs(NZ_raster)
rawmeangrid <- projectRaster(rawmeangrid, crs=crs(NZ_raster),res = res(NZ_raster), method="bilinear")
writeRaster(rawmeangrid, file.path(DEMO_DIR, "/hurdles/maori_mean.grd"), overwrite=TRUE)

rawsdgrid <- raster (file.path(DEMO_DIR, "/hurdles/Maori_StdabundSD.asc"))
crs(rawsdgrid) <- crs(NZ_raster)
rawsdgrid <- projectRaster(rawsdgrid, crs=crs(NZ_raster),res = res(NZ_raster), method="bilinear")
writeRaster(rawsdgrid, file.path(DEMO_DIR, "/hurdles/maori_sd.grd"), overwrite=TRUE)

  