# This script takes the habitat suitability rasters generated in the previous script and uses them as a template for a
# spatially-explicit population model executed using the 'poems' package. This script goes through all the requirements for
# developing the 'poems' model, running 25,000 simulations of the model, and validating the simulations against empirical
# targets using Approximate Bayesian Computation. It also optimises the simulations by resampling the parameter space.

# All the analyses are currently coded to write output to the DEMO_DIR folder to avoid writing over the existing data.
# All the plots and analyses are conducted using these existing data files.

#### Loading Libraries ####
library(abc)
library(bayestestR)
library(data.table)
library(easyGgplot2)
library(fitdistrplus)
library(foreach)
library(mc2d)
library(poems)
library(raster)

source("./scripts/pp_check.R")

# Load boutique ABC functions
source("scripts/custom_funcs.R")


#### Establishing source files ####
DATA_DIR <- "data"
DEMO_DIR <- "demonstration"
OUTPUT_DIR <- "models"
PLOT_DIR <- "figures"

#### 'poems' parameters ####
SAMPLES <- 25000 # the number of samples generated by Latin Hypercube Sampling
PARALLEL_CORES <- 6 # the number of cores defaulted to in parallel simulations
TIMESTEPS <- 1050 # the number of time steps over which the simulations run (in years)


#### Step 1: Establish the study region ####

NZ_raster <- raster(file.path(DATA_DIR, "nz_region_mask.grd"))
raster::plot(NZ_raster , main = "New Zealand raster",
             xlab = "Longitude (degrees)", ylab = "Latitude (degrees)",
             colNA = "blue")

region <- Region$new(template_raster = NZ_raster)
cells <- region$region_cells
raster::plot(region$region_raster, main = "New Zealand study region (cell indices)",
             xlab = "Longitude (degrees)", ylab = "Latitude (degrees)",
             colNA = "blue")

nz_bioregions <- raster::raster(file.path(DATA_DIR, "nz_bioregions.grd"))
raster::values(nz_bioregions) <- raster::values(nz_bioregions)
raster::plot(nz_bioregions, colNA = "blue")

reg_data <- read.csv(file.path(DATA_DIR, "nz_region_data.csv"))
reg_data

bioregional_indices <- lapply(as.list(reg_data$index),
                       function(i) {
                         which(nz_bioregions[region$region_indices] == i)
                       })

# Read mean and sd human habitat suitability
mean_raster <- raster(file.path(DATA_DIR, "maori_mean.grd"))*NZ_raster
sd_raster <- raster(file.path(DATA_DIR, "maori_sd.grd"))*NZ_raster
raster::plot(mean_raster, colNA = "blue")
raster::plot(sd_raster, colNA = "blue")

# Establish environmental correlation
env_corr <- SpatialCorrelation$new(region = region, amplitude = 0.496, breadth = 80, distance_scale = 1000) # km
env_corr$calculate_compact_decomposition(decimals = 4)
lapply(env_corr$get_compact_decomposition(), dim)

# Calculate distances between cells for possible neighbourhoods
dispersal_gen <- DispersalGenerator$new(region = region,
                                        distance_scale = 1000, # 1 km
                                        dispersal_max_distance = 70)
dispersal_gen$calculate_distance_data() # note: distance class = distance when < 1000 classes
distance_data <- dispersal_gen$distance_data[[1]][, c(1:2, 4)]
names(distance_data)[3] <- "distance"
head(distance_data)


#### Step 2: Build the simulation model for the study region ####

# Simulation model
model_template <- SimulationModel$new(region = region,
                                      time_steps = TIMESTEPS,
                                      model_attributes = c("human_density"))
model_template$simulation_function <- function(model) {
  return(model$get_attributes("human_density"))
}

# Build generators for dynamically generating model parameters

human_density_gen <- Generator$new(description = "human density",
                                   region = region,
                                   spatial_correlation = env_corr,
                                   time_steps = TIMESTEPS,
                                   generate_rasters = FALSE,
                                   simulation_start_year = 850, # AD
                                   hs_mean = mean_raster[region$region_indices],
                                   hs_sd = sd_raster[region$region_indices],
                                   distance_data = dispersal_gen$distance_data[[1]][, c(1:2, 4)],
                                   inputs = c("human_founding_population",    # 1. Founding population size (200 - 600)
                                              "human_growth_rate",            # 2. Intrinsic growth rate (~R = 1.005 - 1.022)
                                              "human_settlement_time",        # 3. Settlement time (1260 - 1320)
                                              "human_neighbourhood_distance", # 4. Neighbourhood/foraging range distance (30-70) km [rounded]
                                              "human_min_neighbourhood"),     # 5. Minimum neighbourhood (settlement) size
                                   outputs = "human_density",
                                   generative_requirements = list(human_density = "function"))

human_density_gen$add_distribution_template("sample_hs",
                                            distr_type = "beta",
                                            distr_params = list(mean = "hs_mean", sd = "hs_sd"))

human_density_gen$add_function_template(
  "human_density",
  function_def = function (params) {

    # Unpack parameters
    region <- params$region
    cells <- region$region_cells
    founding_population <- params$human_founding_population
    growth_rate <- params$human_growth_rate
    settlement_time <- params$human_settlement_time
    neighbourhood_distance <- params$human_neighbourhood_distance
    min_neighbourhood <- params$human_min_neighbourhood
    sample_hs <- params$sample_hs[,1]
    distance_data <- params$distance_data
    time_steps <- params$time_steps
    simulation_start_year <- params$simulation_start_year

    # Calculate human occupation sequence based on best available cells (deterministic) with the
    # sampled HS. Occupied cells must be within a neighbourhood having a total population of
    # at least the minimum (settlement) size.

    # Define neighbourhoods (reachable booleans) from distance data
    neighbourhood_matrix <- diag(nrow = cells, ncol = cells) > 0
    colnames(neighbourhood_matrix) <- rownames(neighbourhood_matrix) <- 1:cells
    in_range_indices <- distance_data[which(distance_data$distance <= neighbourhood_distance), 1:2]
    neighbourhood_matrix[as.matrix(in_range_indices)] <- TRUE
    # neighbourhood_matrix[1:30, 1:10]
    # summary(colSums(neighbourhood_matrix)) # neighbourhood sizes

    # Neighbourhood and occupancy sequence in order of greatest habitat suitability
    sequence_indices <- order(colSums(sample_hs*neighbourhood_matrix), decreasing = TRUE)
    neighbourhood_sequence <- neighbourhood_matrix[, sequence_indices]
    # neighbourhood_sequence[1:30, 1:10]
    occupancy_sequence <- apply(matrix(1:cells), 1, function(i) {
      as.logical(rowSums(as.matrix(neighbourhood_sequence[,1:i])))
    })
    dimnames(occupancy_sequence) <- dimnames(neighbourhood_sequence)
    # occupancy_sequence[1:30, 1:10]

    # Create a sequence of normalised HS for the unfolding occupancy
    sequence_matrix <- occupancy_sequence*sample_hs
    sequence_matrix <- sequence_matrix/array(rep(colSums(sequence_matrix), each = cells),
                                             dim(sequence_matrix))
    # sequence_matrix[1:30, 1:8]
    # unname(colSums(sequence_matrix))
    sequence_fraction <- colSums(sequence_matrix*neighbourhood_sequence)

    # Calculate human population via growth curve
    human_population <- array(0, time_steps)
    human_population[settlement_time - simulation_start_year + 1] <- founding_population
    for (i in (settlement_time - simulation_start_year + 2):time_steps) {
      human_population[i] <- round(human_population[i - 1]*growth_rate)
    }

    # Unfold the sequence using the minimum neighbourhood (settlement) size
    settlement_sequence <- array(NA, time_steps)
    for (tm in 1:time_steps) {
      threshold_indices <- which(human_population[tm]*sequence_fraction < min_neighbourhood)
      if (length(threshold_indices)) {
        settlement_sequence[tm] <- max(threshold_indices[1] - 1, 1)
      } else {
        settlement_sequence[tm] <- cells
      }
    }
    # settlement_sequence

    # Calculate human population across the settlement sequence
    return(sequence_matrix[, settlement_sequence]*
             array(rep(human_population, each = cells), c(cells, time_steps)))
  },
  call_params = c("region",
                  "human_founding_population",    # 1. Founding population size (200 - 600)
                  "human_growth_rate",            # 2. Intrinsic growth rate (~R = 1.005 - 1.022)
                  "human_settlement_time",        # 3. Settlement time (1260 - 1320)
                  "human_neighbourhood_distance", # 4 Neighbourhood/foraging range distance (30-70) km [rounded]
                  "human_min_neighbourhood",      # 5. Minimum neighbourhood (settlement) size
                  "sample_hs",
                  "distance_data",
                  "time_steps",
                  "simulation_start_year"))

# OPTIONAL STEP: test human generator

# this step isn't essential to running the simulations later, but it tests the simulation framework established above
# using values that are median to the sampling to come next. Essentially, it's a way to see if the simulation has
# glitches or errors coded into it

test_gen <- human_density_gen$generate(input_values = list(
  human_founding_population = 400,   # 1. Founding population size (200 - 600)
  human_growth_rate = 1.013,         # 2. Intrinsic growth rate (~R = 1.005 - 1.022)
  human_settlement_time = 1314,      # 3. Settlement time (1260 - 1320)
  human_neighbourhood_distance = 50, # 4. Neighbourhood/foraging range distance (30-70) km [rounded]
  human_min_neighbourhood = 25       # 5. Minimum neighbourhood (settlement) size
))
raster::plot(region$raster_from_values(test_gen$human_density[,55]), colNA = "blue")
raster::plot(region$raster_from_values(test_gen$human_density[,465]), colNA = "blue")
raster::plot(region$raster_from_values(test_gen$human_density[,800]), colNA = "blue")

rm(test_gen) #removes the test generation to avoid any conflation in subsequent simulations


### Step 3: Generator uninformed parameters for each simulation - Latin Hypercube Sampling ####
lhs_gen <- LatinHypercubeSampler$new()
lhs_gen$set_uniform_parameter("human_founding_population", lower = 200, upper = 600, decimals = 0)
lhs_gen$set_uniform_parameter("human_growth_rate", lower = 1.005, upper = 1.05, decimals = 3)
lhs_gen$set_uniform_parameter("human_settlement_time", lower = 1230, upper = 1314, decimals = 0)
lhs_gen$set_uniform_parameter("human_neighbourhood_distance", lower = 30, upper = 70, decimals = 0)
lhs_gen$set_uniform_parameter("human_min_neighbourhood", lower = 20, upper = 80, decimals = 0)
sample_data <- lhs_gen$generate_samples(number = SAMPLES,
                                        #random_seed = 123
                                        )
head(sample_data, 10) # examine

if (file.exists(file.path(DEMO_DIR, "/v1"))) {
  cat("The folder already exists")
} else {
  dir.create(file.path(DEMO_DIR, "/v1"))
}

write.csv(sample_data,file.path(DEMO_DIR, "/v1/sample_data.csv"))


### Step 4: Build a simulation manager to run each simulation ####
 
sample_data <- read.csv(file.path(DEMO_DIR, "/v1/sample_data.csv"))

# OR reads existing data from the OUTPUT_DIR (all 25k samples)
# sample_data <- read.csv(file.path(OUTPUT_DIR, "/v1/sample_data.csv"))
sample_data$X <- NULL

if (file.exists(file.path(DEMO_DIR, "/v1/sims"))) {
  cat("The folder already exists")
} else {
  dir.create(file.path(DEMO_DIR, "/v1/sims"))
}

sim_manager <- SimulationManager$new(sample_data = sample_data,
                                     model_template = model_template,
                                     generators = list(human_density_gen),
                                     parallel_cores = PARALLEL_CORES,
                                     results_ext = ".RDS",
                                     results_dir = paste0(DEMO_DIR, "/v1/sims"))
run_output <- sim_manager$run()
run_output$summary
run_output$failed_indices

### Step 5: Build a results manager to generate summary results (metrics) ####

# Set targets for our summary metrics (used to calculate combined metric errors)

settlements <- read.csv(file.path(DATA_DIR, "Maori_14C_targets.csv"))
settlements <- subset(settlements, settlements$Filter_2 == "KEEP")

settlements$region_index <- apply(as.matrix(settlements[, c("Longitude", "Latitude")]), 1,
                                  function (coord) which.min(geosphere::distGeo(region$coordinates, coord)))
earliest_indices <- apply(matrix(1:region$region_cells), 1,
                          function(i) {
                            Cal_Age_i <- (settlements$Cal_Age*(settlements$region_index == i)*
                                            (settlements$Reliability == "reliable")*
                                            (settlements$Cal_Age <= 1000))
                            which.max(Cal_Age_i)*(any(Cal_Age_i > 0) | NA)
                          })
earliest_settlements <- settlements[earliest_indices[!is.na(earliest_indices)],]
settlements_sites <- region$region_raster*0
settlements_sites[region$region_indices[earliest_settlements$region_index]] <-
  earliest_settlements$Cal_Age - earliest_settlements$Cal_Err
raster::plot(settlements_sites, colNA = "blue")

youngest_sett_array <- array(settlements_sites[region$region_indices])
youngest_sett_array[youngest_sett_array == 0] <-NA

settlements_sites[region$region_indices[earliest_settlements$region_index]] <-
  earliest_settlements$Cal_Age + earliest_settlements$Cal_Err
raster::plot(settlements_sites, colNA = "blue")

oldest_sett_array <- array(settlements_sites[region$region_indices])
oldest_sett_array[oldest_sett_array == 0] <-NA

targets <- list(
  south_settlement = array(c(1086,712)),
  oldest_settlement = oldest_sett_array, # data frame with locations and upper CI YBP
  youngest_settlement = youngest_sett_array, # data frame with locations and upper CI YBP
  
  total_abundance = c(100000, 150000) # est. CI 100-150% of Cook's 1769 population estimates (from Holdaway et al 2014)
)

#### TEST the summary functions via:

# From this point forwards the script can be repeated iteratively to generate increasingly refined parameter space to optimise
# the simulations. Each repeat set of simulation parameters will be sampled from the posteriors of previous simulations. In order
# to set which set of samples and simulations are being tested, the value of 'runset' needs to be changed.

runset <- 1 # sets the sampling version (numeric from 1 to n)

TEST_DIR <- paste0(OUTPUT_DIR, "/v", runset) # reads existing data from the OUTPUT_DIR 
TESTWRITE_DIR <- paste0(DEMO_DIR, "/v",runset) # writes new data to the DEMO_DIR
SIMS_DIR <- paste0(DEMO_DIR, "/v",runset,"/sims") # writes new data to the DEMO_DIR


sample_data <- read.csv(file.path(DEMO_DIR, "v1/sample_data.csv"))
# OR reads existing data from the OUTPUT_DIR (all 25k samples)
# sample_data <- read.csv(file.path(TEST_DIR, "sample_data.csv"))
sample_data$X <- NULL

results_test <- NULL
 sample_matrix_list <-NULL
 human_density_list <- vector("list", SAMPLES)
 occupancy_date_list <- vector("list", SAMPLES)
 regional_settlement_list <- vector("list", SAMPLES)
 final_abundance_list <- vector("list", SAMPLES)


# PARALLEL TESTING TARGETS
cl <- parallel::makeCluster(PARALLEL_CORES)
doParallel::registerDoParallel(cl)
partest <- foreach(s = 1:SAMPLES, .packages = "poems") %dopar% {

  sample = paste0("sample_",s,"_results.RDS")
  results = PopulationResults$new(results = readRDS(file.path(SIMS_DIR, sample)),
                                nz_bioregions = nz_bioregions, # attachments
                                bioregional_indices = bioregional_indices,
                                targets = targets)

# total population size
  final_abundance <- sum(results$get_attribute("human_density")[,920]) #Maori population in layer 920 (1769) = 150 000

## TESTING ACCURACY
  population_accuracy <- if (final_abundance>targets[[4]][1] & final_abundance<targets[[4]][2]) {
    population_accuracy <- 0} else {
      if (final_abundance<targets[[4]][1]){
        population_accuracy <- (targets[[4]][1]-final_abundance)/targets[[4]][1]
      }else{
        population_accuracy <- (final_abundance-targets[[4]][2])/targets[[4]][2]
      }}
  
# SouthIs_settlement
  north<-unlist(results$get_attribute("bioregional_indices")[c(1:10)])
  south<-unlist(results$get_attribute("bioregional_indices")[c(11:18)])
  
  island_indices <- list(north,south)
  
  regional_abundance <- t(array(unlist(lapply(island_indices,
                                               function (indices) {
                                                 colSums(results$get_attribute("human_density")[indices,])
                                               })), c(TIMESTEPS, 2)))#time_steps by number of regions
   settlement_date <- list(c(1:2))
   for(j in 1:2) {
     sett_yr1 <- if(sum(regional_abundance[j,]) == 0){sett_yr = 0}else{min(which(regional_abundance[j,] > 0))}
     settlement_date[[1]][j] <- 1950-(human_density_gen$template_attached$simulation_start_year + sett_yr1)
   }
  
## TESTING ACCURACY
   if(settlement_date[[1]][2]<targets[[1]][1] & settlement_date[[1]][2]>targets[[1]][2]){
     South_settlement_timing <- 0}else {
        if (settlement_date[[1]][2]>targets[[1]][1]){
          South_settlement_timing <- (settlement_date[[1]][2]-targets[[1]][1])
     }else{
       South_settlement_timing <- (targets[[1]][2]-settlement_date[[1]][2])
       }}
   
   
#location_settlement
  occupancy <- results$get_attribute("human_density")
  occupancy_date <- list(c(1:532))

  for(j in 1:532) {
    sett_yr2 <- if(sum(occupancy[j,]) == 0){sett_yr = 0}else{min(which(occupancy[j,] > 0))}
    occupancy_date[[1]][j] <- 1950-(human_density_gen$template_attached$simulation_start_year + sett_yr2)
  }

## TESTING ACCURACY
  # for hitting before oldest evidence
   occupancy_accuracy <- NULL
     for(i in 1:532){
       occupancy_accuracy[i] <- occupancy_date[[1]][i]-targets[[2]][i]
                       }
   occupancy_accuracy <-  length(which(occupancy_accuracy>0))/length(which(targets[[2]]>0))

## build summary results
  sim_ID <- sub(".RDS.*", "", sample)
  human_founding_population <- sample_data[s,1]
  human_growth_rate <- sample_data[s,2]
  human_settlement_time <- sample_data[s,3]
  human_neighbourhood_distance <- sample_data[s,4]
  human_min_neighbourhood <- sample_data[s,5]
  final_abundance
  summary_results<-NULL
  summary_results <- cbind(sim_ID,
                           human_founding_population,
                           human_growth_rate,
                           human_settlement_time,
                           human_neighbourhood_distance,
                           human_min_neighbourhood,
                           final_abundance,
                           population_accuracy,
                           South_settlement_timing,
                           occupancy_accuracy)
  }

#results_test
summary_metric_data <- as.data.frame(do.call(rbind,partest))

# assemble results metrics into a data frame and write to file
#summary_metric_data <- as.data.frame(results_test)
summary_metric_data[,2] <- as.numeric(summary_metric_data[,2])
summary_metric_data[,3] <- as.numeric(summary_metric_data[,3])
summary_metric_data[,4] <- as.numeric(summary_metric_data[,4])
summary_metric_data[,5] <- as.numeric(summary_metric_data[,5])
summary_metric_data[,6] <- as.numeric(summary_metric_data[,6])
summary_metric_data[,7] <- as.numeric(summary_metric_data[,7])
summary_metric_data[,8] <- as.numeric(summary_metric_data[,8])
summary_metric_data[,9] <- as.numeric(summary_metric_data[,9])
summary_metric_data[,10] <- as.numeric(summary_metric_data[,10])

parallel::stopCluster(cl)

summary_metric_data
write.csv(summary_metric_data, file.path(TESTWRITE_DIR,"summary_metric_data.csv")) # writes new data to the DEMO_DIR


### Step 6: Build a validator to select a model ensemble ####
# Create a validator for selecting the 'best' example models
summary_metric_data <- read.csv(file.path(TESTWRITE_DIR,"summary_metric_data.csv")) # reads existing data from the OUTPUT_DIR
summary_metric_data$X <- NULL
summary_metric_data

sample_data <- read.csv(file.path(DEMO_DIR,"v1/sample_data.csv")) # reads existing data from the OUTPUT_DIR
sample_data$X <- NULL

na_idx <- apply(summary_metric_data, 1, anyNA)

# Create validator data sets for selecting the 'best' example models

simulation_parameters <- sample_data
simulation_summary_metrics <- summary_metric_data[,c(8,10)]
observed_metric_targets = c(population_accuracy = 0
                            ,occupancy_accuracy = 1)

# run and plot ABC outputs
maori.ABC<- abc.stu(target = observed_metric_targets,
                    param = simulation_parameters,
                    sumstat = simulation_summary_metrics,
                    tol = 10/nrow(simulation_parameters),
                    subset = !na_idx,
                    method = 'rejection')

plot.rej(maori.ABC, param = simulation_parameters)

plot.targets(x = maori.ABC,
             model_targets = as.matrix(simulation_summary_metrics),
             obs_targets = as.matrix(observed_metric_targets)) +
  scale_y_continuous(trans = "log1p", breaks = c(0, 100, 500, 2500, 5000, 10000))

saveRDS(maori.ABC, file.path(TESTWRITE_DIR,"abc_obj.RDS")) # writes new data to the DEMO_DIR

My_first_BF <- bayesfactor_parameters(posterior = as.data.frame(maori.ABC$unadj.values), 
                                      prior = sample_data)

write.csv(My_first_BF, file.path(TESTWRITE_DIR,"bayes_factors.csv")) # writes new data to the DEMO_DIR

summary_metric_data <- setDT(summary_metric_data)
summary_metric_data$Weights <- 1/(maori.ABC$dist+.Machine$double.eps)

selected_sims <- summary_metric_data[, `:=`(Region = maori.ABC$region,
                                            Weights = 1/(maori.ABC$dist+.Machine$double.eps))][
                                              Region == TRUE, ]

write.csv(selected_sims, file.path(TESTWRITE_DIR,"selected_sims_index_final.csv")) # writes new data to the DEMO_DIR


### Step 7: Resample models ####

# This sequence resamples the posterior parameter space from the selected samples identified by the ABC and uses them to generate
# 25000 new combinations of the parameters to act as informed priors for the next round of simulations.

options(scipen = 999)

sample_params <- fread(file.path(DEMO_DIR,"v1/sample_data.csv")) # reads existing data from the OUTPUT_DIR
selected_params <- fread(file.path(DEMO_DIR,"v1/selected_sims_index_final.csv")) # reads existing data from the OUTPUT_DIR
abc_pass <- readRDS(file.path(DEMO_DIR,"v1/abc_obj.RDS")) # reads existing data from the OUTPUT_DIR

x <- selected_params[["human_growth_rate"]]
w <- selected_params[["Weights"]]/sum(selected_params[["Weights"]])
min.max <- as.vector(quantile(x, c(0.05,0.95)))
{density_x <- density(x, from = min.max[1], to = min.max[2],
                       bw = "SJ", kernel = "gaussian",
                       n = 1000, adjust = 0.25,
                       weights = w)
  resample_x <- approx(
    x = cumsum(density_x$y)/sum(density_x$y),
    y = density_x$x,
    n = 25000,
    yleft = min(density_x$x), yright = max(density_x$x),
    na.rm = TRUE)$y
}
summary(resample_x)

par(mfrow = c(2, 1))
hist(x[x >= min.max[1] & x <= min.max[2]], breaks = 30)
hist(resample_x, breaks = 30)
par(mfrow = c(1,1))

## Iterate through each demo variable and check distributions
param_dists <- lapply(colnames(selected_params)[3:7], function(param) {
  message(param)
  x <- selected_params[[param]]
  w <- selected_params[["Weights"]]/sum(selected_params[["Weights"]])
  min.max <- as.vector(quantile(x, c(0.05,0.95)))
  
  # https://stackoverflow.com/questions/32871602/r-generate-data-from-a-probability-density-distribution
  {density_x <- density(x, from = min.max[1], to = min.max[2],
                         bw = "SJ", kernel = "gaussian",
                         n = 1000, adjust = 0.25,
                         weights = w)
    resample_x <- approx(
      x = cumsum(density_x$y)/sum(density_x$y),
      y = density_x$x,
      n = 10000,
      yleft = min(density_x$x), yright = max(density_x$x),
      na.rm = TRUE)$y
  }
  par(mfrow = c(2, 1))
  {plot(density(x, from = min(x), to = max(x)), main = param)
    abline(v = min.max, col = "grey70", lty = 2, lwd = 2)}
  disc <- ifelse(param %in% c("human_founding_population", "human_settlement_time", "human_min_neighbourhood"), TRUE, FALSE)
  if (param %in% c("human_founding_population", "human_settlement_time", "human_min_neighbourhood")) {
    resample_x <- round(resample_x)
  }
  # Estimate parameters from interpolated/density estimated obs
  orig_dists <- list(
    normD <- tryCatch(fitdist(resample_x, distr = "norm", method = "mge",
                              gof = "KS", discrete = disc),
                      error = function(err) invisible({err})),
    lnD <- tryCatch(fitdist(resample_x, distr = "lnorm", method = "mge",
                            gof = "KS", discrete = disc),
                    error = function(err) invisible({err})),
    gD <- tryCatch(fitdist(resample_x, distr = "gamma", method = "mge",
                           gof = "KS", discrete = disc),
                   error = function(err) invisible({err})),
    bD <- tryCatch(fitdist(resample_x, distr = "beta", method = "mge",
                           gof = "KS", discrete = disc),
                   error = function(err) invisible({err})),
    triD <- tryCatch(fitdist(resample_x, distr = "triang", method = "mge",
                             gof = "KS", discrete = disc),
                     error = function(err) invisible({err})),
    unifD <- tryCatch(fitdist(resample_x, distr = "unif",method = "mge",
                              gof = "KS", discrete = disc),
                      error = function(err) invisible({err})),
    poisD <- tryCatch(fitdist(resample_x, distr = "pois", method = "mme",
                              discrete = TRUE),
                      error = function(err) invisible({err})),
    negbD <- tryCatch(fitdist(resample_x, distr = "nbinom", method = "mme",
                              discrete = TRUE),
                      error = function(err) invisible({err}))
  )
  # index for dist that errored
  keep_dists <- sapply(orig_dists, function(i) inherits(i, "error"))
  orig_dists <- orig_dists[!keep_dists]
  names(orig_dists) <- c("norm", "lnorm", "gamma", "beta",
                         "triangular",
                         "unif", "pois", "negbin")[!keep_dists]
  cdfcomp(orig_dists, main = param, addlegend = TRUE, ylim = c(0, 1))
  par(mfrow = c(1,1))
  
  return(
    list(
      Output = orig_dists)
      )
})
names(param_dists) <- colnames(selected_params)[3:7]

# For each param, calculate the min AIC, then calc. delta AIC for each distribution
delta_aic <- lapply(names(param_dists), function(param) {
  var_dist <- param_dists[param][param][[1]]$Output
  dist_aic <- sapply(var_dist, function(i) i$aic)
  delta_aic <- dist_aic - min(dist_aic, na.rm = TRUE)
  delta_aic <- sort(delta_aic, na.last = TRUE)
  return(delta_aic)})
names(delta_aic) <- names(param_dists)
delta_aic 

# Make 25,000 resamples based on results. Here each parameter is resampled depending on the truncated distribution functions
# that are identified as being the most appropriate. Hence, a number of the resampling functions can be included or excluded
# depending on the results specific to the resampling run.

{dt_lhs_run2 <- data.table(
    ## founding population = normal
    # human_founding_population = round(
    #           rnormt(n = 25000,
    #           range = quantile(selected_params[["human_founding_population"]], probs = c(0.05,0.95)),
    #           mean = param_dists$human_founding_population$Output$norm$estimate[1],
    #           sd = param_dists$human_founding_population$Output$norm$estimate[2])),
    ## human_founding_population = negbin
    human_founding_population = round(
      rnbt(25000, range = quantile(selected_params[["human_founding_population"]], probs = c(0.05,0.95)),
           size = param_dists$human_founding_population$Output$negbin$estimate[1],
           mu = param_dists$human_founding_population$Output$negbin$estimate[2])),
    #  ## human_growth_rate = normal
    # human_growth_rate = rlnormt(25000,
    #                     range = quantile(selected_params[["human_growth_rate"]], probs = c(0.05,0.95)),
    #                     meanlog = param_dists$human_growth_rate$Output$lnorm$estimate[1],
    #                     sdlog = param_dists$human_growth_rate$Output$lnorm$estimate[2]),
    # human_growth_rate = gamma
    human_growth_rate = rgammat(25000, range = quantile(selected_params[["human_growth_rate"]], probs = c(0.05,0.95)),
                                shape = param_dists$human_growth_rate$Output$gamma$estimate[1],
                                rate = param_dists$human_growth_rate$Output$gamma$estimate[2]),
    ## human_settlement_time = gamma
    human_settlement_time = round(
      rgammat(n = 25000,
              range = quantile(selected_params[["human_settlement_time"]], probs = c(0.05,0.95)),
              shape = param_dists$human_settlement_time$Output$gamma$estimate[1],
              rate = param_dists$human_settlement_time$Output$gamma$estimate[2])),
    ## human_neighbourhood_distance = normal
    human_neighbourhood_distance = rnormt(n = 25000,
                                          range = quantile(selected_params[["human_neighbourhood_distance"]], probs = c(0.05,0.95)),
                                          mean = param_dists$human_neighbourhood_distance$Output$norm$estimate[1],
                                          sd = param_dists$human_neighbourhood_distance$Output$norm$estimate[2]),
    #  ## human_min_neighbourhood = normal
    # human_min_neighbourhood =round(
    #                                rnormt(n = 25000,
    #                                range = quantile(selected_params[["human_min_neighbourhood"]], probs = c(0.05,0.95)),
    #                                mean = param_dists$human_min_neighbourhood$Output$norm$estimate[1],
    #                                sd = param_dists$human_min_neighbourhood$Output$norm$estimate[2]))
    # ## human_min_neighbourhood = lognormal
    # human_min_neighbourhood = round(
    #                             rlnormt(25000,
    #                             range = quantile(selected_params[["human_min_neighbourhood"]], probs = c(0.05,0.95)),
    #                             meanlog = param_dists$human_min_neighbourhood$Output$lnorm$estimate[1],
    #                             sdlog = param_dists$human_min_neighbourhood$Output$lnorm$estimate[2]))
    # ## human_min_neighbourhood = gamma
    # human_min_neighbourhood = round(
    #   rgammat(25000, range = quantile(selected_params[["human_min_neighbourhood"]], probs = c(0.05,0.95)),
    #           shape = param_dists$human_min_neighbourhood$Output$gamma$estimate[1],
    #           rate = param_dists$human_min_neighbourhood$Output$gamma$estimate[2]))
    # 
    ## human_min_neighbourhood = negbin
    human_min_neighbourhood = round(
      rnbt(n = 25000,
           range = quantile(selected_params[["human_min_neighbourhood"]], probs = c(0.05,0.95)),
           size = param_dists$human_min_neighbourhood$Output$negbin$estimate[1],
           mu = param_dists$human_min_neighbourhood$Output$negbin$estimate[2]))
  )}

dt_lhs_run2

summary(dt_lhs_run2)

par(mfrow = n2mfrow(ncol(dt_lhs_run2)))
lapply(colnames(dt_lhs_run2), function(param) {
  orig <- selected_params[[param]]
  sampled <- dt_lhs_run2[[param]]
  dOrig <- density(orig, from = min(orig), to = max(orig))
  dSamp <- density(sampled, from = min(sampled), to = max(sampled))
  plot(dOrig, ylim = c(min(c(dOrig$y, dSamp$y)), max(c(dOrig$y, dSamp$y))),
       main = param,
       col = "black", lty = 1)
  rug(orig, side = 1, col = "black")
  rug(sampled, side = 3, col = "blue")
  lines(dSamp, col = "blue", lty = 2, lwd = 2)
  abline(v = quantile(orig, c(0.05,0.95)), lty = 3, col = "grey70", lwd = 2)
})

reset <- runset + 1 # sets the resampling version
RESAMPLE_DIR <- paste0(DEMO_DIR, "/v", reset) # writes new data to the DEMO_DIR

if (file.exists(file.path(RESAMPLE_DIR))) {
  cat("The folder already exists")
} else {
  dir.create(file.path(RESAMPLE_DIR))
}

fwrite(dt_lhs_run2, file.path(RESAMPLE_DIR,"sample_data.csv")) # writes new data to the DEMO_DIR


### Step 8: Build a simulation manager to run each simulation ####

# Here, the new set of sample parameters are used to run a refined set of simulations. After these simulations are run, the
# process of validation and refinement using Bayes Factors needs to be repeated by returning to line 300. The reiteration
# can be halted when the Bayes Factors are all below 1.

TEST_DIR <- paste0(OUTPUT_DIR, "/v", reset) # reads existing data from the OUTPUT_DIR 
TESTWRITE_DIR <- paste0(DEMO_DIR, "/v",reset) # writes new data to the DEMO_DIR
SIMS_DIR <- paste0(DEMO_DIR, "/v",reset,"/sims") # writes new data to the DEMO_DIR

sample_data <- read.csv(file.path(TEST_DIR, "sample_data.csv")) # reads existing data from the OUTPUT_DIR 
sample_data$X <- NULL

sim_manager <- SimulationManager$new(sample_data = sample_data,
                                     model_template = model_template,
                                     generators = list(human_density_gen),
                                     parallel_cores = PARALLEL_CORES,
                                     results_ext = ".RDS",
                                     results_dir = SIMS_DIR) # writes new data to the DEMO_DIR
run_output <- sim_manager$run()
run_output$summary
run_output$failed_indices


# Step 9: Posterior predictive checks ####

run_1 <- read.csv(file.path(OUTPUT_DIR, "/v1/summary_metric_data.csv")) # reads existing data from the OUTPUT_DIR 
run_1 <- run_1[,c(9,11)]

pp_check(model_metrics = run_1, obs_targets = c(0, 1),
         test = "t.test")

run_2 <- read.csv(file.path(OUTPUT_DIR, "/v2/summary_metric_data.csv")) # reads existing data from the OUTPUT_DIR 
run_2 <- run_2[,c(9,11)]

pp_check(model_metrics = run_2, obs_targets = c(0, 1),
         test = "t.test")

run_3 <- read.csv(file.path(OUTPUT_DIR, "/v3/summary_metric_data.csv")) # reads existing data from the OUTPUT_DIR 
run_3 <- run_3[,c(9,11)]

pp_check(model_metrics = run_3, obs_targets = c(0, 1),
         test = "t.test")

# plotting prior and posterior proability distributions to visualise sample overlap

itn = run_2[sample(nrow(run_2), 20000), ]

obs_targets <- c(0, 1)
scaled.sumstat <- itn
sdScale <- function (x, y) {
  if (sd(y) == 0) {
    return(x)
  } else {
    return(x / sd(y))
  }
}
for (j in 1:2) {
  scaled.sumstat[, j] <- sdScale(x = itn[, j], y = itn[, j])
}
target <- obs_targets
for (j in 1:2) {
  target[j] <- sdScale(obs_targets[j], itn[, j])
}
d_obs <- rep(NA, 20000)
for (j in 1:20000) {
  d_obs[j] <- dist(
    matrix(c(scaled.sumstat[j,], target), ncol = 2, byrow = T))
}
d_rep <- as.vector(dist(scaled.sumstat))
summ <- data.frame(dist = c(d_obs, d_rep), 
                   set = c(rep("d_obs", 20000), rep("d_rep", 199990000)))

ggplot2.density(data=summ, xName='dist',
                     groupName='set', showLegend=FALSE,
                     fillGroupDensity=TRUE, alpha=0.75,
                     xlim=c(0,7.5))+
  scale_fill_manual(values=c("blue4", "darkred")) +
  labs(x = "Euclidean distance", y = "density")+
  theme_bw() +
  theme(
    plot.title = element_text(size=12),
    legend.position = "none")