# This script takes the habitat suitability rasters generated in the previous script and uses them as a template for a
# spatially-explicit population model executed using the 'poems' package. This script goes through all the requirements for
# developing the 'poems' model, running 25,000 simulations of the model, and validating the simulations against empirical
# targets using Approximate Bayesian Computation. It also optimises the simulations by resampling the parameter space.

# All the analyses are currently coded to write output to the DEMO_DIR folder to avoid writing over the existing data.
# All the plots and analyses are conducted using these existing data files.

#### Loading Libraries ####
library(abc)
library(bayesplot)
library(bayestestR)
library(corrplot)
library(data.table)
library(easyGgplot2)
library(fitdistrplus)
library(foreach)
library(ggplot2)
library(logspline)
library(loo)
library(mc2d)
library(parameters)
library(performance)
library(poems)
library(raster)
library(rstanarm)
library(report)
library(see)



#### Establishing source files ####
DATA_DIR <- "./data"
DEMO_DIR <- "./demonstration"
MODS_DIR <- "./models"
OUTPUT_DIR <- "./sensitivity"
PLOT_DIR <- "./figures"

#### GLOBAL SENSITIVITY ANALYSIS ####

for(i in 1:3){
  rep <- paste0("v",i)
  dat <- read.csv(file.path (MODS_DIR, rep, "summary_metric_data.csv"))
  sels <- read.csv(file.path (MODS_DIR, rep, "selected_sims_index_final.csv"))
  dat$sampling_round <- paste0("R",i)
  dat$sel_vec <- as.numeric(dat$sim_ID %in% sels$sim_ID)
  
  if(i==1){
    df <- as.data.table(dat)
  }else{
    df <- as.data.table(rbind(df, dat))
  }
  
}


options(mc.cores = 2L)

abc_obj <- lapply(list(
  file.path (MODS_DIR, "v1/abc_obj.RDS"),
  file.path (MODS_DIR, "v2/abc_obj.RDS"),
  file.path (MODS_DIR, "v3/abc_obj.RDS")),
  readRDS)
abc_obj

# Read in simulation rounds
region <- sapply(abc_obj, function(x) x$region,
                 simplify = TRUE)
region <- as.integer(unlist(region))
sum(region)
table(region)

na_idx <- sapply(abc_obj, function(x) x$na.action,
                 simplify = TRUE)
na_idx <- unlist(na_idx)

distance <- sapply(abc_obj, function(x) x$dist, simplify = TRUE)
distance <- unlist(distance)

cols <- colnames(df)[which(colnames(df) %in% colnames(abc_obj[[1]]$unadj.values))]
df <- copy(df)[, ..cols]
df[, round := c(rep("1", length(abc_obj[[1]]$region)),
                rep("2", length(abc_obj[[2]]$region)),
                rep("3", length(abc_obj[[3]]$region)))]
df[, round := factor(round, levels = c("1", "2", "3"))]
df[, region := region]
df[, na_idx := na_idx]
df[, dist := distance]
df

df[, .(N = .N), by = "round"]

df[, .(min = min(dist),
       max = max(dist)),
   by = c("region", "round")]

zvals <- scale(as.data.frame(df[round == 1,])[colnames(abc_obj[[1]]$unadj.values)],
               center = TRUE, scale = TRUE)
attr(zvals, "scaled:center")
attr(zvals, "scaled:scale")
summary(zvals)

# exclude any "broken" sims
df_nomiss <- copy(df)[na_idx == TRUE, ]
df_nomiss

# Rescale each variable within each round
cols <- colnames(df_nomiss)
cols <- cols[c(1:5,9)]
cols
for(col in cols) {
  set(df_nomiss, j = col, value = as.numeric(df_nomiss[[col]]))
}
df_nomiss[, .(min = min(dist),
              max = max(dist),
              N = .N),
          by = c("region", "round")][order(round, region), ]
df_nomiss[, .(min = min(dist),
              max = max(dist),
              N = .N),
          by = c("region")]

# center and scale all variables relative to first round
## scaling done above
cols <- colnames(zvals)
df_z <- copy(df_nomiss)[, ..cols]
df_z <- as.matrix(df_z)
df_z <- scale(df_z,
              center = attr(zvals, "scaled:center"),
              scale = attr(zvals, "scaled:scale"))
df_z <- as.data.table(df_z)
df_z

# keep sims from round 1
reg_df <- copy(df_z)[df_nomiss[["round"]] == 1, ]
reg_df[, `:=`(round = df_nomiss[round == 1, ][["round"]],
              region = df_nomiss[round == 1, ][["region"]],
              dist = df_nomiss[round == 1, ][["dist"]])]
setcolorder(reg_df, c("round", "region"))
reg_df
reg_df[, .(min = min(dist),
           max = max(dist),
           N = .N),
       by = c("region")]
var(reg_df$dist[reg_df$region == 1])

corrplot(cor(reg_df[round == 1, c(3:7)], method = "spearman"),
         method = "number")

m1 <- rstanarm::stan_lm(dist ~ human_founding_population + human_growth_rate +
                          human_settlement_time + human_neighbourhood_distance +
                          human_min_neighbourhood,
                        data = reg_df,
                        prior = R2(location = 0.5),
                        chains = 3L, iter = 2000, warmup = 1000,
                        cores = 3L,
                        seed = 49435)

summary(m1)
parameters(m1, centrality = "median", ci = 0.89,
           ci_method = "eti", exponentiate = TRUE)

plot_dat <- as.data.frame(cbind(#names(m1$coefficients),
  m1$coefficients,
  m1$ses))
plot_dat <- tibble::rownames_to_column(plot_dat, "VALUE")
colnames(plot_dat) <- c("param", "median", "error")
plot_dat$cols <- plot_dat$median

p1 <- ggplot(plot_dat[c(2:6),], aes(x=param, y=median)) +
  geom_errorbar(width=.1, aes(ymin=median-error, ymax=median+error), size = 1.0) +
  geom_point(aes(fill=cols), shape=21, size=4,stroke = 1.5) +
  scale_fill_gradient2(low = scales::muted("purple"), mid = "white",
                       high = scales::muted("green"), midpoint = 0) +
  coord_flip() +
  theme_bw() +
  geom_hline(yintercept=0, color = "blue", size=1, alpha = 0.5) +
  annotate("rect", xmin = 0, xmax = 5.5, ymin = -0.1, ymax = 0.1,
           alpha = .1,fill = "blue") +
  scale_x_discrete(labels = c('founder size',
                              'population growth rate',
                              'community size',
                              'foraging distance',
                              'settlement year')) +
  theme(legend.position = "none",
        axis.title=element_blank(),
        text = element_text(size = 24))


ggsave(filename = "sensitivity.jpg",
       plot = p1,
       path = file.path(PLOT_DIR),
       height = 7, width = 15, units = "in", dpi = 300)


#### STRUCTURAL SENSITIVITY ANALYSIS ####

# Load boutique ABC functions
source("scripts/custom_funcs.R")


#### 'poems' parameters ####
SAMPLES <- 25000 # the number of samples generated by Latin Hypercube Sampling
PARALLEL_CORES <- 12 # the number of cores defaulted to in parallel simulations
TIMESTEPS <- 1050 # the number of time steps over which the simulations run (in years)


#### Step 1: Establish the study region ####

NZ_raster <- raster(file.path(DATA_DIR, "nz_region_mask.grd"))
raster::plot(NZ_raster , main = "New Zealand raster",
             xlab = "Longitude (degrees)", ylab = "Latitude (degrees)",
             colNA = "blue")


region <- Region$new(template_raster = NZ_raster)
cells <- region$region_cells
raster::plot(region$region_raster, main = "New Zealand study region (cell indices)",
             xlab = "Longitude (degrees)", ylab = "Latitude (degrees)",
             colNA = "blue")


nz_bioregions <- raster::raster(file.path(DATA_DIR, "nz_bioregions.grd"))
raster::values(nz_bioregions) <- raster::values(nz_bioregions)
raster::plot(nz_bioregions, colNA = "blue")

reg_data <- read.csv(file.path(DATA_DIR, "nz_region_data.csv"))
reg_data

bioregional_indices <- lapply(as.list(reg_data$index),
                              function(i) {
                                which(nz_bioregions[region$region_indices] == i)
                              })

# Read mean and sd human habitat suitability
mean_raster <- raster(file.path(DATA_DIR, "maori_mean.grd"))*NZ_raster
sd_raster <- raster(file.path(DATA_DIR, "maori_sd.grd"))*NZ_raster
raster::plot(mean_raster, colNA = "blue")
raster::plot(sd_raster, colNA = "blue")

# Establish environmental correlation
env_corr <- SpatialCorrelation$new(region = region, amplitude = 0.496, breadth = 80, distance_scale = 1000) # km
env_corr$calculate_compact_decomposition(decimals = 4)
lapply(env_corr$get_compact_decomposition(), dim)

# Calculate distances between cells for possible neighbourhoods
dispersal_gen <- DispersalGenerator$new(region = region,
                                        distance_scale = 1000, # 1 km
                                        dispersal_max_distance = 70)
dispersal_gen$calculate_distance_data() # note: distance class = distance when < 1000 classes
distance_data <- dispersal_gen$distance_data[[1]][, c(1:2, 4)]
names(distance_data)[3] <- "distance"
head(distance_data)


#### Step 2: Build the simulation model for the study region ####

# Simulation model
model_template <- SimulationModel$new(region = region,
                                      time_steps = TIMESTEPS,
                                      model_attributes = c("human_density"))
model_template$simulation_function <- function(model) {
  return(model$get_attributes("human_density"))
}

# Build generators for dynamically generating model parameters

human_density_gen <- Generator$new(description = "human density",
                                   region = region,
                                   spatial_correlation = env_corr,
                                   time_steps = TIMESTEPS,
                                   generate_rasters = FALSE,
                                   simulation_start_year = 850, # AD
                                   hs_mean = mean_raster[region$region_indices],
                                   hs_sd = sd_raster[region$region_indices],
                                   distance_data = dispersal_gen$distance_data[[1]][, c(1:2, 4)],
                                   inputs = c("human_founding_population",    # 1. Founding population size (200 - 600)
                                              "human_growth_rate",            # 2. Intrinsic growth rate (~R = 1.005 - 1.022)
                                              "human_settlement_time",        # 3. Settlement time (1260 - 1320)
                                              "human_neighbourhood_distance", # 4. Neighbourhood/foraging range distance (30-70) km [rounded]
                                              "human_min_neighbourhood"),     # 5. Minimum neighbourhood (settlement) size
                                   outputs = "human_density",
                                   generative_requirements = list(human_density = "function"))

human_density_gen$add_distribution_template("sample_hs",
                                            distr_type = "beta",
                                            distr_params = list(mean = "hs_mean", sd = "hs_sd"))

human_density_gen$add_function_template(
  "human_density",
  function_def = function (params) {
    
    # Unpack parameters
    region <- params$region
    cells <- region$region_cells
    founding_population <- params$human_founding_population
    growth_rate <- params$human_growth_rate
    settlement_time <- params$human_settlement_time
    neighbourhood_distance <- params$human_neighbourhood_distance
    min_neighbourhood <- params$human_min_neighbourhood
    sample_hs <- params$sample_hs[,1]
    distance_data <- params$distance_data
    time_steps <- params$time_steps
    simulation_start_year <- params$simulation_start_year
    
    # Calculate human occupation sequence based on best available cells (deterministic) with the
    # sampled HS. Occupied cells must be within a neighbourhood having a total population of
    # at least the minimum (settlement) size.
    
    # Define neighbourhoods (reachable booleans) from distance data
    neighbourhood_matrix <- diag(nrow = cells, ncol = cells) > 0
    colnames(neighbourhood_matrix) <- rownames(neighbourhood_matrix) <- 1:cells
    in_range_indices <- distance_data[which(distance_data$distance <= neighbourhood_distance), 1:2]
    neighbourhood_matrix[as.matrix(in_range_indices)] <- TRUE
    # neighbourhood_matrix[1:30, 1:10]
    # summary(colSums(neighbourhood_matrix)) # neighbourhood sizes
    
    # Neighbourhood and occupancy sequence in order of greatest habitat suitability
    sequence_indices <- order(colSums(sample_hs*neighbourhood_matrix), decreasing = TRUE)
    neighbourhood_sequence <- neighbourhood_matrix[, sequence_indices]
    # neighbourhood_sequence[1:30, 1:10]
    occupancy_sequence <- apply(matrix(1:cells), 1, function(i) {
      as.logical(rowSums(as.matrix(neighbourhood_sequence[,1:i])))
    })
    dimnames(occupancy_sequence) <- dimnames(neighbourhood_sequence)
    # occupancy_sequence[1:30, 1:10]
    
    # Create a sequence of normalised HS for the unfolding occupancy
    sequence_matrix <- occupancy_sequence*sample_hs
    sequence_matrix <- sequence_matrix/array(rep(colSums(sequence_matrix), each = cells),
                                             dim(sequence_matrix))
    # sequence_matrix[1:30, 1:8]
    # unname(colSums(sequence_matrix))
    sequence_fraction <- colSums(sequence_matrix*neighbourhood_sequence)
    
    # Calculate human population via growth curve
    human_population <- array(0, time_steps)
    human_population[settlement_time - simulation_start_year + 1] <- founding_population
    for (i in (settlement_time - simulation_start_year + 2):time_steps) {
      human_population[i] <- round(human_population[i - 1]*growth_rate)
    }
    
    # Unfold the sequence using the minimum neighbourhood (settlement) size
    settlement_sequence <- array(NA, time_steps)
    for (tm in 1:time_steps) {
      threshold_indices <- which(human_population[tm]*sequence_fraction < min_neighbourhood)
      if (length(threshold_indices)) {
        settlement_sequence[tm] <- max(threshold_indices[1] - 1, 1)
      } else {
        settlement_sequence[tm] <- cells
      }
    }
    # settlement_sequence
    
    # Calculate human population across the settlement sequence
    return(sequence_matrix[, settlement_sequence]*
             array(rep(human_population, each = cells), c(cells, time_steps)))
  },
  call_params = c("region",
                  "human_founding_population",    # 1. Founding population size (200 - 600)
                  "human_growth_rate",            # 2. Intrinsic growth rate (~R = 1.005 - 1.022)
                  "human_settlement_time",        # 3. Settlement time (1260 - 1320)
                  "human_neighbourhood_distance", # 4 Neighbourhood/foraging range distance (30-70) km [rounded]
                  "human_min_neighbourhood",      # 5. Minimum neighbourhood (settlement) size
                  "sample_hs",
                  "distance_data",
                  "time_steps",
                  "simulation_start_year"))

# OPTIONAL STEP: test human generator

# this step isn't essential to running the simulations later, but it tests the simulation framework established above
# using values that are median to the sampling to come next. Essentially, it's a way to see if the simulation has
# glitches or errors coded into it

test_gen <- human_density_gen$generate(input_values = list(
  human_founding_population = 400,   # 1. Founding population size (200 - 600)
  human_growth_rate = 1.013,         # 2. Intrinsic growth rate (~R = 1.005 - 1.022)
  human_settlement_time = 1314,      # 3. Settlement time (1260 - 1320)
  human_neighbourhood_distance = 50, # 4. Neighbourhood/foraging range distance (30-70) km [rounded]
  human_min_neighbourhood = 25       # 5. Minimum neighbourhood (settlement) size
))
raster::plot(region$raster_from_values(test_gen$human_density[,55]), colNA = "blue")
raster::plot(region$raster_from_values(test_gen$human_density[,465]), colNA = "blue")
raster::plot(region$raster_from_values(test_gen$human_density[,800]), colNA = "blue")

rm(test_gen) #removes the test generation to avoid any conflation in subsequent simulations


### Step 3: Refit with logistic model ####

# Redefine human density generation
human_density_gen <- Generator$new(description = "human density",
                                   region = region,
                                   spatial_correlation = env_corr,
                                   time_steps = TIMESTEPS,
                                   generate_rasters = FALSE,
                                   simulation_start_year = 850, # AD
                                   hs_mean = mean_raster[region$region_indices],
                                   hs_sd = sd_raster[region$region_indices],
                                   distance_data = dispersal_gen$distance_data[[1]][, c(1:2, 4)],
                                   inputs = c("human_founding_population",    # 1. Founding population size (200 - 600)
                                              "human_growth_rate",            # 2. Intrinsic growth rate (~R = 1.005 - 1.022)
                                              "human_carrying_capacity",      # 3. maximum population size (4700 - 285000)
                                              "human_settlement_time",        # 5. Settlement time (1260 - 1320)
                                              "human_neighbourhood_distance", # 6. Neighbourhood/foraging range distance (30-70) km [rounded]
                                              "human_min_neighbourhood"),     # 7. Minimum neighbourhood (settlement) size
                                   outputs = "human_density",
                                   generative_requirements = list(human_density = "function"))

human_density_gen$add_distribution_template("sample_hs",
                                            distr_type = "beta",
                                            distr_params = list(mean = "hs_mean", sd = "hs_sd"))
# raster::plot(region$raster_from_values(human_density_gen$new_clone()$get_attribute("sample_hs")), colNA = "blue")

human_density_gen$add_function_template(
  "human_density",
  function_def = function (params) {
    
    # Unpack parameters
    region <- params$region
    cells <- region$region_cells
    founding_population <- params$human_founding_population
    #growth_rate <- params$human_growth_rate
    growth_rate <- params$human_growth_rate-1 #conversion to logistic growth rate
    carrying_capacity <- params$human_carrying_capacity
    settlement_time <- params$human_settlement_time
    neighbourhood_distance <- params$human_neighbourhood_distance
    min_neighbourhood <- params$human_min_neighbourhood
    sample_hs <- params$sample_hs[,1]
    distance_data <- params$distance_data
    time_steps <- params$time_steps
    simulation_start_year <- params$simulation_start_year
    
    # Calculate human occupation sequence based on best available cells (deterministic) with the
    # sampled HS. Occupied cells must be within a neighbourhood having a total population of
    # at least the minimum (settlement) size.
    
    # Define neighbourhoods (reachable booleans) from distance data
    neighbourhood_matrix <- diag(nrow = cells, ncol = cells) > 0
    colnames(neighbourhood_matrix) <- rownames(neighbourhood_matrix) <- 1:cells
    in_range_indices <- distance_data[which(distance_data$distance <= neighbourhood_distance), 1:2]
    neighbourhood_matrix[as.matrix(in_range_indices)] <- TRUE
    # neighbourhood_matrix[1:30, 1:10]
    # summary(colSums(neighbourhood_matrix)) # neighbourhood sizes
    
    # Neighbourhood and occupancy sequence in order of greatest habitat suitability
    sequence_indices <- order(colSums(sample_hs*neighbourhood_matrix), decreasing = TRUE)
    neighbourhood_sequence <- neighbourhood_matrix[, sequence_indices]
    # neighbourhood_sequence[1:30, 1:10]
    occupancy_sequence <- apply(matrix(1:cells), 1, function(i) {
      as.logical(rowSums(as.matrix(neighbourhood_sequence[,1:i])))
    })
    dimnames(occupancy_sequence) <- dimnames(neighbourhood_sequence)
    # occupancy_sequence[1:30, 1:10]
    
    # Create a sequence of normalised HS for the unfolding occupancy
    sequence_matrix <- occupancy_sequence*sample_hs
    sequence_matrix <- sequence_matrix/array(rep(colSums(sequence_matrix), each = cells),
                                             dim(sequence_matrix))
    # sequence_matrix[1:30, 1:8]
    # unname(colSums(sequence_matrix))
    sequence_fraction <- colSums(sequence_matrix*neighbourhood_sequence)
    
    # Calculate human population via growth curve
    human_population <- array(0, time_steps)
    human_population[settlement_time - simulation_start_year + 1] <- founding_population
    
    #for (i in (settlement_time - simulation_start_year + 2):time_steps) {
    for (i in (settlement_time - simulation_start_year):time_steps) {
      year <- i - (settlement_time - simulation_start_year)
      human_population[i] <- round(carrying_capacity /
                                     (1+((carrying_capacity-founding_population)/founding_population)*
                                        exp((-1*growth_rate)*year)))
    }
    
    # Unfold the sequence using the minimum neighbourhood (settlement) size
    settlement_sequence <- array(NA, time_steps)
    for (tm in 1:time_steps) {
      threshold_indices <- which(human_population[tm]*sequence_fraction < min_neighbourhood)
      if (length(threshold_indices)) {
        settlement_sequence[tm] <- max(threshold_indices[1] - 1, 1)
      } else {
        settlement_sequence[tm] <- cells
      }
    }
    # settlement_sequence
    
    # Calculate human population across the settlement sequence
    return(sequence_matrix[, settlement_sequence]*
             array(rep(human_population, each = cells), c(cells, time_steps)))
  },
  call_params = c("region",
                  "human_founding_population",    # 1. Founding population size (200 - 600)
                  "human_growth_rate",            # 2. Intrinsic growth rate (~R = 1.005 - 1.022)
                  "human_carrying_capacity",      # 3. maximum population size (47000 - 285000)
                  "human_settlement_time",        # 5. Settlement time (1260 - 1320)
                  "human_neighbourhood_distance", # 6. Neighbourhood/foraging range distance (30-70) km [rounded]
                  "human_min_neighbourhood",     # 7. Minimum neighbourhood (settlement) size (20-100)
                  "sample_hs",
                  "distance_data",
                  "time_steps",
                  "simulation_start_year"))


# OPTIONAL STEP: test human generator

# this step isn't essential to running the simulations later, but it tests the simulation framework established above
# using values that are median to the sampling to come next. Essentially, it's a way to see if the simulation has
# glitches or errors coded into it

test_gen <- human_density_gen$generate(input_values = list(
  human_founding_population = 517,       # 1. Founding population size (200 - 600)
  human_growth_rate = 1.013391555,              # 2. Intrinsic growth rate (~R = 1.005 - 1.022)
  human_carrying_capacity = 135000,      # 3. maximum population size (47000 - 285000)
  human_settlement_time = 1244,          # 5. Settlement time (1260 - 1320)
  human_neighbourhood_distance = 66,     # 6. Neighbourhood/foraging range distance (30-70) km [rounded]
  human_min_neighbourhood = 28          # 7. Minimum neighbourhood (settlement) size (20-100)
))
raster::plot(region$raster_from_values(test_gen$human_density[,395]), colNA = "blue")
raster::plot(region$raster_from_values(test_gen$human_density[,465]), colNA = "blue")
raster::plot(region$raster_from_values(test_gen$human_density[,200]), colNA = "blue")

popn_size <- colSums(test_gen$human_density)
plot(popn_size)
popn_size[919]
popn_size[395]


rm(test_gen) #removes the test generation to avoid any conflation in subsequent simulations


# Build a simulation manager to run each simulation

# the simulations here are built using the optimised values from the final run of the original simulations.
# These samples are modified with additional uniformly sampled parameters suitable for each sensitivity test

GROWTHMOD_DIR <- paste0(DEMO_DIR, "/logistic") # writes new data to the DEMO_DIR
SIMS_DIR <- paste0(GROWTHMOD_DIR, "/sims") # writes new data to the DEMO_DIR

if (file.exists(GROWTHMOD_DIR)) {
  cat("The folder already exists")
} else {
  dir.create(GROWTHMOD_DIR)
}

if (file.exists(SIMS_DIR)) {
  cat("The folder already exists")
} else {
  dir.create(SIMS_DIR)
}

sample_data <- read.csv(file.path(MODS_DIR, "sample_data.csv"))  # reads existing data from the OUTPUT_DIR 
sample_data$X <- NULL
sample_data$human_carrying_capacity <- round(runif(25000,47000,285000),0) # uniform sample of human carrying capacity

write.csv(sample_data, file.path(GROWTHMOD_DIR, "sample_data.csv")) # writes new data to the DEMO_DIR

sim_manager <- SimulationManager$new(sample_data = sample_data,
                                     model_template = model_template,
                                     generators = list(human_density_gen),
                                     parallel_cores = PARALLEL_CORES,
                                     results_ext = ".RDS",
                                     results_dir = SIMS_DIR) # writes new data to the DEMO_DIR
run_output <- sim_manager$run()
run_output$summary
run_output$failed_indices


### Step 4: Build a results manager to generate summary results (metrics) ####

# Set targets for our summary metrics (used to calculate combined metric errors)

settlements <- read.csv(file.path(DATA_DIR, "Maori_14C_targets.csv"))
settlements <- subset(settlements, settlements$Filter_2 == "KEEP")

settlements$region_index <- apply(as.matrix(settlements[, c("Longitude", "Latitude")]), 1,
                                  function (coord) which.min(geosphere::distGeo(region$coordinates, coord)))
earliest_indices <- apply(matrix(1:region$region_cells), 1,
                          function(i) {
                            Cal_Age_i <- (settlements$Cal_Age*(settlements$region_index == i)*
                                            (settlements$Reliability == "reliable")*
                                            (settlements$Cal_Age <= 1000))
                            which.max(Cal_Age_i)*(any(Cal_Age_i > 0) | NA)
                          })
earliest_settlements <- settlements[earliest_indices[!is.na(earliest_indices)],]
settlements_sites <- region$region_raster*0
settlements_sites[region$region_indices[earliest_settlements$region_index]] <-
  earliest_settlements$Cal_Age - earliest_settlements$Cal_Err
raster::plot(settlements_sites, colNA = "blue")

youngest_sett_array <- array(settlements_sites[region$region_indices])
youngest_sett_array[youngest_sett_array == 0] <-NA

settlements_sites[region$region_indices[earliest_settlements$region_index]] <-
  earliest_settlements$Cal_Age + earliest_settlements$Cal_Err
raster::plot(settlements_sites, colNA = "blue")

oldest_sett_array <- array(settlements_sites[region$region_indices])
oldest_sett_array[oldest_sett_array == 0] <-NA

targets <- list(
  south_settlement = array(c(1086,712)),
  oldest_settlement = oldest_sett_array, # data frame with locations and upper CI YBP
  youngest_settlement = youngest_sett_array, # data frame with locations and upper CI YBP
  
  total_abundance = c(100000, 150000) # est. CI 100-150% of Cook's 1769 population estimates (from Holdaway et al 2014)
)

#### TEST the summary functions via:

sample_data <- read.csv(file.path(GROWTHMOD_DIR, "sample_data.csv")) # reads new data from the DEMO_DIR
sample_data$X <- NULL

results_test <- NULL
sample_matrix_list <-NULL
human_density_list <- vector("list", SAMPLES)
occupancy_date_list <- vector("list", SAMPLES)
regional_settlement_list <- vector("list", SAMPLES)
final_abundance_list <- vector("list", SAMPLES)


# PARALLEL TESTING TARGETS
cl <- parallel::makeCluster(PARALLEL_CORES)
doParallel::registerDoParallel(cl)
partest <- foreach(s = 1:SAMPLES, .packages = "poems") %dopar% {
  
  sample = paste0("sample_",s,"_results.RDS")
  results = PopulationResults$new(results = readRDS(file.path(SIMS_DIR, sample)),
                                  nz_bioregions = nz_bioregions, # attachments
                                  bioregional_indices = bioregional_indices,
                                  targets = targets)
  
  # total population size
  final_abundance <- sum(results$get_attribute("human_density")[,920]) #Maori population in layer 920 (1769) = 150 000
  
  ## TESTING ACCURACY
  population_accuracy <- if (final_abundance>targets[[4]][1] & final_abundance<targets[[4]][2]) {
    population_accuracy <- 0} else {
      if (final_abundance<targets[[4]][1]){
        population_accuracy <- (targets[[4]][1]-final_abundance)/targets[[4]][1]
      }else{
        population_accuracy <- (final_abundance-targets[[4]][2])/targets[[4]][2]
      }}
  
  # SouthIs_settlement
  north<-unlist(results$get_attribute("bioregional_indices")[c(1:10)])
  south<-unlist(results$get_attribute("bioregional_indices")[c(11:18)])
  
  island_indices <- list(north,south)
  
  regional_abundance <- t(array(unlist(lapply(island_indices,
                                              function (indices) {
                                                colSums(results$get_attribute("human_density")[indices,])
                                              })), c(TIMESTEPS, 2)))#time_steps by number of regions
  settlement_date <- list(c(1:2))
  for(j in 1:2) {
    sett_yr1 <- if(sum(regional_abundance[j,]) == 0){sett_yr = 0}else{min(which(regional_abundance[j,] > 0))}
    settlement_date[[1]][j] <- 1950-(human_density_gen$template_attached$simulation_start_year + sett_yr1)
  }
  
  ## TESTING ACCURACY
  if(settlement_date[[1]][2]<targets[[1]][1] & settlement_date[[1]][2]>targets[[1]][2]){
    South_settlement_timing <- 0}else {
      if (settlement_date[[1]][2]>targets[[1]][1]){
        South_settlement_timing <- (settlement_date[[1]][2]-targets[[1]][1])
      }else{
        South_settlement_timing <- (targets[[1]][2]-settlement_date[[1]][2])
      }}
  
  
  #location_settlement
  occupancy <- results$get_attribute("human_density")
  occupancy_date <- list(c(1:532))
  
  for(j in 1:532) {
    sett_yr2 <- if(sum(occupancy[j,]) == 0){sett_yr = 0}else{min(which(occupancy[j,] > 0))}
    occupancy_date[[1]][j] <- 1950-(human_density_gen$template_attached$simulation_start_year + sett_yr2)
  }
  
  ## TESTING ACCURACY
  # for hitting before oldest evidence
  occupancy_accuracy <- NULL
  for(i in 1:532){
    occupancy_accuracy[i] <- occupancy_date[[1]][i]-targets[[2]][i]
  }
  occupancy_accuracy <-  length(which(occupancy_accuracy>0))/length(which(targets[[2]]>0))
  
  ## build summary results
  sim_ID <- sub(".RDS.*", "", sample)
  human_founding_population <- sample_data[s,1]
  human_growth_rate <- sample_data[s,2]
  human_settlement_time <- sample_data[s,3]
  human_neighbourhood_distance <- sample_data[s,4]
  human_min_neighbourhood <- sample_data[s,5]
  final_abundance
  summary_results<-NULL
  summary_results <- cbind(sim_ID,
                           human_founding_population,
                           human_growth_rate,
                           human_settlement_time,
                           human_neighbourhood_distance,
                           human_min_neighbourhood,
                           final_abundance,
                           population_accuracy,
                           South_settlement_timing,
                           occupancy_accuracy)
}

#results_test
summary_metric_data <- as.data.frame(do.call(rbind,partest))

# assemble results metrics into a data frame and write to file
#summary_metric_data <- as.data.frame(results_test)
summary_metric_data[,2] <- as.numeric(summary_metric_data[,2])
summary_metric_data[,3] <- as.numeric(summary_metric_data[,3])
summary_metric_data[,4] <- as.numeric(summary_metric_data[,4])
summary_metric_data[,5] <- as.numeric(summary_metric_data[,5])
summary_metric_data[,6] <- as.numeric(summary_metric_data[,6])
summary_metric_data[,7] <- as.numeric(summary_metric_data[,7])
summary_metric_data[,8] <- as.numeric(summary_metric_data[,8])
summary_metric_data[,9] <- as.numeric(summary_metric_data[,9])
summary_metric_data[,10] <- as.numeric(summary_metric_data[,10])

parallel::stopCluster(cl)

summary_metric_data
write.csv(summary_metric_data, file.path(GROWTHMOD_DIR,"summary_metric_data.csv")) # writes new data to the DEMO_DIR

# Create a validator for selecting the 'best' example models
summary_metric_data <- read.csv(file.path(GROWTHMOD_DIR,"summary_metric_data.csv")) # reads new data from the DEMO_DIR
summary_metric_data$X <- NULL

sample_data <- read.csv(file.path(GROWTHMOD_DIR,"sample_data.csv")) # reads new data from the DEMO_DIR
sample_data$X <- NULL

na_idx <- apply(summary_metric_data, 1, anyNA)


# Create validator data sets for selecting the 'best' example models

simulation_parameters <- sample_data
simulation_summary_metrics <- summary_metric_data[,c(8,10)]
observed_metric_targets = c(population_accuracy = 0
                            ,occupancy_accuracy = 1)

# run and plot ABC outputs
maori.ABC<- abc.stu(target = observed_metric_targets,
                    param = simulation_parameters,
                    sumstat = simulation_summary_metrics,
                    tol = 0.01,
                    subset = !na_idx,
                    method = 'rejection')

plot.rej(maori.ABC, param = simulation_parameters)

plot.targets(x = maori.ABC,
             model_targets = as.matrix(simulation_summary_metrics),
             obs_targets = as.matrix(observed_metric_targets)) +
  scale_y_continuous(trans = "log1p", breaks = c(0, 100, 500, 2500, 5000, 10000))

saveRDS(maori.ABC, file.path(GROWTHMOD_DIR,"abc_obj.RDS")) # writes new data to the DEMO_DIR

summary_metric_data <- setDT(summary_metric_data)
summary_metric_data$Weights <- 1/(maori.ABC$dist+.Machine$double.eps)

selected_sims <- summary_metric_data[, `:=`(Region = maori.ABC$region,
                                            Weights = 1/(maori.ABC$dist+.Machine$double.eps))][
                                              Region == TRUE, ]

write.csv(selected_sims, file.path(GROWTHMOD_DIR,"selected_sims_index_final.csv")) # writes new data to the DEMO_DIR


### Step 5: Refit with split founders ####

# Redefine generation
human_density_gen <- Generator$new(description = "human density",
                                   region = region,
                                   spatial_correlation = env_corr,
                                   time_steps = TIMESTEPS,
                                   generate_rasters = FALSE,
                                   simulation_start_year = 850, # AD
                                   hs_mean = mean_raster[region$region_indices],
                                   hs_sd = sd_raster[region$region_indices],
                                   distance_data = dispersal_gen$distance_data[[1]][, c(1:2, 4)],
                                   inputs = c("human_founding_population",    # 1. Founding population size (200 - 600)
                                              "human_growth_rate",            # 2. Intrinsic growth rate (~R = 1.005 - 1.022)
                                              "human_settlement_time",        # 3. Settlement time (1260 - 1320)
                                              "human_neighbourhood_distance", # 4. Neighbourhood/foraging range distance (30-70) km [rounded]
                                              "human_min_neighbourhood",
                                              "human_founding_split"),     # 5. Minimum neighbourhood (settlement) size
                                   outputs = "human_density",
                                   generative_requirements = list(human_density = "function"))

human_density_gen$add_distribution_template("sample_hs",
                                            distr_type = "beta",
                                            distr_params = list(mean = "hs_mean", sd = "hs_sd"))
# raster::plot(region$raster_from_values(human_density_gen$new_clone()$get_attribute("sample_hs")), colNA = "blue")

human_density_gen$add_function_template(
  "human_density",
  function_def = function (params) {
    
    # Unpack parameters
    region <- params$region
    cells <- region$region_cells
    founding_population <- params$human_founding_population
    founding_split <- params$human_founding_split
    growth_rate <- params$human_growth_rate
    settlement_time <- params$human_settlement_time
    neighbourhood_distance <- params$human_neighbourhood_distance
    min_neighbourhood <- params$human_min_neighbourhood
    sample_hs <- params$sample_hs[,1]
    distance_data <- params$distance_data
    time_steps <- params$time_steps
    simulation_start_year <- params$simulation_start_year
    
    # Calculate human occupation sequence based on best available cells (deterministic) with the
    # sampled HS. Occupied cells must be within a neighbourhood having a total population of
    # at least the minimum (settlement) size.
    
    # Define neighbourhoods (reachable booleans) from distance data
    neighbourhood_matrix <- diag(nrow = cells, ncol = cells) > 0
    colnames(neighbourhood_matrix) <- rownames(neighbourhood_matrix) <- 1:cells
    in_range_indices <- distance_data[which(distance_data$distance <= neighbourhood_distance), 1:2]
    neighbourhood_matrix[as.matrix(in_range_indices)] <- TRUE
    # neighbourhood_matrix[1:30, 1:10]
    # summary(colSums(neighbourhood_matrix)) # neighbourhood sizes
    
    # Neighbourhood and occupancy sequence in order of greatest habitat suitability
    sequence_indices <- order(colSums(sample_hs*neighbourhood_matrix), decreasing = TRUE)
    neighbourhood_sequence <- neighbourhood_matrix[, sequence_indices]
    # neighbourhood_sequence[1:30, 1:10]
    occupancy_sequence <- apply(matrix(1:cells), 1, function(i) {
      as.logical(rowSums(as.matrix(neighbourhood_sequence[,1:i])))
    })
    dimnames(occupancy_sequence) <- dimnames(neighbourhood_sequence)
    # occupancy_sequence[1:30, 1:10]
    
    # Create a sequence of normalised HS for the unfolding occupancy
    sequence_matrix <- occupancy_sequence*sample_hs
    sequence_matrix <- sequence_matrix/array(rep(colSums(sequence_matrix), each = cells),
                                             dim(sequence_matrix))
    # sequence_matrix[1:30, 1:8]
    # unname(colSums(sequence_matrix))
    sequence_fraction <- colSums(sequence_matrix*neighbourhood_sequence)
    
    # Calculate human population via growth curve
    human_population <- array(0, time_steps)
    human_population[settlement_time - simulation_start_year + 1] <- (founding_population/founding_split)
    
    for (i in (settlement_time - simulation_start_year + 2):time_steps) {
      human_population[i] <- round(human_population[i - 1]*growth_rate)
      if (i < founding_split){
        human_population[i] <- human_population[i] + (founding_population/founding_split)
      }else{
        human_population[i] <- human_population[i]
      }
    }
    
    # Unfold the sequence using the minimum neighbourhood (settlement) size
    settlement_sequence <- array(NA, time_steps)
    for (tm in 1:time_steps) {
      threshold_indices <- which(human_population[tm]*sequence_fraction < min_neighbourhood)
      if (length(threshold_indices)) {
        settlement_sequence[tm] <- max(threshold_indices[1] - 1, 1)
      } else {
        settlement_sequence[tm] <- cells
      }
    }
    # settlement_sequence
    
    # Calculate human population across the settlement sequence
    return(sequence_matrix[, settlement_sequence]*
             array(rep(human_population, each = cells), c(cells, time_steps)))
  },
  call_params = c("region",
                  "human_founding_population",    # 1. Founding population size (200 - 600)
                  "human_growth_rate",            # 2. Intrinsic growth rate (~R = 1.005 - 1.022)
                  "human_settlement_time",        # 3. Settlement time (1260 - 1320)
                  "human_neighbourhood_distance", # 4 Neighbourhood/foraging range distance (30-70) km [rounded]
                  "human_min_neighbourhood",      # 5. Minimum neighbourhood (settlement) size
                  "human_founding_split",
                  "sample_hs",
                  "distance_data",
                  "time_steps",
                  "simulation_start_year"))

# Build a simulation manager to run each simulation
FOUNDERMOD_DIR <- paste0(DEMO_DIR, "/split_founders") # writes new data to the DEMO_DIR
SIMS_DIR <- paste0(FOUNDERMOD_DIR, "/sims") # writes new data to the DEMO_DIR


if (file.exists(FOUNDERMOD_DIR)) {
  cat("The folder already exists")
} else {
  dir.create(FOUNDERMOD_DIR)
}

if (file.exists(SIMS_DIR)) {
  cat("The folder already exists")
} else {
  dir.create(SIMS_DIR)
}

sample_data <- read.csv(file.path(MODS_DIR, "sample_data.csv")) # reads existing data from the DEMO_DIR
sample_data$X <- NULL

splitter <- rep(c(1,2,5,10),each=nrow(sample_data))

sample_data <- as.data.frame(sapply(sample_data, rep.int, times=4))
nrow(sample_data)
sample_data$human_founding_split <- splitter

write.csv(sample_data, file.path(FOUNDERMOD_DIR, "sample_data.csv")) # writes new data to the DEMO_DIR

sim_manager <- SimulationManager$new(sample_data = sample_data,
                                     model_template = model_template,
                                     generators = list(human_density_gen),
                                     parallel_cores = PARALLEL_CORES,
                                     results_ext = ".RDS",
                                     results_dir = SIMS_DIR) # writes new data to the DEMO_DIR
run_output <- sim_manager$run()
run_output$summary
run_output$failed_indices


### Step 7: Build a results manager to generate summary results (metrics) ####

# Set targets for our summary metrics (used to calculate combined metric errors)

settlements <- read.csv(file.path(DATA_DIR, "Maori_14C_targets.csv"))
settlements <- subset(settlements, settlements$Filter_2 == "KEEP")

settlements$region_index <- apply(as.matrix(settlements[, c("Longitude", "Latitude")]), 1,
                                  function (coord) which.min(geosphere::distGeo(region$coordinates, coord)))
earliest_indices <- apply(matrix(1:region$region_cells), 1,
                          function(i) {
                            Cal_Age_i <- (settlements$Cal_Age*(settlements$region_index == i)*
                                            (settlements$Reliability == "reliable")*
                                            (settlements$Cal_Age <= 1000))
                            which.max(Cal_Age_i)*(any(Cal_Age_i > 0) | NA)
                          })
earliest_settlements <- settlements[earliest_indices[!is.na(earliest_indices)],]
settlements_sites <- region$region_raster*0
settlements_sites[region$region_indices[earliest_settlements$region_index]] <-
  earliest_settlements$Cal_Age - earliest_settlements$Cal_Err
raster::plot(settlements_sites, colNA = "blue")

youngest_sett_array <- array(settlements_sites[region$region_indices])
youngest_sett_array[youngest_sett_array == 0] <-NA

settlements_sites[region$region_indices[earliest_settlements$region_index]] <-
  earliest_settlements$Cal_Age + earliest_settlements$Cal_Err
raster::plot(settlements_sites, colNA = "blue")

oldest_sett_array <- array(settlements_sites[region$region_indices])
oldest_sett_array[oldest_sett_array == 0] <-NA

targets <- list(
  south_settlement = array(c(1086,712)),
  oldest_settlement = oldest_sett_array, # data frame with locations and upper CI YBP
  youngest_settlement = youngest_sett_array, # data frame with locations and upper CI YBP
  
  total_abundance = c(100000, 150000) # est. CI 100-150% of Cook's 1769 population estimates (from Holdaway et al 2014)
)

#### TEST the summary functions via:

sample_data <- read.csv(file.path(FOUNDERMOD_DIR, "sample_data.csv"))
sample_data$X <- NULL

results_test <- NULL
sample_matrix_list <-NULL
human_density_list <- vector("list", SAMPLES)
occupancy_date_list <- vector("list", SAMPLES)
regional_settlement_list <- vector("list", SAMPLES)
final_abundance_list <- vector("list", SAMPLES)


# PARALLEL TESTING TARGETS
cl <- parallel::makeCluster(PARALLEL_CORES)
doParallel::registerDoParallel(cl)
partest <- foreach(s = 1:SAMPLES, .packages = "poems") %dopar% {
  
  sample = paste0("sample_",s,"_results.RDS")
  results = PopulationResults$new(results = readRDS(file.path(SIMS_DIR, sample)),
                                  nz_bioregions = nz_bioregions, # attachments
                                  bioregional_indices = bioregional_indices,
                                  targets = targets)
  
  # total population size
  final_abundance <- sum(results$get_attribute("human_density")[,920]) #Maori population in layer 920 (1769) = 150 000
  
  ## TESTING ACCURACY
  population_accuracy <- if (final_abundance>targets[[4]][1] & final_abundance<targets[[4]][2]) {
    population_accuracy <- 0} else {
      if (final_abundance<targets[[4]][1]){
        population_accuracy <- (targets[[4]][1]-final_abundance)/targets[[4]][1]
      }else{
        population_accuracy <- (final_abundance-targets[[4]][2])/targets[[4]][2]
      }}
  
  # SouthIs_settlement
  north<-unlist(results$get_attribute("bioregional_indices")[c(1:10)])
  south<-unlist(results$get_attribute("bioregional_indices")[c(11:18)])
  
  island_indices <- list(north,south)
  
  regional_abundance <- t(array(unlist(lapply(island_indices,
                                              function (indices) {
                                                colSums(results$get_attribute("human_density")[indices,])
                                              })), c(TIMESTEPS, 2)))#time_steps by number of regions
  settlement_date <- list(c(1:2))
  for(j in 1:2) {
    sett_yr1 <- if(sum(regional_abundance[j,]) == 0){sett_yr = 0}else{min(which(regional_abundance[j,] > 0))}
    settlement_date[[1]][j] <- 1950-(human_density_gen$template_attached$simulation_start_year + sett_yr1)
  }
  
  ## TESTING ACCURACY
  if(settlement_date[[1]][2]<targets[[1]][1] & settlement_date[[1]][2]>targets[[1]][2]){
    South_settlement_timing <- 0}else {
      if (settlement_date[[1]][2]>targets[[1]][1]){
        South_settlement_timing <- (settlement_date[[1]][2]-targets[[1]][1])
      }else{
        South_settlement_timing <- (targets[[1]][2]-settlement_date[[1]][2])
      }}
  
  
  #location_settlement
  occupancy <- results$get_attribute("human_density")
  occupancy_date <- list(c(1:532))
  
  for(j in 1:532) {
    sett_yr2 <- if(sum(occupancy[j,]) == 0){sett_yr = 0}else{min(which(occupancy[j,] > 0))}
    occupancy_date[[1]][j] <- 1950-(human_density_gen$template_attached$simulation_start_year + sett_yr2)
  }
  
  ## TESTING ACCURACY
  # for hitting before oldest evidence
  occupancy_accuracy <- NULL
  for(i in 1:532){
    occupancy_accuracy[i] <- occupancy_date[[1]][i]-targets[[2]][i]
  }
  occupancy_accuracy <-  length(which(occupancy_accuracy>0))/length(which(targets[[2]]>0))
  
  ## build summary results
  sim_ID <- sub(".RDS.*", "", sample)
  human_founding_population <- sample_data[s,1]
  human_growth_rate <- sample_data[s,2]
  human_settlement_time <- sample_data[s,3]
  human_neighbourhood_distance <- sample_data[s,4]
  human_min_neighbourhood <- sample_data[s,5]
  final_abundance
  summary_results<-NULL
  summary_results <- cbind(sim_ID,
                           human_founding_population,
                           human_growth_rate,
                           human_settlement_time,
                           human_neighbourhood_distance,
                           human_min_neighbourhood,
                           final_abundance,
                           population_accuracy,
                           South_settlement_timing,
                           occupancy_accuracy)
}

#results_test
summary_metric_data <- as.data.frame(do.call(rbind,partest))

# assemble results metrics into a data frame and write to file
#summary_metric_data <- as.data.frame(results_test)
summary_metric_data[,2] <- as.numeric(summary_metric_data[,2])
summary_metric_data[,3] <- as.numeric(summary_metric_data[,3])
summary_metric_data[,4] <- as.numeric(summary_metric_data[,4])
summary_metric_data[,5] <- as.numeric(summary_metric_data[,5])
summary_metric_data[,6] <- as.numeric(summary_metric_data[,6])
summary_metric_data[,7] <- as.numeric(summary_metric_data[,7])
summary_metric_data[,8] <- as.numeric(summary_metric_data[,8])
summary_metric_data[,9] <- as.numeric(summary_metric_data[,9])
summary_metric_data[,10] <- as.numeric(summary_metric_data[,10])

parallel::stopCluster(cl)

summary_metric_data
summary_metric_data$human_founding_split <- splitter
write.csv(summary_metric_data, file.path(FOUNDERMOD_DIR,"summary_metric_data.csv")) # writes new data to the DEMO_DIR

# Create a validator for selecting the 'best' example models
summary_metric_data <- read.csv(file.path(FOUNDERMOD_DIR,"summary_metric_data.csv")) # reads new data from the DEMO_DIR
summary_metric_data$X <- NULL

sample_data <- read.csv(file.path(FOUNDERMOD_DIR,"sample_data.csv")) # reads new data from the DEMO_DIR
sample_data$X <- NULL

na_idx <- apply(summary_metric_data, 1, anyNA)

# Load boutique ABC functions

# Function for plotting ABC results with the rejection method posteriors
plot.rej <- function(x, param, dens_kernel = "epanechnikov", ...) {
  if (!inherits(x, "abc"))
    stop("Use only with objects of class \"abc\".", call. = FALSE)
  abc.out <- x
  mymethod <- abc.out$method
  if (mymethod != "rejection") {
    stop("plot.rej can only be used when method is 'rejection'",
         call. = FALSE)
  }
  if (!is.matrix(param) && !is.data.frame(param) && !is.vector(param)) {
    stop("'param' has to be a matrix, data.frame or vector.",
         call. = F)
  }
  if (is.null(dim(param))) {
    param <- matrix(param, ncol = 1)
  }
  if (is.data.frame(param)) {
    param <- as.matrix(param)
  }
  np <- abc.out$numparam
  numsim <- length(param)/np
  parnames <- abc.out$names$parameter.names
  cond <- isTRUE(c(match(colnames(param), parnames),
                   match(parnames, colnames(param))))
  if (cond) {
    stop("'abc.out' and 'param' are not compatible; paramater names are different.",
         call. = FALSE)
  }
  # selected with rejection or not?
  rej <- abc.out$unadj.values
  # make sure matrices are in same order
  if (is.vector(param)) {
    np.orig <- 1
    nsim <- length(param)
  } else if (is.matrix(param)) {
    np.orig <- dim(param)[2]
    nsim <- dim(param)[1]
    myorder <- match(parnames, colnames(param))
    if (isTRUE(myorder - 1:np)) {
      param <- param[, myorder]
      warning("'param' is being re-ordered according to 'abc.out'...",
              call. = FALL, immediate. = TRUE)
    }
  }
  if (np.orig != np) {
    stop("The number parameters supplied in \"param\" is different from that in \"x\".",
         call. = FALSE)
  }
  par(mfrow = n2mfrow(nr.plots = np))
  for (i in 1:np) {
    prior.d <- density(param[, i],
                       from = min(param[,i]),
                       to = max(param[, i]),
                       kernel = dens_kernel)
    rej.d <- density(rej[, i],
                     from = min(rej[, i], na.rm = T),
                     to = max(rej[, i], na.rm = TRUE),
                     kernel = dens_kernel)
    rej.dl <- density(rej[, i],
                      from = quantile(rej[, i], 0.05, na.rm = TRUE),
                      to = quantile(rej[, i], 0.95, na.rm = TRUE),
                      kernel = dens_kernel)
    myxlim <- range(c(prior.d$x, rej.d$x))
    myylim <- range(c(prior.d$y, rej.d$y))
    myxlab <- parnames[i]
    plot(prior.d, main = myxlab, col = "#1b9e77", lty = 2,
         lwd = 1, xlab = "", ylim = myylim, xlim = myxlim)
    title(sub = paste0(paste("N =", prior.d$n, "  Bandwidth =", formatC(prior.d$bw)), "; ",
                       sprintf("mean = %0.3f; sd = %0.3f",
                               mean(param[, i],na.rm = T), sd(param[, i],na.rm = T)), "\n",
                       sprintf("meanSel = %0.3f; sdSel = %0.3f",
                               mean(rej[, i],na.rm = T), sd(rej[, i],na.rm = T))))
    lines(rej.d, col = "#d95f02", lty = 1, lwd = 1)
    rug(rej[, i], side = 1, ticksize = 0.02, lwd = 1, col = "#d95f02")
    abline(v = range(rej.dl$x), col = "grey90", lty = 1, lwd = 1)
    if (i == np) {
      legend("bottomright",
             c("Prior", "Rej."),
             lty=c(1,1),
             col = c("#1b9e77", "#d95f02"),
             inset = c(0,1), xpd = TRUE, horiz = TRUE,
             bty="n"
      )
    }
  }
  par(mfcol = c(1, 1))
}

# plot regression adjusted methods with rejection sims
plot.adj <- function(x, param) {
  if (!inherits(x, "abc"))
    stop("Use only with objects of class \"abc\".", call. = FALSE)
  abc.out <- x
  mymethod <- abc.out$method
  if (mymethod == "rejection")
    stop("plot.rej can only be used when method is \"loclinear\", \"neuralnet\" or \"ridge\".",
         call. = FALSE)
  if (!is.matrix(param) && !is.data.frame(param) && !is.vector(param))
    stop("'param' has to be a matrix, data.frame or vector.",
         call. = F)
  if (is.null(dim(param)))
    param <- matrix(param, ncol = 1)
  if (is.data.frame(param))
    param <- as.matrix(param)
  np <- abc.out$numparam
  numsim <- length(param)/np
  parnames <- abc.out$names$parameter.names
  cond <- isTRUE(c(match(colnames(param), parnames), match(parnames,
                                                           colnames(param))))
  if (cond)
    stop("'abc.out' and 'param' are not compatible; paramater names are different.",
         call. = FALSE)
  rej <- abc.out$unadj.values
  post <- abc.out$adj.values
  if (np == 1)
    rej <- matrix(rej, ncol = 1)
  if (is.vector(param)) {
    np.orig <- 1
    nsim <- length(param)
  } else if (is.matrix(param)) {
    np.orig <- dim(param)[2]
    nsim <- dim(param)[1]
    myorder <- match(parnames, colnames(param))
    if (isTRUE(myorder - 1:np)) {
      param <- param[, myorder]
      warning("'param' is being re-ordered according to 'abc.out'...",
              call. = FALL, immediate. = TRUE)
    }
  }
  if (np.orig != np) {
    stop("The number parameters supplied in \"param\" is different from that in \"x\".",
         call. = FALSE)
  }
  par(mfrow = n2mfrow(nr.plots = np))
  for (i in 1:np) {
    prior.d <- density(param[, i], from = min(param[,i]),
                       to = max(param[, i]))
    rej.d <- density(rej[, i], from = min(rej[, i], na.rm = T),
                     to = max(rej[, i], na.rm = TRUE))
    post.d <- density(post[, i], from = min(post[, i], na.rm = T),
                      to = max(post[, i], na.rm = TRUE))
    myxlim <- range(c(prior.d$x, rej.d$x, post.d$x))
    myylim <- range(c(prior.d$y, rej.d$y, post.d$y))
    myxlab <- parnames[i]
    plot(prior.d, main = myxlab, col = "#1b9e77", lty = 2,
         lwd = 1, xlab = "", ylim = myylim, xlim = myxlim)
    title(sub = paste0(paste("N =", prior.d$n, "  Bandwidth =", formatC(prior.d$bw)), "; ",
                       sprintf("mean = %0.3f; sd = %0.3f",
                               mean(param[, i],na.rm = T), sd(param[, i],na.rm = T)), "\n",
                       sprintf("meanRej = %0.3f; sdRej = %0.3f",
                               mean(rej[, i],na.rm = T), sd(rej[, i],na.rm = T)), "\n",
                       sprintf("meanAdj = %0.3f; sdAdj = %0.3f",
                               mean(post[, i],na.rm = T), sd(post[, i],na.rm = T))))
    lines(rej.d, col = "#d95f02", lty = 1, lwd = 1)
    lines(post.d, col = "#7570b3", lty = 1, lwd = 1)
    rug(rej[, i], side = 3, ticksize = 0.02, lwd = 1,
        col = "#d95f02")
    rug(post[, i], side = 1, ticksize = 0.02, lwd = 1,
        col = "#7570b3")
    if (i == np) {
      legend("bottomright",
             c("Prior", "Rej.", "Adj."),
             lty=c(1,1,1),
             col = c("#1b9e77", "#d95f02", "#7570b3"),
             inset = c(0,1), xpd = TRUE, horiz = TRUE,
             bty="n"
      )
    }
  }
  par(mfcol = c(1, 1))
}

# Default ABC function with some extra warnings/context for errors
abc.stu <- function (target, param, sumstat, tol, method, hcorr = TRUE, scaling = "sd",
                     transf = "none", logit.bounds = c(0, 0), subset = NULL, kernel = "epanechnikov",
                     numnet = 10, sizenet = 5, lambda = c(1e-04, 0.001, 0.01),
                     trace = FALSE, maxit = 500, ...) {
  call <- match.call()
  if (missing(target))
    stop("'target' is missing")
  if (missing(param))
    stop("'param' is missing")
  if (missing(sumstat))
    stop("'sumstat' is missing")
  if (!is.matrix(param) && !is.data.frame(param) && !is.vector(param))
    stop("'param' has to be a matrix, data.frame or vector.")
  if (!is.matrix(sumstat) && !is.data.frame(sumstat) && !is.vector(sumstat))
    stop("'sumstat' has to be a matrix, data.frame or vector.")
  if (missing(tol))
    stop("'tol' is missing")
  if (missing(method))
    stop("'method' is missing with no default")
  if (!any(method == c("rejection", "loclinear", "neuralnet",
                       "ridge"))) {
    stop("Method must be rejection, loclinear, or neuralnet or ridge")
  }
  if (!any(scaling == c("sd", "mad"))) {
    stop("Scaling must be sd or mad")
  }
  if (method == "rejection"){
    rejmethod <- TRUE
  }else {rejmethod <- FALSE}
  if (!any(kernel == c("gaussian", "epanechnikov", "rectangular",
                       "triangular", "biweight", "cosine"))) {
    kernel <- "epanechnikov"
    warning("Kernel is incorrectly defined. Setting to default (Epanechnikov)")
  }
  if (is.data.frame(param))
    param <- as.matrix(param)
  if (is.data.frame(sumstat))
    sumstat <- as.matrix(sumstat)
  if (is.list(target))
    target <- unlist(target)
  if (is.vector(sumstat))
    sumstat <- matrix(sumstat, ncol = 1)
  if (length(target) != dim(sumstat)[2])
    stop("Number of summary statistics in 'target' has to be the same as in 'sumstat'.")
  nss <- length(sumstat[1, ])
  cond1 <- !any(as.logical(apply(sumstat, 2, function(x) length(unique(x)) - 1)))
  if (cond1)
    stop("Zero variance in the summary statistics.")
  ltransf <- length(transf)
  if (is.vector(param)) {
    numparam <- 1
    param <- matrix(param, ncol = 1)
  } else {
    numparam <- dim(param)[2]}
  for (i in 1:ltransf) {
    if (sum(transf[i] == c("none", "log", "logit")) == 0) {
      stop("Transformations must be none, log, or logit.")
    }
    if (transf[i] == "logit") {
      if (logit.bounds[i, 1] >= logit.bounds[i, 2]) {
        stop("Logit bounds are incorrect.")
      }
    }
  }
  if (rejmethod) {
    if (!all(transf == "none")) {
      warning("No transformation is applied when the simple rejection is used.",
              call. = F)
    }
    transf[1:numparam] <- "none"
  } else {
    if (numparam != ltransf) {
      if (length(transf) == 1) {
        transf <- rep(transf[1], numparam)
        warning("All parameters are \"", transf[1], "\" transformed.",
                sep = "", call. = F)
      }
      else stop("Number of parameters is not the same as number of transformations.",
                sep = "", call. = F)
    }
  }
  gwt <- rep(TRUE, length(sumstat[, 1]))
  gwt[attributes(na.omit(sumstat))$na.action] <- FALSE
  if (missing(subset)){
    subset <- rep(TRUE, length(sumstat[, 1]))}
  gwt <- as.logical(gwt * subset)
  if (!length(colnames(param))) {
    warning("No parameter names are given, using P1, P2, ...")
    paramnames <- paste("P", 1:numparam, sep = "")
  } else {
    paramnames <- colnames(param)}
  if (!length(colnames(sumstat))) {
    warning("No summary statistics names are given, using S1, S2, ...")
    statnames <- paste("S", 1:nss, sep = "")
  } else {
    statnames <- colnames(sumstat)
  }
  scaled.sumstat <- sumstat
  for (j in 1:nss) {
    if (scaling == "mad") {
      scaled.sumstat[, j] <- abc:::normalise(x = sumstat[, j],
                                             y = sumstat[,j][gwt])
    } else {
      sdScale <- function (x, y) {
        if (sd(y) == 0) {
          return(x)
        } else {
          return(x/sd(y))
        }}
      scaled.sumstat[, j] <- sdScale(x = sumstat[, j], y = sumstat[,j][gwt])
    }
  }
  for (j in 1:nss) {
    if (scaling == "mad") {
      target[j] <- abc:::normalise(target[j], sumstat[, j][gwt])
    } else {
      target[j] <- sdScale(target[j], sumstat[, j][gwt])
    }
  }
  sum1 <- 0
  for (j in 1:nss) {
    sum1 <- sum1 + (scaled.sumstat[, j] - target[j])^2
  }
  dist <- sqrt(sum1)
  dist[!gwt] <- floor(max(dist[gwt]) + 10)
  nacc <- ceiling(length(dist) * tol)
  ds <- sort(dist)[nacc]
  wt1 <- (dist <= ds)
  aux <- cumsum(wt1)
  wt1 <- wt1 & (aux <= nacc)
  if (kernel == "gaussian") {
    wt1 <- rep(TRUE, length(dist))
  }
  for (i in 1:numparam) {
    if (transf[i] == "log") {
      if (min(param[, i]) <= 0) {
        cat("log transform: values out of bounds - correcting...")
        x.tmp <- ifelse(param[, i] <= 0, max(param[, i]), param[, i])
        x.tmp.min <- min(x.tmp)
        param[, i] <- ifelse(param[, i] <= 0, x.tmp.min,
                             param[, i])
      }
      param[, i] <- log(param[, i])
    } else if (transf[i] == "logit") {
      if (min(param[, i]) <= logit.bounds[i, 1]) {
        x.tmp <- ifelse(param[, i] <= logit.bounds[i, 1], max(param[, i]), param[, i])
        x.tmp.min <- min(x.tmp)
        param[, i] <- ifelse(param[, i] <= logit.bounds[i,1], x.tmp.min, param[, i])
      }
      if (max(param[, i]) >= logit.bounds[i, 2]) {
        x.tmp <- ifelse(param[, i] >= logit.bounds[i,
                                                   2], min(param[, i]), param[, i])
        x.tmp.max <- max(x.tmp)
        param[, i] <- ifelse(param[, i] >= logit.bounds[i,
                                                        2], x.tmp.max, param[, i])
      }
      param[, i] <- (param[, i] - logit.bounds[i, 1])/(logit.bounds[i,
                                                                    2] - logit.bounds[i, 1])
      param[, i] <- log(param[, i]/(1 - param[, i]))
    }
  }
  ss <- sumstat[wt1, ]
  unadj.values <- param[wt1, ]
  statvar <- as.logical(apply(cbind(sumstat[wt1, ]), 2, function(x) length(unique(x)) - 1))
  cond2 <- any(!statvar)
  if (cond2 && !rejmethod)
    stop(paste0("Zero variance in the summary statistics in the selected region. Try: checking summary statistics, choosing larger tolerance, or rejection method.\n",
                paste(colnames(ss)[!statvar], collapse = ", ")))
  if (rejmethod) {
    if (cond2)
      warning(paste0("Zero variance in the summary statistics in the selected region. Try: checking summary statistics, choosing larger tolerance.\n",
                     paste(colnames(ss)[!statvar], collapse = ", ")))
    weights <- rep(1, length = sum(wt1))
    adj.values <- NULL
    residuals <- NULL
    lambda <- NULL
  } else {
    if (cond2)
      cat("Warning messages:\nStatistic(s)", statnames[!statvar],
          "has/have zero variance in the selected region.\nConsider using larger tolerance or the rejection method or discard this/these statistics, which might solve the collinearity problem in 'lsfit'.\n",
          sep = ", ")
    if (kernel == "epanechnikov")
      weights <- 1 - (dist[wt1]/ds)^2
    if (kernel == "rectangular")
      weights <- dist[wt1]/ds
    if (kernel == "gaussian")
      weights <- 1/sqrt(2 * pi) * exp(-0.5 * (dist/(ds/2))^2)
    if (kernel == "triangular")
      weights <- 1 - abs(dist[wt1]/ds)
    if (kernel == "biweight")
      weights <- (1 - (dist[wt1]/ds)^2)^2
    if (kernel == "cosine")
      weights <- cos(pi/2 * dist[wt1]/ds)
    if (method == "loclinear") {
      fit1 <- lsfit(scaled.sumstat[wt1, ], param[wt1, ],
                    wt = weights)
      pred <- t(structure(cbind(fit1$coefficients)[fit1$qr$pivot,
      ], names = names(fit1$coefficients))) %*% c(1,
                                                  target)
      pred <- matrix(pred, ncol = numparam, nrow = sum(wt1),
                     byrow = TRUE)
      residuals <- param[wt1, ] - t(t(structure(cbind(fit1$coefficients)[fit1$qr$pivot,
      ], names = names(fit1$coefficients))) %*% t(cbind(1,
                                                        scaled.sumstat[wt1, ])))
      residuals <- cbind(residuals)
      the_m <- apply(residuals, FUN = mean, 2)
      residuals <- sapply(1:numparam, FUN = function(x) {
        residuals[, x] - the_m[x]
      })
      pred <- sapply(1:numparam, FUN = function(x) {
        pred[, x] + the_m[x]
      })
      sigma2 <- apply(as.matrix(residuals), FUN = function(x) {
        sum((x)^2 * weights)/sum(weights)
      }, MARGIN = 2)
      aic <- sum(wt1) * sum(log(sigma2)) + 2 * (nss + 1) *
        numparam
      bic <- sum(wt1) * sum(log(sigma2)) + log(sum(wt1)) *
        (nss + 1) * numparam
      if (hcorr == TRUE) {
        fit2 <- lsfit(scaled.sumstat[wt1, ], log(residuals^2),
                      wt = weights)
        auxaux <- t(structure(cbind(fit2$coefficients)[fit2$qr$pivot,
        ], names = names(fit2$coefficients))) %*% c(1,
                                                    target)
        pred.sd <- sqrt(exp(auxaux))
        pred.sd <- matrix(pred.sd, nrow = sum(wt1), ncol = numparam,
                          byrow = T)
        pred.si <- t(t(structure(cbind(fit2$coefficients)[fit2$qr$pivot,
        ], names = names(fit2$coefficients))) %*% t(cbind(1,
                                                          scaled.sumstat[wt1, ])))
        pred.si <- sqrt(exp(pred.si))
        adj.values <- pred + (pred.sd * residuals)/pred.si
        residuals <- (pred.sd * residuals)/pred.si
      }
      else {
        adj.values <- pred + residuals
      }
      colnames(adj.values) <- colnames(unadj.values)
      lambda <- NULL
    }
    if (method == "neuralnet") {
      linout <- TRUE
      param.mad <- c()
      for (i in 1:numparam) {
        param.mad[i] <- stats::mad(param[, i][gwt])
        param[, i] <- abc:::normalise(param[, i], param[, i][gwt])
      }
      lambda <- sample(lambda, numnet, replace = T)
      fv <- array(dim = c(sum(wt1), numparam, numnet))
      pred <- matrix(nrow = numparam, ncol = numnet)
      for (i in 1:numnet) {
        fit1 <- nnet(scaled.sumstat[wt1, ], param[wt1,
        ], weights = weights, decay = lambda[i], size = sizenet,
        trace = trace, linout = linout, maxit = maxit,
        ...)
        cat(i)
        fv[, , i] <- fit1$fitted.values
        pred[, i] <- predict(fit1, data.frame(rbind(target)))
      }
      cat("\n")
      pred.med <- apply(pred, 1, median)
      pred.med <- matrix(pred.med, nrow = sum(wt1), ncol = numparam,
                         byrow = T)
      fitted.values <- apply(fv, c(1, 2), median)
      residuals <- param[wt1, ] - fitted.values
      if (hcorr == TRUE) {
        pred2 <- matrix(nrow = numparam, ncol = numnet)
        fv2 <- array(dim = c(sum(wt1), numparam, numnet))
        for (i in 1:numnet) {
          fit2 <- nnet(scaled.sumstat[wt1, ], log(residuals^2),
                       weights = weights, decay = lambda[i], size = sizenet,
                       trace = trace, linout = linout, ...)
          cat(i)
          fv2[, , i] <- fit2$fitted.values
          pred2[, i] <- predict(fit2, data.frame(rbind(target)))
        }
        cat("\n")
        pred.sd <- sqrt(exp(apply(pred2, 1, median)))
        pred.sd <- matrix(pred.sd, nrow = sum(wt1), ncol = numparam,
                          byrow = T)
        fv.sd <- sqrt(exp(apply(fv2, c(1, 2), median)))
        adj.values <- pred.med + (pred.sd * residuals)/fv.sd
        residuals <- (pred.sd * residuals)/fv.sd
      }
      else {
        adj.values <- pred.med + residuals
      }
      colnames(adj.values) <- colnames(unadj.values)
      for (i in 1:numparam) {
        adj.values[, i] <- adj.values[, i] * param.mad[i]
      }
    }
    if (method == "ridge") {
      param.mad <- c()
      for (i in 1:numparam) {
        param.mad[i] <- stats::mad(param[, i][gwt])
        param[, i] <- abc:::normalise(param[, i], param[, i][gwt])
      }
      numnet <- length(lambda)
      fv <- array(dim = c(sum(wt1), numparam, numnet))
      pred <- matrix(nrow = numparam, ncol = numnet)
      mataux <- sqrt(diag(weights))
      paramaux <- as.matrix(mataux %*% param[wt1, ])
      scaledaux <- mataux %*% scaled.sumstat[wt1, ]
      for (parcur in (1:numparam)) {
        #message(parcur)
        fit1 <- MASS::lm.ridge(paramaux[, parcur] ~ scaledaux,
                               lambda = lambda)
        for (i in 1:numnet) {
          fv[, parcur, i] <- drop(cbind(1, scaled.sumstat[wt1,
          ]) %*% (rbind(coef(fit1))[i, ]))
          pred[parcur, i] <- drop(c(1, target) %*% (rbind(coef(fit1))[i,
          ]))
        }
      }
      pred.med <- apply(pred, 1, median)
      pred.med <- matrix(pred.med, nrow = sum(wt1), ncol = numparam,
                         byrow = T)
      fitted.values <- apply(fv, c(1, 2), median)
      residuals <- param[wt1, ] - fitted.values
      if (hcorr == TRUE) {
        pred2 <- matrix(nrow = numparam, ncol = numnet)
        fv2 <- array(dim = c(sum(wt1), numparam, numnet))
        for (parcur in (1:numparam)) {
          lresidaux <- (mataux %*% (log(residuals[, parcur]^2)))
          fit2 <- lm.ridge(lresidaux ~ scaledaux, lambda = lambda)
          for (i in 1:numnet) {
            fv2[, parcur, i] <- drop(cbind(1, scaled.sumstat[wt1,
            ]) %*% (rbind(coef(fit2))[i, ]))
            pred2[parcur, i] <- drop(c(1, target) %*%
                                       (rbind(coef(fit2))[i, ]))
          }
        }
        cat("\n")
        pred.sd <- sqrt(exp(apply(pred2, 1, median)))
        pred.sd <- matrix(pred.sd, nrow = sum(wt1), ncol = numparam,
                          byrow = T)
        fv.sd <- sqrt(exp(apply(fv2, c(1, 2), median)))
        adj.values <- pred.med + (pred.sd * residuals)/fv.sd
        residuals <- (pred.sd * residuals)/fv.sd
      }
      else {
        adj.values <- pred.med + residuals
      }
      colnames(adj.values) <- colnames(unadj.values)
      for (i in 1:numparam) {
        adj.values[, i] <- adj.values[, i] * param.mad[i]
      }
    }
  }
  if (numparam == 1) {
    unadj.values <- matrix(unadj.values, ncol = 1)
    if (method != "rejection") {
      adj.values <- matrix(adj.values, ncol = 1)
      residuals <- matrix(residuals, ncol = 1)
    }
  }
  for (i in 1:numparam) {
    if (transf[i] == "log") {
      unadj.values[, i] <- exp(unadj.values[, i])
      adj.values[, i] <- exp(adj.values[, i])
    }
    else if (transf[i] == "logit") {
      unadj.values[, i] <- exp(unadj.values[, i])/(1 +
                                                     exp(unadj.values[, i]))
      unadj.values[, i] <- unadj.values[, i] * (logit.bounds[i,
                                                             2] - logit.bounds[i, 1]) + logit.bounds[i, 1]
      adj.values[, i] <- exp(adj.values[, i])/(1 + exp(adj.values[,
                                                                  i]))
      adj.values[, i] <- adj.values[, i] * (logit.bounds[i,
                                                         2] - logit.bounds[i, 1]) + logit.bounds[i, 1]
    }
  }
  abc:::abc.return(transf, logit.bounds, method, call, numparam,
                   nss, paramnames, statnames, unadj.values, adj.values,
                   ss, weights, residuals, dist, wt1, gwt, lambda, hcorr,
                   aic, bic)
}

# plot accepted sims against targets
plot.targets <- function(x, model_targets, obs_targets, ...) {
  require(ggplot2); require(Hmisc); require(data.table)
  if (!inherits(x, "abc")){
    stop("Use only with objects of class \"abc\".", call. = FALSE)}
  if (!is.matrix(obs_targets) && !is.data.frame(obs_targets) && !is.vector(obs_targets)){
    stop("'obs_targets' has to be a matrix, data.frame or vector.",
         call. = F)}
  if (is.null(dim(obs_targets))){
    obs_targets <- matrix(obs_targets, ncol = 1, dimnames = list(colnames(x$ss), "Target"))}
  if (is.data.frame(obs_targets)){
    obs_targets<- as.matrix(obs_targets)}
  nt <- x$numstat
  numsim <- nrow(model_targets)
  tarnames <- x$names$statistics.names
  cond <- isTRUE(c(match(colnames(obs_targets), tarnames),
                   match(tarnames, colnames(model_targets))))
  if (cond) {
    stop("'obs_targets' and 'model_targets' are not compatible; paramater names are different.",
         call. = FALSE)}
  sel_sims <- x$ss
  df <- rbindlist(list(
    as.data.table(sel_sims)[, Scen := "Selected"],
    as.data.table(model_targets)[, Scen := "AllSims"][!na_idx, ]))
  suppressWarnings({mnDF <- melt(
    df[Scen == "Selected",
       lapply(.SD, weighted.mean, w = 1/x$dist[x$region]),
       .SDcols = tarnames])})
  mnDF[, Scen := "Selected"]
  suppressWarnings({allDF <- melt(
    df[Scen == "AllSims",
       lapply(.SD, weighted.mean, w = 1/x$dist[!na_idx]),
       .SDcols = tarnames])})
  allDF[, Scen := "AllSims"]
  mnDF <- rbindlist(
    list(mnDF,allDF))
  df <- melt(df, id.vars = "Scen")
  p1 <- ggplot(data = df, aes(x = value, fill = Scen, group = Scen)) +
    facet_wrap(~variable, scales = "free") +
    geom_histogram(col = "grey70", bins = 50) +
    geom_vline(data = mnDF,
               aes(xintercept = value, colour = Scen)) +
    scale_y_continuous(expand = c(0.01, 0.01), ...) +
    geom_vline(data = as.data.table(obs_targets)[,variable := unique(unique(df$variable))],
               aes(xintercept = V1)) +
    theme_minimal() +
    theme(legend.position = "bottom",
          legend.direction = "horizontal",
          legend.title = element_blank()) +
    labs(x = NULL, y = NULL)
  return(print(p1))
}

# Create validator data sets for selecting the 'best' example models

# This process needs to be repeated for each of the four replicates of founding events. This can be done by chaning
# the value of 'splitnum' and repeating the validation process.

splitnum <- 1 # the number of founding events (1, 2, 5, or 10) will be used to select the best simulations for each scenario

simulation_parameters <- subset(sample_data, sample_data$human_founding_split == splitnum)
simulation_summary_metrics <- subset(summary_metric_data, summary_metric_data$human_founding_split == splitnum)
simulation_summary_metrics <- simulation_summary_metrics[,c(8,10)]

observed_metric_targets = c(population_accuracy = 0
                            ,occupancy_accuracy = 1)

# run and plot ABC outputs
maori.ABC<- abc.stu(target = observed_metric_targets,
                    param = simulation_parameters,
                    sumstat = simulation_summary_metrics,
                    tol = 0.01,
                    subset = !na_idx,
                    method = 'rejection')

plot.rej(maori.ABC, param = simulation_parameters)

plot.targets(x = maori.ABC,
             model_targets = as.matrix(simulation_summary_metrics),
             obs_targets = as.matrix(observed_metric_targets)) +
  scale_y_continuous(trans = "log1p", breaks = c(0, 100, 500, 2500, 5000, 10000))

saveRDS(maori.ABC, file.path(FOUNDERMOD_DIR, paste0("abc_obj_", splitnum ,".RDS"))) # writes new data to the DEMO_DIR

summary_metric_data <- setDT(summary_metric_data)
summary_metric_data$Weights <- 1/(maori.ABC$dist+.Machine$double.eps)

selected_sims <- summary_metric_data[, `:=`(Region = maori.ABC$region,
                                            Weights = 1/(maori.ABC$dist+.Machine$double.eps))][
                                              Region == TRUE, ]

write.csv(selected_sims, file.path(FOUNDERMOD_DIR,paste0("selected_sims_index_final", splitnum, ".csv"))) # writes new data to the DEMO_DIR
