# This script takes all the models and projections generated through the colonisation models and generates a series of figures.
# Some of these figures are diagnostic, some of the figures are demonstrative. There is also a section that produces all the
# figures required to assemble and animation of the ensemble settlement sequence, but the animation is assembled outside R

# All the analyses are currently coded to write output to the DEMO_DIR folder to avoid writing over the existing data.
# All the plots and analyses are conducted using these existing data files, and are written to the PLOT_DIR.

#### Loading Libraries ####
library(data.table)
library(easyGgplot2)
library(foreach)
library(ggplot2)
library(ggridges)
library(patchwork)
library(poems)
library(raster)
library(rgdal)
library(sp)
library(tidyverse)


#### Establishing source files ####
DATA_DIR <- "./data"
DEMO_DIR <- "./demonstration"
MOD_DIR <- "./models"
SENS_DIR <- "./sensitivity"
PLOT_DIR <- "./figures"


### Step 1: Plotting SADIE grids ####
# import New Zealand coastline shapefile
landsea <- raster(file.path(DATA_DIR, "NZ_landsea.grd"))

NZ<-readOGR(file.path(DATA_DIR, "NZ_coast.shp"))
NZ <- spTransform(NZ, raster::crs(landsea))
NZ <- crop(NZ, raster::extent(landsea))

fifty <- stack(file.path(MOD_DIR, "/sadie/SADIE_50km_stack.grd"))
twentyfive <- stack(file.path(MOD_DIR, "/sadie/SADIE_25km_stack.grd"))
ten <- stack(file.path(MOD_DIR, "/sadie/SADIE_10km_stack.grd"))

names(fifty) <- c("archsites", "datedsites", "delta")
plot_df <- as.data.frame(fifty, xy=TRUE, long=TRUE)
plot_df <- plot_df[complete.cases(plot_df), ]
layer.labs <- c(archsites = "Archaeological sites",
                datedsites = "Radiocarbon dates",
                delta = "Δ SADIE distance")

p1.1 <- ggplot(data = subset(plot_df, plot_df$layer == "archsites"),
             aes(x = x, y = y, fill = value)) +
  geom_raster(alpha = 1) +
  facet_wrap(~layer, labeller = labeller(layer = layer.labs),
             scale="free_y", ncol = 1) +
  scale_fill_gradient2(
    low = "blue",
    mid = "white",
    high = "red",
    #midpoint = mean(plot_df$value, na.rm = TRUE),
    space = "Lab",
    na.value = "white",
    guide = "colourbar",
    aesthetics = "fill") + 
  geom_polygon(data=NZ, aes(x=long, y=lat, group=group), fill='transparent', col="black") +
  theme_bw() +
  ## use the extents from above.
  theme(legend.position = c(0.9, 0.3),
        legend.title = element_blank(),
        legend.key.height= unit(1.25, 'cm'),
        legend.key.width= unit(0.5, 'cm'),
        strip.text = element_text(size = 14),
        strip.text.x = element_text(angle = 0, hjust = 0.05),
        strip.background = element_blank(),
        strip.placement = "inside",
        axis.text = element_blank(),
        #plot.title = element_text(size = 28, margin=margin(t=0,b=-30)),
        legend.text=element_text(size=12),
        panel.grid.major = element_blank(),
        panel.grid.minor = element_blank())+
  labs(x = NULL, y = NULL)

p1.2 <- ggplot(data = subset(plot_df, plot_df$layer == "datedsites"),
               aes(x = x, y = y, fill = value)) +
  geom_raster(alpha = 1) +
  facet_wrap(~layer, labeller = labeller(layer = layer.labs),
             scale="free_y", ncol = 1) +
  scale_fill_gradient2(
    low = "blue",
    mid = "white",
    high = "red",
    #midpoint = mean(plot_df$value, na.rm = TRUE),
    space = "Lab",
    na.value = "white",
    guide = "colourbar",
    aesthetics = "fill") + 
  geom_polygon(data=NZ, aes(x=long, y=lat, group=group), fill='transparent', col="black") +
  theme_bw() +
  ## use the extents from above.
  theme(legend.position = c(0.9, 0.3),
        legend.title = element_blank(),
        legend.key.height= unit(1.25, 'cm'),
        legend.key.width= unit(0.5, 'cm'),
        strip.text = element_text(size = 14),
        strip.text.x = element_text(angle = 0, hjust = 0.05),
        strip.background = element_blank(),
        strip.placement = "inside",
        axis.text = element_blank(),
        #plot.title = element_text(size = 28, margin=margin(t=0,b=-30)),
        legend.text=element_text(size=12),
        panel.grid.major = element_blank(),
        panel.grid.minor = element_blank())+
  labs(x = NULL, y = NULL)

scales::show_col(viridis::viridis_pal()(3))

p1.3 <- ggplot(data = subset(plot_df, plot_df$layer == "delta"),
               aes(x = x, y = y, fill = value)) +
  geom_raster(alpha = 1) +
  facet_wrap(~layer, labeller = labeller(layer = layer.labs),
             scale="free_y", ncol = 1) +
  # scale_fill_gradient2(
  #   low = "#440154FF",
  #   mid = "#21908CFF",
  #   high = "#FDE725FF",
  #   #midpoint = mean(plot_df$value, na.rm = TRUE),
  #   space = "Lab",
  #   na.value = "white",
  #   guide = "colourbar",
  #   aesthetics = "fill") + 
  viridis::scale_fill_viridis(option="viridis",direction = 1,
                              guide = guide_colourbar(title = "NA",
                                                      title.position = "right",
                                                      title.theme = element_text(size = 8, colour = "black", face='bold'),
                                                      title.hjust = 0.5,
                                                      title.vjust = 5,
                                                      label = TRUE,
                                                      label.position = "right",
                                                      barwidth = 0.5,
                                                      barheight = 15,
                                                      ticks = FALSE,
                                                      direction = "vertical",
                                                      frame.colour = "black")) +
  geom_polygon(data=NZ, aes(x=long, y=lat, group=group), fill='transparent', col="black") +
  theme_bw() +
  ## use the extents from above.
  theme(legend.position = c(0.9, 0.3),
        legend.title = element_blank(),
        legend.key.height= unit(1.25, 'cm'),
        legend.key.width= unit(0.5, 'cm'),
        strip.text = element_text(size = 14),
        strip.text.x = element_text(angle = 0, hjust = 0.05),
        strip.background = element_blank(),
        strip.placement = "inside",
        axis.text = element_blank(),
        #plot.title = element_text(size = 28, margin=margin(t=0,b=-30)),
        legend.text=element_text(size=12),
        panel.grid.major = element_blank(),
        panel.grid.minor = element_blank())+
  labs(x = NULL, y = NULL)


names(twentyfive) <- c("archsites", "datedsites", "delta")
plot_df <- as.data.frame(twentyfive, xy=TRUE, long=TRUE)
plot_df <- plot_df[complete.cases(plot_df), ]
layer.labs <- c(archsites = "Archaeological sites",
                datedsites = "Radiocarbon dates",
                delta = "Δ SADIE distance")

p2.1 <- ggplot(data = subset(plot_df, plot_df$layer == "archsites"),
               aes(x = x, y = y, fill = value)) +
  geom_raster(alpha = 1) +
  facet_wrap(~layer, labeller = labeller(layer = layer.labs),
             scale="free_y", ncol = 1) +
  scale_fill_gradient2(
    low = "blue",
    mid = "white",
    high = "red",
    midpoint = mean(plot_df$value, na.rm = TRUE),
    space = "Lab",
    na.value = "white",
    guide = "colourbar",
    aesthetics = "fill") + 
  geom_polygon(data=NZ, aes(x=long, y=lat, group=group), fill='transparent', col="black") +
  theme_bw() +
  ## use the extents from above.
  theme(legend.position = c(0.9, 0.3),
        legend.title = element_blank(),
        legend.key.height= unit(1.25, 'cm'),
        legend.key.width= unit(0.5, 'cm'),
        strip.text = element_text(size = 14),
        strip.text.x = element_text(angle = 0, hjust = 0.05),
        strip.background = element_blank(),
        strip.placement = "inside",
        axis.text = element_blank(),
        #plot.title = element_text(size = 28, margin=margin(t=0,b=-30)),
        legend.text=element_text(size=12),
        panel.grid.major = element_blank(),
        panel.grid.minor = element_blank())+
  labs(x = NULL, y = NULL)

p2.2 <- ggplot(data = subset(plot_df, plot_df$layer == "datedsites"),
               aes(x = x, y = y, fill = value)) +
  geom_raster(alpha = 1) +
  facet_wrap(~layer, labeller = labeller(layer = layer.labs),
             scale="free_y", ncol = 1) +
  scale_fill_gradient2(
    low = "blue",
    mid = "white",
    high = "red",
    midpoint = mean(plot_df$value, na.rm = TRUE),
    space = "Lab",
    na.value = "white",
    guide = "colourbar",
    aesthetics = "fill") + 
  geom_polygon(data=NZ, aes(x=long, y=lat, group=group), fill='transparent', col="black") +
  theme_bw() +
  ## use the extents from above.
  theme(legend.position = c(0.9, 0.3),
        legend.title = element_blank(),
        legend.key.height= unit(1.25, 'cm'),
        legend.key.width= unit(0.5, 'cm'),
        strip.text = element_text(size = 14),
        strip.text.x = element_text(angle = 0, hjust = 0.05),
        strip.background = element_blank(),
        strip.placement = "inside",
        axis.text = element_blank(),
        #plot.title = element_text(size = 28, margin=margin(t=0,b=-30)),
        legend.text=element_text(size=12),
        panel.grid.major = element_blank(),
        panel.grid.minor = element_blank())+
  labs(x = NULL, y = NULL)

p2.3 <- ggplot(data = subset(plot_df, plot_df$layer == "delta"),
               aes(x = x, y = y, fill = value)) +
  geom_raster(alpha = 0.75) +
  facet_wrap(~layer, labeller = labeller(layer = layer.labs),
             scale="free_y", ncol = 1) +
  viridis::scale_fill_viridis(option="viridis",direction = 1,
                              guide = guide_colourbar(title = "NA",
                                                      title.position = "right",
                                                      title.theme = element_text(size = 8, colour = "black", face='bold'),
                                                      title.hjust = 0.5,
                                                      title.vjust = 5,
                                                      label = TRUE,
                                                      label.position = "right",
                                                      barwidth = 0.5,
                                                      barheight = 15,
                                                      ticks = FALSE,
                                                      direction = "vertical",
                                                      frame.colour = "black")) + 
  geom_polygon(data=NZ, aes(x=long, y=lat, group=group), fill='transparent', col="black") +
  theme_bw() +
  ## use the extents from above.
  theme(legend.position = c(0.9, 0.3),
        legend.title = element_blank(),
        legend.key.height= unit(1.25, 'cm'),
        legend.key.width= unit(0.5, 'cm'),
        strip.text = element_text(size = 14),
        strip.text.x = element_text(angle = 0, hjust = 0.05),
        strip.background = element_blank(),
        strip.placement = "inside",
        axis.text = element_blank(),
        #plot.title = element_text(size = 28, margin=margin(t=0,b=-30)),
        legend.text=element_text(size=12),
        panel.grid.major = element_blank(),
        panel.grid.minor = element_blank())+
  labs(x = NULL, y = NULL)


names(ten) <- c("archsites", "datedsites", "delta")
plot_df <- as.data.frame(ten, xy=TRUE, long=TRUE)
plot_df <- plot_df[complete.cases(plot_df), ]
layer.labs <- c(archsites = "Archaeological sites",
                datedsites = "Radiocarbon dates",
                delta = "Δ SADIE distance")

p3.1 <- ggplot(data = subset(plot_df, plot_df$layer == "archsites"),
               aes(x = x, y = y, fill = value)) +
  geom_raster(alpha = 1) +
  facet_wrap(~layer, labeller = labeller(layer = layer.labs),
             scale="free_y", ncol = 1) +
  scale_fill_gradient2(
    low = "blue",
    mid = "white",
    high = "red",
    midpoint = mean(plot_df$value, na.rm = TRUE),
    space = "Lab",
    na.value = "white",
    guide = "colourbar",
    aesthetics = "fill") + 
  geom_polygon(data=NZ, aes(x=long, y=lat, group=group), fill='transparent', col="black") +
  theme_bw() +
  ## use the extents from above.
  theme(legend.position = c(0.9, 0.3),
        legend.title = element_blank(),
        legend.key.height= unit(1.25, 'cm'),
        legend.key.width= unit(0.5, 'cm'),
        strip.text = element_text(size = 14),
        strip.text.x = element_text(angle = 0, hjust = 0.05),
        strip.background = element_blank(),
        strip.placement = "inside",
        axis.text = element_blank(),
        #plot.title = element_text(size = 28, margin=margin(t=0,b=-30)),
        legend.text=element_text(size=12),
        panel.grid.major = element_blank(),
        panel.grid.minor = element_blank())+
  labs(x = NULL, y = NULL)

p3.2 <- ggplot(data = subset(plot_df, plot_df$layer == "datedsites"),
               aes(x = x, y = y, fill = value)) +
  geom_raster(alpha = 1) +
  facet_wrap(~layer, labeller = labeller(layer = layer.labs),
             scale="free_y", ncol = 1) +
  scale_fill_gradient2(
    low = "blue",
    mid = "white",
    high = "red",
    midpoint = mean(plot_df$value, na.rm = TRUE),
    space = "Lab",
    na.value = "white",
    guide = "colourbar",
    aesthetics = "fill") + 
  geom_polygon(data=NZ, aes(x=long, y=lat, group=group), fill='transparent', col="black") +
  theme_bw() +
  ## use the extents from above.
  theme(legend.position = c(0.9, 0.3),
        legend.title = element_blank(),
        legend.key.height= unit(1.25, 'cm'),
        legend.key.width= unit(0.5, 'cm'),
        strip.text = element_text(size = 14),
        strip.text.x = element_text(angle = 0, hjust = 0.05),
        strip.background = element_blank(),
        strip.placement = "inside",
        axis.text = element_blank(),
        #plot.title = element_text(size = 28, margin=margin(t=0,b=-30)),
        legend.text=element_text(size=12),
        panel.grid.major = element_blank(),
        panel.grid.minor = element_blank())+
  labs(x = NULL, y = NULL)

p3.3 <- ggplot(data = subset(plot_df, plot_df$layer == "delta"),
               aes(x = x, y = y, fill = value)) +
  geom_raster(alpha = 0.75) +
  facet_wrap(~layer, labeller = labeller(layer = layer.labs),
             scale="free_y", ncol = 1) +
  viridis::scale_fill_viridis(option="viridis",direction = 1,
                              guide = guide_colourbar(title = "NA",
                                                      title.position = "right",
                                                      title.theme = element_text(size = 8, colour = "black", face='bold'),
                                                      title.hjust = 0.5,
                                                      title.vjust = 5,
                                                      label = TRUE,
                                                      label.position = "right",
                                                      barwidth = 0.5,
                                                      barheight = 15,
                                                      ticks = FALSE,
                                                      direction = "vertical",
                                                      frame.colour = "black")) + 
  geom_polygon(data=NZ, aes(x=long, y=lat, group=group), fill='transparent', col="black") +
  theme_bw() +
  ## use the extents from above.
  theme(legend.position = c(0.9, 0.3),
        legend.title = element_blank(),
        legend.key.height= unit(1.25, 'cm'),
        legend.key.width= unit(0.5, 'cm'),
        strip.text = element_text(size = 14),
        strip.text.x = element_text(angle = 0, hjust = 0.05),
        strip.background = element_blank(),
        strip.placement = "inside",
        axis.text = element_blank(),
        #plot.title = element_text(size = 28, margin=margin(t=0,b=-30)),
        legend.text=element_text(size=12),
        panel.grid.major = element_blank(),
        panel.grid.minor = element_blank())+
  labs(x = NULL, y = NULL)


combo <- p1.1 + ggtitle(" a) 50km resolution") + theme(plot.title = element_text(size = 28)) + p1.2 + p1.3 + 
         p2.1 + ggtitle(" b) 25km resolution") + theme(plot.title = element_text(size = 28)) + p2.2 + p2.3 +
         p3.1 + ggtitle(" c) 10km resolution") + theme(plot.title = element_text(size = 28)) + p3.2 + p1.3 

# writes plot to the PLOT_DIR
ggsave(filename = "sadie_plots.jpg", plot = combo,
       device = "jpg", type = "cairo", path = PLOT_DIR,
       width = 15, height = 20, units = "in",
       dpi = 300)


### Step 2: Plotting BRT Suitability plots ####

landsea <- raster(file.path(DATA_DIR, "NZ_landsea.grd"))

# import New Zealand coastline shapefile
NZ<-readOGR(file.path(DATA_DIR, "NZ_coast.shp"))
NZ <- spTransform(NZ, raster::crs(landsea))
NZ <- crop(NZ, raster::extent(landsea))
raster::plot(NZ)

NZ_raster <- raster(file.path(DATA_DIR, "nz_region_mask.grd"))
region <- Region$new(template_raster = NZ_raster)
cells <- region$region_cells
raster::plot(region$region_raster, main = "New Zealand study region (cell indices)",
             xlab = "Longitude (degrees)", ylab = "Latitude (degrees)",
             colNA = "blue")

mean_raster <- raster(file.path(MOD_DIR, "/hurdle outputs/Maori_EstabundMean.asc"))
crs(mean_raster) <- crs(NZ_raster)
mean_raster <- projectRaster(mean_raster, crs=crs(NZ_raster),res = res(NZ_raster), method="bilinear")
mean_raster <- mean_raster*NZ_raster

sd_raster <- raster(file.path(MOD_DIR, "/hurdle outputs/Maori_EstabundSD.asc"))
crs(sd_raster) <- crs(NZ_raster)
sd_raster <- projectRaster(sd_raster, crs=crs(NZ_raster),res = res(NZ_raster), method="bilinear")
sd_raster <- sd_raster*NZ_raster

DEM <- raster(file.path(DATA_DIR, "NZ_DEM_WGS84_20k.grd"))
crs(DEM) <- crs(NZ_raster)
DEM <- DEM*NZ_raster

xy <- read.csv(file.path(DATA_DIR, "NZ25km_current_v2.csv"))
xy <- xy[,c(4,5,7)]
colnames(xy) <- c("x", "y", "value")

dem_df <- as.data.frame(DEM, xy=TRUE, long=TRUE)
p1 <- ggplot(data = dem_df[complete.cases(dem_df), ],
             aes(x = x, y = y, fill = value)) +
  geom_raster(alpha = 1) +
  scale_fill_gradient2(
    low = "#018571",
    mid = "#80cdc1",
    high = "#ffffff",
    midpoint = mean(dem_df$value, na.rm = TRUE),
    space = "Lab",
    na.value = "grey50",
    guide = "none",
    aesthetics = "fill") + 
  geom_polygon(data=NZ, aes(x=long, y=lat, group=group), fill='transparent', col="black") +
  geom_point(data=subset(xy, xy$value>1), aes(size = value), col = "red") +
  scale_size(breaks=c(5,10,15,20,25,30,35,40),
             labels=c("5","10","15","20","25","30", "35", "40"),guide="legend") +
  theme_bw() +
  ## use the extents from above.
  theme(legend.position = c(0.9, 0.31),
        legend.title = element_blank(),
        legend.key.height= unit(1.15, 'cm'),
        legend.key.width= unit(0.5, 'cm'),
        axis.text = element_blank(),
        plot.title = element_text(size = 28, margin=margin(t=0,b=-30), vjust = -2.5),
        legend.text=element_text(size=12),
        panel.grid.major = element_blank(),
        panel.grid.minor = element_blank())+
  labs(x = NULL, y = NULL) +
  ggtitle(" a)")


# build data frame of effect sizes from BRT
presmod <- data.frame(
  factor = c("slope", "coast", "water", "coldest"),
  effect = c(29.5, 29.2, 19.1, 13.1),
  mod = rep("presence",4)
)

abundmod <- data.frame(
  factor = c("warmest", "rain", "ET0", "coldest"),
  effect = c(46.0, 20.8, 19.2, 13.8),
  mod = rep("abundance",4)
)

plotdf <- rbind(presmod,abundmod)
plotdf$mod <- factor(plotdf$mod, levels = c("presence", "abundance"))

p2 <- ggplot(plotdf, aes(x = reorder(factor, effect), y = effect))+
  geom_col(aes(fill = effect), width = 0.7) +
  geom_text(aes(label = effect),  hjust = 1.2, colour = "black") +
  scale_fill_gradient(
    low = "aliceblue",
    high = "royalblue3",
    guide = "none",
    aesthetics = "fill") +
  coord_flip() +
  facet_wrap(~mod, scale="free_y", ncol = 1,)+
  geom_hline(yintercept = 0, 
             color = "black") +
  theme_bw() +
  theme(legend.position = "none",
        legend.title = element_blank(),
        legend.key.height= unit(1.25, 'cm'),
        legend.key.width= unit(0.5, 'cm'),
        strip.text = element_text(size = 14),
        strip.text.x = element_text(angle = 0, hjust = 0.05),
        strip.background = element_blank(),
        plot.title = element_text(size = 28, margin=margin(t=0,b=-30), hjust = -0.06, vjust = -2.5),
        legend.text=element_text(size=22),
        panel.grid.major = element_blank(),
        panel.grid.minor = element_blank())+
  labs(x = NULL, y = "percent contribution") +
  ggtitle("b)")
  
  

mean_df <- as.data.frame(mean_raster, xy=TRUE, long=TRUE)
mean_df$round <- round(mean_df$value,0)
mean_df$round[mean_df$round == 0] <- NA

p3 <- ggplot(data = mean_df[complete.cases(mean_df), ],
             aes(x = x, y = y, fill = value)) +
  geom_raster(alpha = 0.85) +
  scale_fill_gradient2(
    low = "white",
    mid = "yellow",
    high = "red",
    #midpoint = mean(mean_df$value, na.rm = TRUE),
    space = "Lab",
    na.value = "grey50",
    guide = "colourbar",
    aesthetics = "fill") + 
  geom_polygon(data=NZ, aes(x=long, y=lat, group=group), fill='transparent', col="black") +
  theme_bw() +
  ## use the extents from above.
  theme(legend.position = c(0.9, 0.3),
        legend.title = element_blank(),
        legend.key.height= unit(1.25, 'cm'),
        legend.key.width= unit(0.5, 'cm'),
        axis.text = element_blank(),
        plot.title = element_text(size = 28, margin=margin(t=0,b=-30)),
        legend.text=element_text(size=12),
        panel.grid.major = element_blank(),
        panel.grid.minor = element_blank())+
  labs(x = NULL, y = NULL) +
  ggtitle(" c)")


sd_df <- as.data.frame(sd_raster, xy=TRUE, long=TRUE)
sd_df$round <- round(mean_df$value,0)
sd_df$round[sd_df$round == 0] <- NA

p4 <- ggplot(data = sd_df[complete.cases(sd_df), ],
             aes(x = x, y = y, fill = value)) +
  geom_raster(alpha = 1) +
  scale_fill_gradient2(
    #low = "#018571",
    low = "blue",
    mid = "#80cdc1",
    high = "#ffffcc",
    midpoint = mean(sd_df$value, na.rm = TRUE),
    space = "Lab",
    na.value = "grey50",
    guide = "colourbar",
    aesthetics = "fill") + 
  # viridis::scale_fill_viridis(option="magma",direction = 1,
  #                           guide = guide_colourbar(title = "NA",
  #                                                   title.position = "right",
  #                                                   title.theme = element_text(size = 8, colour = "black", face='bold'),
  #                                                   title.hjust = 0.5,
  #                                                   title.vjust = 5,
  #                                                   label = TRUE,
  #                                                   label.position = "right",
  #                                                   barwidth = 0.5,
  #                                                   barheight = 15,
  #                                                   ticks = FALSE,
  #                                                   direction = "vertical",
  #                                                   frame.colour = "black"))+
  geom_polygon(data=NZ, aes(x=long, y=lat, group=group), fill='transparent', col="black") +
  theme_bw() +
  theme(legend.position = c(0.9, 0.3),
        legend.title = element_blank(),
        legend.key.height= unit(1.25, 'cm'),
        legend.key.width= unit(0.5, 'cm'),
        axis.text = element_blank(),
        plot.title = element_text(size = 28, margin=margin(t=0,b=-30)),
        legend.text=element_text(size=12),
        panel.grid.major = element_blank(),
        panel.grid.minor = element_blank())+
  labs(x = NULL, y = NULL) +
  ggtitle(" d)")


# combine the plots
combo <- p1 + p2 + p3 + p4 + plot_layout(nrow = 2)
combo

# writes plot to the PLOT_DIR
ggsave(filename = "brt_plots.jpg", plot = combo,
       device = "jpg", type = "cairo", path = PLOT_DIR,
       width = 15, height = 15, units = "in",
       dpi = 300)


### Step 3: Building plots and ensemble rasters ####

#### Prior and posterior density plots ####
sample.data <- data.frame(read.csv(file.path(paste0(MOD_DIR,"/v1"),"sample_data.csv"))) # reads existing data from the MOD_DIR
sample.data2 <- data.frame(read.csv(file.path(paste0(MOD_DIR,"/v3"),"sample_data.csv"))) # reads existing data from the MOD_DIR

selected <- data.frame(read.csv(file.path(paste0(MOD_DIR,"/v3"),"selected_sims_index_final.csv"))) # reads existing data from the MOD_DIR
select.vec <- as.numeric(stringr::str_extract(string = selected[,2], pattern = "(?<=_)[^/]*(?=\\_)"))

myp <- as.data.frame(scale(sample.data[,2:6], center = TRUE, scale = TRUE))
myp$sample.no <- sample.data$X

myp2 <- as.data.frame(scale(sample.data2[,1:5], center = TRUE, scale = TRUE))
myp2$sample.no <- seq(from = 1, to = nrow(sample.data2), by =1)

mys <- myp2[myp2$sample.no %in% select.vec, ]

priors <- as.data.table(myp)
priors <- melt(priors, id.vars = "sample.no", value.name = "value", variable.name = "Variable")
levels(priors$Variable)

posteriors <- as.data.table(mys)
posteriors <- melt(posteriors, id.vars = "sample.no", value.name = "value", variable.name = "Variable")
levels(posteriors$Variable)

# Calculate 90% CI
posteriors[, Q050 := quantile(value, 0.050), by = Variable][
  , Q950 := quantile(value, 0.950), by = Variable]
priors[, Q050 := quantile(value, 0.050), by = Variable][
  , Q950 := quantile(value, 0.950), by = Variable]
idx_post <- which(posteriors$value >= posteriors$Q050 & posteriors$value <= posteriors$Q950)
idx_prior <- which(priors$value >= priors$Q050 & priors$value <= priors$Q950)

sub_post <- posteriors[idx_post, ]
sub_prior <- priors[idx_prior, ]

# summary stats
sumPrior <- priors[, .(min = min(value),
                       max = max(value),
                       median = median(value),
                       mode = raster::modal(value)),
                   by = Variable]
sumPrior
sumPost <- posteriors[, .(min = min(value),
                          max = max(value),
                          median = median(value),
                          mode = raster::modal(value)),
                      by = Variable]
sumPost
levels(sumPost$Variable)

# plotting error bars
p1 <- ggplot() +
  # Priors in grey
  geom_errorbar(data = sumPrior, width = 0.5,
                aes(x = Variable, ymin = min, ymax = max, group = Variable),
                colour = "grey80", size = 1, lty = 2) +
  geom_point(data = sumPrior,
             aes(x = Variable, y = median, group = Variable),
             size = 2, colour = "black", fill = "grey80", pch = 21) +
  # coloured posteriors
  geom_errorbar(data = sumPost, width = 0.5, size = 1,
                aes(x = Variable, ymin = min,ymax = max, group = Variable, colour = Variable)) +
  geom_point(data = sumPost,
             aes(x = Variable, y = median, group = Variable, fill = Variable),
             size = 2, colour = "black", pch = 23, show.legend = FALSE) +
  # define the colours
  scale_colour_manual(values = c("human_founding_population" = "darkred",
                                 "human_growth_rate" = "blue4",
                                 "human_settlement_time" = "green4",
                                 "human_neighbourhood_distance" = "maroon4",
                                 "human_min_neighbourhood" = "orangered")) +
  scale_fill_manual(values = c("human_founding_population" = "darkred",
                               "human_growth_rate" = "blue4",
                               "human_settlement_time" = "green4",
                               "human_neighbourhood_distance" = "maroon4",
                               "human_min_neighbourhood" = "orangered")) +
  # define the labels
  scale_x_discrete(labels =
                     c("Founding Pop.", "Growth Rate", "Settlement Time", "Neighbourhood Dist.", "Min. Neighbourhood")) +
  labs(y = "Scaled value", x = NULL) +
  theme_bw() +
  theme(
    panel.grid = element_blank(),
    axis.text.x = element_text(colour = "black", size = 10),
    legend.position = "none")
p1

# plotting ridges
dens_post <- sub_post[, Group := "posterior"]
dens_prior <- priors[, Group := "prior"]
dens_post; dens_prior
dens_plot <- rbindlist(list(dens_prior, dens_post))
dens_plot
dens_plot$Group <- factor(dens_plot$Group, levels = c("prior", "posterior"))

## uniform prior for all variables so can plot only 1
ridge_plot_prior <- dens_plot[Group == "prior" & Variable == "human_founding_population", ]
ridge_plot_prior$Variable <- "Unif. prior"
ridge_plot <- rbindlist(list(dens_plot[Group == "posterior", ], ridge_plot_prior))
ridge_plot[, "VarGroup"  := paste0(ridge_plot$Variable, "_", ridge_plot$Group)]
ridge_plot

levels(ridge_plot$Variable)
levels(ridge_plot$VarGroup)
ridge_plot$VarGroup <- factor(ridge_plot$VarGroup, levels = c(levels(as.factor(ridge_plot$VarGroup))))
levels(ridge_plot$VarGroup)

ridge_plot$Variable <- factor(ridge_plot$Variable, levels = c("human_founding_population",
                                                              "human_growth_rate",
                                                              "human_settlement_time",
                                                              "human_neighbourhood_distance",
                                                              "human_min_neighbourhood",
                                                              "Unif. prior"))
p2 <- ggplot() +
  geom_density_ridges(data = ridge_plot,
                      size = 0.5,
                      rel_min_height = 0.0001,
                      scale = 5,## controls the amount of overlap
                      colour = "black", alpha = 0.8,
                      aes(x = value, y = Variable,
                          fill = Variable),
  ) +
  scale_fill_manual(values = c("Unif. prior" = "grey80",
                               "human_founding_population" = "darkred",
                               "human_growth_rate" = "blue4",
                               "human_settlement_time" = "green4",
                               "human_neighbourhood_distance" = "maroon4",
                               "human_min_neighbourhood" = "orangered")) +
  scale_y_discrete(labels = c("Founding Pop.", "Growth Rate",
                              "Settlement Time", "Neighbourhood Dist.",
                              "Min. Neighbourhood","Uniform Prior")) +
  theme_minimal() +
  theme(legend.position = "none",
        axis.text = element_text(colour = "black", size = 10),
        axis.line.x = element_line(colour = "black", size = 0.5),
        axis.ticks.x = element_line(colour = "black", size = 0.5),
        panel.background = element_blank(),
        plot.background = element_blank(),
        panel.grid = element_blank()) +
  labs(x = "Scaled value", y = NULL)
p2

# combine the plots
p3 <- p1 / p2 + plot_layout(nrow = 2)
p3

# writes plot to the PLOT_DIR
ggsave(filename = "post_prior_combined.jpg", plot = p3,
       device = "jpg", type = "cairo", path = PLOT_DIR,
       width = 15, height = 10, units = "in",
       dpi = 300)


# Maps and pop growth ####

# plotting both the suitability in space and the trends

landsea <- raster(file.path(DATA_DIR, "NZ_landsea.grd"))

# import New Zealand coastline shapefile
NZ<-readOGR(file.path(DATA_DIR, "NZ_coast.shp"))
NZ <- spTransform(NZ, raster::crs(landsea))
NZ <- crop(NZ, raster::extent(landsea))
raster::plot(NZ)

NZ_raster <- raster(file.path(DATA_DIR, "nz_region_mask.grd"))
region <- Region$new(template_raster = NZ_raster)
cells <- region$region_cells
raster::plot(region$region_raster, main = "New Zealand study region (cell indices)",
             xlab = "Longitude (degrees)", ylab = "Latitude (degrees)",
             colNA = "blue")

# building ensembles
selected_sims<-read.csv(file.path(MOD_DIR,"/v3/selected_sims_index_final.csv")) # reads existing data from the MOD_DIR
selected_sims$X <- NULL
selected_weights<-selected_sims$Weights

# This requires simulations constructed in script "03_Maori_Simulations".These simulations can be established at line 1351
# by setting reset = 3 and running the simulation script.

SIMS_DIR <- paste0(DEMO_DIR, "/v3/sims")

ensemble_timeseries<-NULL
for(r in 1:nrow(selected_sims)){
  sample = paste0(selected_sims[r,1],".RDS")
  results = PopulationResults$new(results = readRDS(file.path(SIMS_DIR, sample)))
  
  occupancy <- results$get_attribute("human_density")
  
  density_raster <- region$raster_from_values(occupancy)
  names(density_raster) <- seq(from = 850, to = 1899, by = 1)
  
  # create dataframe for suitability trend
  # define time frame
  min<-as.numeric(str_sub(unlist(names(density_raster)[[1]]),2))
  max<-as.numeric(str_sub(unlist(names(density_raster)[[1050]]),2))
  
  time <- sort(seq(min, max, 1), decreasing = F)
  
  sp1 <- density_raster
  values(sp1)[values(sp1) == 0] <- NA
  aw_noz1 <- raster::mask(raster::area(sp1), sp1)
  weighted_hs1 <- sp1 * aw_noz1
  ts_hs1 <- raster::cellStats(weighted_hs1, sum, na.rm = TRUE) / raster::cellStats(aw_noz1, sum, na.rm = TRUE)
  sd1 <- sapply(1:nlayers(sp1), function(x,...) sqrt(Hmisc::wtd.var(values(sp1[[x]]), weights = values(aw_noz1[[x]]), na.rm = TRUE)))
  # adding range area trend
  sp1[!is.na(sp1)] <- 1 ## everything else to 1
  r_hs1 <- cellStats(sp1, sum) ## sum == count of cells that are suitable
  r_hpop1 <- colSums(occupancy)
  ## create data.table
  mdat <- list()
  mdat[[1]] <- data.table(ts_hs1)
  mdat[[1]] <- mdat[[1]][, .(Suitability = ts_hs1)]
  mdat[[1]][, Year := time]
  mdat[[1]][, SD := sd1]
  mdat[[1]][, Range := r_hs1]
  mdat[[1]][, Pop := r_hpop1]
  mdat[[1]][, DB := "Maori"]
  mdat[[1]][,  sampID := selected_sims[r,1]]
  mdat[[1]][,  wieght := selected_sims[r,11]]
  mdat <- rbindlist(mdat)
  mdat
  
  if (r == 1){
    ts_hs <- mdat
  }else{
    ts_hs <- rbind(ts_hs,mdat)
  }
  
}

write.csv(ts_hs, file.path(SIMS_DIR, "ts_hs.csv")) # writes new data to the DEMO_DIR

# Construct ensemble density
ensemble_timeseries<-NULL

human_density_list <- vector("list", nrow(selected_sims))
for(r in 1:nrow(selected_sims)){
  s = selected_sims[r,1]
  sample = paste0(s,".RDS")
  results = PopulationResults$new(results = readRDS(file.path(SIMS_DIR, sample))
  )
  occupancy <- results$get_attribute("human_density")
  human_density_list[[r]] <- occupancy
}

# Calculate the weighted cell human densities

selected_density <- vector("list", nrow(selected_sims))
for(r in 1:nrow(selected_sims)){sd<-(human_density_list[[r]]*selected_weights[[r]])/sum(selected_weights)
selected_density[[r]] <- sd
}


weighted_density <- NULL
ensemble_timeseries <- NULL
for(timestep in 1:TIMESTEPS){
  for(r in 1:nrow(selected_sims)){
    if(r == 1){weighted_density <- selected_density[[r]][,timestep]}else{
      weighted_density <- weighted_density + selected_density[[r]][,timestep]}
  }
  
  
  density_raster <- region$raster_from_values(weighted_density)
  if(timestep == 1){ensemble_timeseries <- density_raster}else{
    ensemble_timeseries <- raster::stack(ensemble_timeseries,density_raster)}
}

names(ensemble_timeseries) <- seq(from = 850, to = 1899, by = 1)

ensemble_timeseries
raster::plot(ensemble_timeseries[[449]])

raster::writeRaster(ensemble_timeseries, file.path(PLOT_DIR, "ensemble_human_abundance.grd"), overwrite = TRUE) # writes new data to the PLOT_DIR  


# Construct model agreement rasters

human_density_list <- vector("list", nrow(selected_sims))
for(r in 1:nrow(selected_sims)){
  s = selected_sims[r,1]
  sample = paste0(s,".RDS")
  results = PopulationResults$new(results = readRDS(file.path(SIMS_DIR, sample))
  )
  occupancy <- results$get_attribute("human_density")
  human_density_list[[r]] <- occupancy
}

human_density_list[[1]][,1]

# Calculate the weighted cell human densities

selected_density <- vector("list", nrow(selected_sims))
for(r in 1:nrow(selected_sims)){sd<-(human_density_list[[r]]*selected_weights[[r]])/sum(selected_weights)
selected_density[[r]] <- sd
}

binary_density <- selected_density
for(timestep in 1:TIMESTEPS){
  for(r in 1:nrow(selected_sims)){
    binary_density[[r]][,timestep][round(binary_density[[r]][,timestep],10) > 0] <- 1
  }
}

weighted_binary <- NULL
binary_timeseries <- NULL
for(timestep in 1:TIMESTEPS){
  for(r in 1:nrow(selected_sims)){
    if(r == 1){weighted_binary <- binary_density[[r]][,timestep]}else{
      weighted_binary <- weighted_binary + binary_density[[r]][,timestep]}
  }
  
  binary_raster <- (region$raster_from_values(weighted_binary))/nrow(selected_sims)
  if(timestep == 1){binary_timeseries <- binary_raster}else{
    binary_timeseries <- raster::stack(binary_timeseries,binary_raster)}
}

names(binary_timeseries) <- seq(from = 850, to = 1899, by = 1)

binary_timeseries
raster::plot(binary_timeseries[[385]])

values(binary_timeseries[[380]])

raster::writeRaster(binary_timeseries, file.path(PLOT_DIR, "model_agreement.grd"), overwrite = TRUE) # writes new data to the PLOT_DIR  


maori<-stack(file.path(MOD_DIR,"ensemble_human_abundance.grd"))  # reads new data from the PLOT_DIR 
maori<-maori[[c(381:951)]] #1230 - 1800

sum(raster::colSums(maori[[1]], na.rm = TRUE))
sum(raster::colSums(maori[[540]], na.rm = TRUE))

reg_data <- read.csv(file.path(DATA_DIR, "nz_region_data.csv"))
reg_data <- transform(reg_data, maori_index= ifelse(index<7, 1, 2))
reg_data <- transform(reg_data, maori_index= ifelse(index>=11 & index<=12, 3, maori_index))
reg_data <- transform(reg_data, maori_index= ifelse(index>=15, 3, maori_index))
reg_data

bioregional_indices <- lapply(as.list(levels(as.factor(reg_data$maori_index))),
                              function(i) {
                                which(nz_bioregions[region$region_indices] == i)
                              })

regional_density <- nz_bioregions
values(regional_density)[which(values(nz_bioregions) == 1)] <- mean(values(maori[[540]])[which(values(nz_bioregions) == 1)])
values(regional_density)[which(values(nz_bioregions) == 2)] <- mean(values(maori[[540]])[which(values(nz_bioregions) == 2)])
values(regional_density)[which(values(nz_bioregions) == 3)] <- mean(values(maori[[540]])[which(values(nz_bioregions) == 3)])

raster::plot(regional_density)

area_rast <- raster::area(regional_density)
area_rast <- mask(area_rast, region$region_raster)
plot(area_rast)

regional_density <- regional_density/area_rast

reg_df <- as.data.frame(regional_density, xy=TRUE, long=TRUE)


lay_stack <- stack(get("maori")[[540]])
lay_stack <- lay_stack/area_rast
raster::plot(lay_stack)

names(lay_stack) <- "Maori"
stack_df <- as.data.frame(lay_stack, xy=TRUE, long=TRUE)

p1 <- ggplot(data = stack_df,
             aes(x = x, y = y, fill = value)) +
  geom_raster() +
  #facet_wrap(~layer, ncol=1) +
  scale_fill_gradient2(low = "white", mid = "yellow",high = "red",
                       midpoint = mean(values(lay_stack), na.rm = TRUE),
                       limits = c(min(values(lay_stack), na.rm = TRUE),
                                  max(values(lay_stack), na.rm = TRUE)),
                       na.value = "transparent",
                       guide = guide_colourbar(title = "NA",
                                               title.position = "right",
                                               title.theme = element_text(size = 8, colour = "black", face='bold'),
                                               title.hjust = 0.5,
                                               title.vjust = 5,
                                               label = TRUE,
                                               label.position = "right",
                                               barwidth = 0.5,
                                               barheight = 15,
                                               ticks = FALSE,
                                               direction = "vertical",
                                               frame.colour = "black")) +
  geom_polygon(data=NZ, aes(x=long, y=lat, group=group), fill='transparent', col="black") +
  theme_bw() +
  ## use the extents from above.
  theme(legend.position = c(0.95, 0.2), axis.text = element_blank()) +
  theme(plot.title = element_text(size = 28),
        legend.text=element_text(size=22))+
  labs(x = NULL, y = NULL) +
  ggtitle("a) maximum population density")


p2 <- ggplot(data = reg_df[complete.cases(reg_df), ],
             aes(x = x, y = y, fill = value)) +
  geom_raster() +
  scale_fill_gradient2(low = "white", mid = "yellow",high = "red",
                       midpoint = mean(values(lay_stack), na.rm = TRUE),
                       limits = c(min(values(lay_stack), na.rm = TRUE),
                                  max(values(lay_stack), na.rm = TRUE)),
                       na.value = "transparent",
                       guide = guide_colourbar(title = "NA",
                                               title.position = "right",
                                               title.theme = element_text(size = 8, colour = "black", face='bold'),
                                               title.hjust = 0.5,
                                               title.vjust = 5,
                                               label = TRUE,
                                               label.position = "right",
                                               barwidth = 0.5,
                                               barheight = 15,
                                               ticks = FALSE,
                                               direction = "vertical",
                                               frame.colour = "black")) +
  geom_polygon(data=NZ, aes(x=long, y=lat, group=group), fill='transparent', col="black") +
  theme_bw() +
  ## use the extents from above.
  theme(legend.position = c(0.95, 0.2), axis.text = element_blank()) +
  theme(plot.title = element_text(size = 28),
        legend.text=element_text(size=22))+
  labs(x = NULL, y = NULL) +
  ggtitle("b) regional maximum population density")

combo <- p1 + p2

# writes new data to the PLOT_DIR
ggsave(filename = "population_density.jpg",
       plot = combo,
       path = PLOT_DIR,
       height = 15, width = 30, units = "in", dpi = 300)

#calculate growth raster

for(timestep in 2:TIMESTEPS){
  rast<-log(ensemble_timeseries[[timestep]]/ensemble_timeseries[[timestep-1]])
  if(timestep == 2){growth_raster <- rast}else{
    growth_raster <- raster::stack(growth_raster,rast)}
}

names(growth_raster) <- seq(from = 851, to = 1899, by = 1)
#growth_raster
values(growth_raster)[values(growth_raster) =="Inf"] = NA

avg_grwth <- calc(growth_raster[[c(1:550)]], mean, na.rm = TRUE)
max_grwth <- calc(growth_raster[[c(1:550)]], function(x) max(x, na.rm = TRUE))

avg_grwth <- 1+(avg_grwth/100)
max_grwth <- 1+(max_grwth/100)

raster::plot(avg_grwth)
raster::plot(max_grwth)

writeRaster(growth_raster, file.path(PLOT_DIR, "growthrate_stack.grd"), overwrite = TRUE) # writes new data to the PLOT_DIR

# average growth rate by economic regions (as per Brown and Crema)

# fix bioregions to 3 regions
nz_bioregions <- raster::raster(file.path(DATA_DIR, "nz_bioregions.grd"))
raster::values(nz_bioregions) <- raster::values(nz_bioregions)
raster::plot(nz_bioregions, colNA = "blue")

raster::plot(nz_bioregions)

nz_bioregions[nz_bioregions <7] <- 1
nz_bioregions[nz_bioregions >6 & nz_bioregions <11] <- 2
nz_bioregions[nz_bioregions >12 & nz_bioregions <15] <- 3
nz_bioregions[nz_bioregions >1 & nz_bioregions <11] <- 2
nz_bioregions[nz_bioregions >2] <- 3

raster::plot(nz_bioregions)

reg_data <- read.csv(file.path(DATA_DIR, "nz_region_data.csv"))
reg_data <- transform(reg_data, maori_index= ifelse(index<7, 1, 2))
reg_data <- transform(reg_data, maori_index= ifelse(index>=11 & index<=12, 3, maori_index))
reg_data <- transform(reg_data, maori_index= ifelse(index>=15, 3, maori_index))
reg_data

bioregional_indices <- lapply(as.list(levels(as.factor(reg_data$maori_index))),
                              function(i) {
                                which(nz_bioregions[region$region_indices] == i)
                              })

regional_growth <- nz_bioregions
values(regional_growth)[which(values(nz_bioregions) == 1)] <- mean(values(max_grwth)[which(values(nz_bioregions) == 1)])
values(regional_growth)[which(values(nz_bioregions) == 2)] <- mean(values(max_grwth)[which(values(nz_bioregions) == 2)])
values(regional_growth)[which(values(nz_bioregions) == 3)] <- mean(values(max_grwth)[which(values(nz_bioregions) == 3)])

raster::plot(regional_growth)


max_df <- as.data.frame(max_grwth, xy=TRUE, long=TRUE)
reg_df <- as.data.frame(regional_growth, xy=TRUE, long=TRUE)

p1 <- ggplot(data = max_df[complete.cases(max_df), ],
             aes(x = x, y = y, fill = value)) +
  geom_raster(alpha = 0.75) +
  viridis::scale_fill_viridis(option="viridis",direction = -1,
                              guide = guide_colourbar(title = "NA",
                                                      title.position = "right",
                                                      title.theme = element_text(size = 8, colour = "black", face='bold'),
                                                      title.hjust = 0.5,
                                                      title.vjust = 5,
                                                      label = TRUE,
                                                      label.position = "right",
                                                      barwidth = 0.5,
                                                      barheight = 15,
                                                      ticks = FALSE,
                                                      direction = "vertical",
                                                      frame.colour = "black"))+
  geom_polygon(data=NZ, aes(x=long, y=lat, group=group), fill='transparent', col="black") +
  theme_bw() +
  ## use the extents from above.
  theme(legend.position = "right", axis.text = element_blank()) +
  theme(plot.title = element_text(size = 28),
        legend.text=element_text(size=22))+
  labs(x = NULL, y = NULL) +
  ggtitle("a) maximum population growth rate")

p2 <- ggplot(data = reg_df[complete.cases(reg_df), ],
             aes(x = x, y = y, fill = value)) +
  geom_raster(alpha = 0.75) +
  viridis::scale_fill_viridis(option="viridis",direction = -1,
                              guide = guide_colourbar(title = "NA",
                                                      title.position = "right",
                                                      title.theme = element_text(size = 8, colour = "black", face='bold'),
                                                      title.hjust = 0.5,
                                                      title.vjust = 5,
                                                      label = TRUE,
                                                      label.position = "right",
                                                      barwidth = 0.5,
                                                      barheight = 15,
                                                      ticks = FALSE,
                                                      direction = "vertical",
                                                      frame.colour = "black"))+
  geom_polygon(data=NZ, aes(x=long, y=lat, group=group), fill='transparent', col="black") +
  theme_bw() +
  ## use the extents from above.
  theme(legend.position = "right", axis.text = element_blank()) +
  theme(plot.title = element_text(size = 28),
        legend.text=element_text(size=22))+
  labs(x = NULL, y = NULL) +
  ggtitle("b) mean regional population growth rate")

combo <- p1 + p2

# writes new data to the PLOT_DIR
ggsave(filename = "growth_rate.jpg",
       plot = combo,
       path = PLOT_DIR,
       height = 15, width = 30, units = "in", dpi = 300)


# create settlement year raster stack
for(r in 1:nrow(selected_sims)){
  sample = paste0(selected_sims[r,1],".RDS")
  dat = readRDS(file.path(SIMS_DIR, sample))
  
  occupancy <- dat$human_density
  settlement <- vector(mode = "list", length = 532)
  for(i in 1:532){
    list<-occupancy[i,]
    settlement[i]<-850+(min(which(list > 0)))
  }
  
  unlist(settlement)
  
  rast <- region$raster_from_values(do.call(rbind, settlement))
  names(rast) <- selected_sims[r,1]
  
  if(r==1){sett.stack <- rast}else{sett.stack <- raster::stack(sett.stack,rast)}
}

writeRaster(sett.stack, file.path(PLOT_DIR, "sett_stack.grd"), overwrite = TRUE)  # writes new data to the PLOT_DIR

# mapping settlement patterns
sett.stack <- stack(file.path(PLOT_DIR, "sett_stack.grd")) # reads new data from the PLOT_DIR

minlyr <- raster::calc(sett.stack, min)
maxlyr <- raster::calc(sett.stack, max)

meanlyr <- raster::calc(sett.stack, mean)
sdlyr <- raster::calc(sett.stack, sd)

CVlyr <- (sdlyr/meanlyr)*100
raster::plot(CVlyr)

mean_df <- as.data.frame(meanlyr, xy=TRUE, long=TRUE)
CV_df <- as.data.frame(CVlyr, xy=TRUE, long=TRUE)
min_df <- as.data.frame(minlyr, xy=TRUE, long=TRUE)
max_df <- as.data.frame(maxlyr, xy=TRUE, long=TRUE)


p1 <- ggplot(data = mean_df[complete.cases(mean_df), ],
             aes(x = x, y = y, fill = value)) +
  geom_raster(alpha = 0.75) +
  viridis::scale_fill_viridis(option="plasma",direction = -1,
                              guide = guide_colourbar(title = "NA",
                                                      title.position = "right",
                                                      title.theme = element_text(size = 8, colour = "black", face='bold'),
                                                      title.hjust = 0.5,
                                                      title.vjust = 5,
                                                      label = TRUE,
                                                      label.position = "right",
                                                      barwidth = 0.5,
                                                      barheight = 15,
                                                      ticks = FALSE,
                                                      direction = "vertical",
                                                      frame.colour = "black"))+
  geom_polygon(data=NZ, aes(x=long, y=lat, group=group), fill='transparent', col="black") +
  theme_bw() +
  theme(legend.position = "right", axis.text = element_blank(),
        text = element_text(size = 25)) +
  labs(x = NULL, y = NULL) +
  ggtitle("a) average estimated year of settlement")

p2 <- ggplot(data = CV_df[complete.cases(CV_df), ],
             aes(x = x, y = y, fill = value)) +
  geom_raster(alpha = 0.75) +
  viridis::scale_fill_viridis(option="plasma",direction = -1,
                              guide = guide_colourbar(title = "NA",
                                                      title.position = "right",
                                                      title.theme = element_text(size = 8, colour = "black", face='bold'),
                                                      title.hjust = 0.5,
                                                      title.vjust = 5,
                                                      label = TRUE,
                                                      label.position = "right",
                                                      barwidth = 0.5,
                                                      barheight = 15,
                                                      ticks = FALSE,
                                                      direction = "vertical",
                                                      frame.colour = "black"))+
  geom_polygon(data=NZ, aes(x=long, y=lat, group=group), fill='transparent', col="black") +
  theme_bw() +
  theme(legend.position = "right", axis.text = element_blank(),
        text = element_text(size = 25)) +
  labs(x = NULL, y = NULL) +
  ggtitle("b) Year of settlement coefficient of variation")

p3 <- ggplot(data = min_df[complete.cases(min_df), ],
             aes(x = x, y = y, fill = value)) +
  geom_raster(alpha = 0.75) +
  viridis::scale_fill_viridis(option="plasma",direction = -1,
                              guide = guide_colourbar(title = "NA",
                                                      title.position = "right",
                                                      title.theme = element_text(size = 8, colour = "black", face='bold'),
                                                      title.hjust = 0.5,
                                                      title.vjust = 5,
                                                      label = TRUE,
                                                      label.position = "right",
                                                      barwidth = 0.5,
                                                      barheight = 15,
                                                      ticks = FALSE,
                                                      direction = "vertical",
                                                      frame.colour = "black"))+
  geom_polygon(data=NZ, aes(x=long, y=lat, group=group), fill='transparent', col="black") +
  theme_bw() +
  theme(legend.position = "right", axis.text = element_blank(),
        text = element_text(size = 25)) +
  labs(x = NULL, y = NULL) +
  ggtitle("c) earliest estimated year of settlement")

p4 <- ggplot(data = max_df[complete.cases(max_df), ],
             aes(x = x, y = y, fill = value)) +
  geom_raster(alpha = 0.75) +
  viridis::scale_fill_viridis(option="plasma",direction = -1,
                              guide = guide_colourbar(title = "NA",
                                                      title.position = "right",
                                                      title.theme = element_text(size = 8, colour = "black", face='bold'),
                                                      title.hjust = 0.5,
                                                      title.vjust = 5,
                                                      label = TRUE,
                                                      label.position = "right",
                                                      barwidth = 0.5,
                                                      barheight = 15,
                                                      ticks = FALSE,
                                                      direction = "vertical",
                                                      frame.colour = "black"))+
  geom_polygon(data=NZ, aes(x=long, y=lat, group=group), fill='transparent', col="black") +
  theme_bw() +
  theme(legend.position = "right", axis.text = element_blank(),
        text = element_text(size = 25)) +
  labs(x = NULL, y = NULL) +
  ggtitle("d) latest estimated year of settlement")


# plot range area
ts_hs <- fread(file.path(SIMS_DIR, "ts_hs.csv"))
ts_hs$V1 <- NULL
ts_hs[is.na(ts_hs)] <- 0

sample_seq <- unlist(levels(as.factor(ts_hs$sampID)))

ts_hs2 <- NULL

for(i in 1:length(sample_seq)){
  ts_1 <- subset(ts_hs,ts_hs$sampID == sample_seq[i])
  ts_1$exp_rate <- ts_1$Range/(min(which(ts_1$Range>0)))
  
  ts_hs2 <- rbind(ts_hs2,ts_1)
}

ts_hs <- ts_hs2

ts_mean <- data.table(aggregate(. ~ Year, data = ts_hs[,c(1:5,9)], mean)) # averages across all selected sims

p6 <- ggplot() + geom_line(data = ts_hs[(Year-1) <= 1899], aes(x=Year, y=Pop), size=0.75, col = "pink") + 
  geom_line(data = ts_mean[(Year-1) <= 1899], aes(x=Year, y=Pop), size=1.5, col = "black") +
  
  geom_line(data = ts_hs[(Year-1) <= 1899], aes(x=Year, y=exp_rate*150000), size=0.75, col = "lightblue") + 
  geom_line(data = ts_mean[(Year-1) <= 1899], aes(x=Year, y=exp_rate*150000), size=1.5, col = "black") +
  
  xlab("Year AD") +
  ylab("Range area (sq km)") +
  theme_bw() +
  xlim(min(min_df$value, na.rm = TRUE)-40, 1775) +
  ylim(min(ts_hs$Pop, na.rm=T), max(ts_hs$Range, na.rm = T)*(20*20)) +
  
  theme(panel.grid.major = element_blank(),
        panel.grid.minor = element_blank(),
        axis.line = element_line(colour = "black"),
        legend.position = "none",
        text = element_text(size = 25)) +
  ggtitle("f) estimated population dynamics")

p6 <- p6 + scale_y_continuous(name = "Population size (n)",
                              sec.axis = sec_axis( ~ . / 150000 , name = "Range expansion rate"),
                              limits = c(min(ts_hs$Pop, na.rm=T), max(ts_hs$Range, na.rm = T)*(20*20)))


# growth rate
growth_raster <- stack(file.path(PLOT_DIR, "growthrate_stack.grd"))
max_grwth <- calc(growth_raster[[c(1:550)]], function(x) max(x, na.rm = TRUE))
max_grwth <- 1+(max_grwth/100)
max_df <- as.data.frame(max_grwth, xy=TRUE, long=TRUE)

p5 <- ggplot(data = max_df[complete.cases(max_df), ],
             aes(x = x, y = y, fill = value)) +
  geom_raster(alpha = 0.75) +
  viridis::scale_fill_viridis(option="viridis",direction = -1,
                              guide = guide_colourbar(title = "NA",
                                                      title.position = "right",
                                                      title.theme = element_text(size = 8, colour = "black", face='bold'),
                                                      title.hjust = 0.5,
                                                      title.vjust = 5,
                                                      label = TRUE,
                                                      label.position = "right",
                                                      barwidth = 0.5,
                                                      barheight = 15,
                                                      ticks = FALSE,
                                                      direction = "vertical",
                                                      frame.colour = "black"))+
  geom_polygon(data=NZ, aes(x=long, y=lat, group=group), fill='transparent', col="black") +
  theme_bw() +
  theme(legend.position = "right", axis.text = element_blank(),
        text = element_text(size = 25)) +
  labs(x = NULL, y = NULL) +
  ggtitle("e) maximum population growth rate")


plot(p1+p2)/(p3+p4)/(p5 +p6)

comboplot <- (p1 + p2 + p3 + p4 + p5 + p6) + plot_layout(ncol = 2, widths = c(2,2))

# writes new data to the PLOT_DIR
ggsave(filename = "settlement_dynamic.jpg",
       plot = comboplot,
       path = PLOT_DIR,
       height = 40, width = 30, units = "in", dpi = 300)


# Histograms and boxplots ####

first_selected <- read.csv (file.path(MOD_DIR, "/v1/summary_metric_data.csv")) # reads existing data from the MOD_DIR
first_selected$X <- NULL
first_selected$resample_run <- 1

second_selected <- read.csv (file.path(MOD_DIR, "/v2/summary_metric_data.csv")) # reads existing data from the MOD_DIR
second_selected$X <- NULL
second_selected$resample_run <- 2

third_selected <- read.csv (file.path(MOD_DIR, "/v3/summary_metric_data.csv")) # reads existing data from the MOD_DIR
third_selected$X <- NULL
third_selected$resample_run <- 3

dat <- rbind(first_selected, second_selected, third_selected)
dat$resample_run <- as.factor(dat$resample_run)

#dat <- dat[dat$resample_run == "1" | dat$resample_run == "2" | dat$resample_run == "3", ]



p <- ggplot2.histogram(data=dat, xName='final_abundance',
                       groupName='resample_run', legendPosition="top",
                       alpha=0.9, bins = 50)+
  scale_x_continuous(trans='log10')+
  scale_y_continuous(trans='log10')+
  viridis::scale_fill_viridis(option="viridis",direction = -1, discrete = TRUE)+
  geom_vline(xintercept = 100000, color = "red", size=1.0)+
  geom_vline(xintercept = 150000, color = "red", size=1.0)+
  labs(x = "log population size", y = "number of simulations")+
  theme_bw() +
  theme(
    panel.grid.major = element_blank(),
    panel.grid.minor = element_blank(),
    text = element_text(size = 18),
    legend.position = "none"
  )


p1 <- ggplot2.histogram(data=dat, xName='occupancy_accuracy',
                        groupName='resample_run', legendPosition="top",
                        alpha=0.9, bins = 33)+
  scale_y_log10()+
  viridis::scale_fill_viridis(option="viridis",direction = -1, discrete = TRUE)+
  geom_vline(xintercept = 1.0, color = "red", size=1.0)+
  labs(x = "proportion accurate", y = "number of simulations")+
  theme_bw() +
  theme(
    panel.grid.major = element_blank(),
    panel.grid.minor = element_blank(),
    text = element_text(size = 18),
    legend.position = "none"
  )


# parameter box plots####

# reshape data frame
dt1 <- as.data.frame(cbind(dat[,2],dat[,11]));colnames(dt1) <- c("space", "resample_run"); dt1$parameter <- "founding population"
dt2 <- as.data.frame(cbind(dat[,3],dat[,11]));colnames(dt2) <- c("space", "resample_run"); dt2$parameter <- "growth rate"
dt3 <- as.data.frame(cbind(dat[,4],dat[,11]));colnames(dt3) <- c("space", "resample_run"); dt3$parameter <- "settlement year"
dt4 <- as.data.frame(cbind(dat[,5],dat[,11]));colnames(dt4) <- c("space", "resample_run"); dt4$parameter <- "neighbourhood distance"
dt5 <- as.data.frame(cbind(dat[,6],dat[,11]));colnames(dt5) <- c("space", "resample_run"); dt5$parameter <- "neighbourhood population"

dat2 <- as.data.frame(rbind(dt1,dt2,dt3,dt4,dt5))
dat2 <- subset(dat2, dat2$resample_run<4)

dat2$resample_run <- as.factor(dat2$resample_run)

p3 <- ggplot(dat2, aes(x=parameter, y=space, fill=resample_run)) + 
  geom_boxplot() +
  facet_wrap(~parameter, scale="free", ncol = 1,
             strip.position = "left",
             labeller = as_labeller(c("founding population" = "founding population (n)",
                                      "growth rate" = "growth rate (R)",
                                      "settlement year" = "settlement year (C.E.)",
                                      "neighbourhood distance" = "neighbourhood distance (km)",
                                      "neighbourhood population" = "neighbourhood size (n)")))+
  viridis::scale_fill_viridis(option="viridis",direction = -1, discrete = TRUE, alpha = 0.5)+
  labs(x = element_blank(), y = element_blank(), fill = "iteration") +
  ggtitle("b) parameter range") +
  theme_bw() +
  theme(
    panel.grid.major = element_blank(),
    panel.grid.minor = element_blank(),
    text = element_text(size = 18),
    axis.title.x=element_blank(),
    axis.text.x=element_blank(),
    axis.ticks.x=element_blank(),
    strip.text.x = element_blank(),
    strip.text.y = element_text(size = 10),
    strip.background = element_blank(),
    strip.placement = "outside",
    legend.position = "none")

p <- p + ggtitle("a) target accuracy")

((p / p1 + plot_layout(guides = 'auto')) | p3) + plot_layout(guides = 'collect')

combo <- ((p / p1 + plot_layout(guides = 'auto')) | p3) +
  plot_layout(guides = 'collect')

ggsave(filename = "Maori_histograms.jpg",
       plot = combo,
       path = PLOT_DIR,
       height = 22, width = 15, units = "in", dpi = 300)


#### Step 4: Plotting Sensitivity Comparisons ####

exp_selected <- read.csv (file.path(MOD_DIR, "/v2/summary_metric_data.csv")) # reads existing data from the MOD_DIR
exp_selected <- data.frame(stack(exp_selected[c(8:9,11)]))
exp_selected$model <- "exponential"

logistic_selected <- read.csv (file.path(SENS_DIR, "/logistic/summary_metric_data.csv")) # reads existing data from the SENS_DIR
logistic_selected <- data.frame(stack(logistic_selected[c(9:10,12)]))
logistic_selected$model <- "logistic"

metrics <- rbind(exp_selected, logistic_selected)
colnames(metrics) <- c( "value", "target","model")

metrics <- subset(metrics, !metrics$target == "population_accuracy")

metrics$target <- as.factor(metrics$target)
levels(metrics$target)
metrics$model <- as.factor(metrics$model)
levels(metrics$model)

dummy <- data.frame(target = c("final_abundance", "final_abundance", "occupancy_accuracy"), Z = c(150000, 100000, 1.0))

p1 <- ggplot(subset(metrics, metrics$target == "final_abundance"),
             aes(x=model, y=value, fill=model, alpha = 0.75)) + 
  geom_boxplot(colour = "black",outlier.shape = NA) +
  #facet_wrap(~target, scale="free")+
  scale_fill_manual(values=c("blue4", "darkred")) +
  geom_hline(data = dummy[c(1:2),], aes(yintercept = Z), linetype='dashed', colour = "darkred", size = 1)+
  labs(x = element_blank(), y = element_blank(), fill = "iteration") +
  ggtitle("a) final abundance") +
  ylim(c(0, 250000)) +
  theme_bw() +
  theme(
    plot.title = element_text(size=25),
    axis.text = element_text(size = 15),
    axis.title = element_text(size = 15),
    strip.text.x = element_text(size = 15)
  ) +
  theme(legend.position = "none")

p2 <- ggplot(subset(metrics, metrics$target == "occupancy_accuracy"),
             aes(x=model, y=value, fill=model, alpha = 0.75)) + 
  geom_boxplot(colour = "black",outlier.shape = NA) +
  #facet_wrap(~target, scale="free")+
  scale_fill_manual(values=c("blue4", "darkred")) +
  geom_hline(data = dummy[3,], aes(yintercept = Z), linetype='dashed', colour = "darkred", size = 1)+
  labs(x = element_blank(), y = element_blank(), fill = "iteration") +
  ggtitle("b) occupancy") +
  theme_bw() +
  theme(
    plot.title = element_text(size=25),
    axis.text = element_text(size = 15),
    axis.title = element_text(size = 15),
    strip.text.x = element_text(size = 15)
  ) +
  theme(legend.position = "none")


# building ensembles and plotting settlement differences
selected_sims<-read.csv(file.path(SENS_DIR,"/logistic/selected_sims_index_final.csv")) # reads existing data from the SENS_DIR
selected_sims$X <- NULL
selected_weights<-selected_sims$Weights

# This requires simulations constructed in script "04_Sensitivity_Analysis".These simulations can be established at line 360
# by setting reset = 3 and running the simulation script.

SIMS_DIR <- paste0(DEMO_DIR, "/logistic/sims")

# create settlement year raster stack
for(r in 1:nrow(selected_sims)){
  sample = paste0(selected_sims[r,1],".RDS")
  dat = readRDS(file.path(SIMS_DIR, sample))
  
  occupancy <- dat$human_density
  settlement <- vector(mode = "list", length = 532)
  for(i in 1:532){
    list<-occupancy[i,]
    settlement[i]<-850+(min(which(list > 0)))
  }
  
  unlist(settlement)
  
  rast <- region$raster_from_values(do.call(rbind, settlement))
  names(rast) <- selected_sims[r,1]
  
  if(r==1){sett.stack <- rast}else{sett.stack <- raster::stack(sett.stack,rast)}
}

writeRaster(sett.stack, file.path(PLOT_DIR, "weighted_settlement_logistic.grd"), overwrite = TRUE)  # writes new data to the PLOT_DIR

# mapping settlement patterns
sett.stack <- stack(file.path(MOD_DIR, "sett_stack.grd")) # reads new data from the PLOT_DIR
logistic.sett.stack <- stack(file.path(SENS_DIR, "/logistic/weighted_settlement_logistic.grd")) # reads new data from the PLOT_DIR

meanexp <- raster::calc(sett.stack, mean)
meanlog <- raster::calc(logistic.sett.stack, mean)

deltalyr <- abs(meanexp - meanlog)

delta_df <- as.data.frame(deltalyr, xy=TRUE, long=TRUE)

p3 <- ggplot(data = delta_df[complete.cases(delta_df), ],
             aes(x = x, y = y, fill = value)) +
  geom_raster(alpha = 0.75) +
  viridis::scale_fill_viridis(option="plasma",direction = 1,
                              guide = guide_colourbar(title = "NA",
                                                      title.position = "right",
                                                      title.theme = element_text(size = 8, colour = "black", face='bold'),
                                                      title.hjust = 0.5,
                                                      title.vjust = 5,
                                                      label = TRUE,
                                                      label.position = "right",
                                                      barwidth = 0.5,
                                                      barheight = 15,
                                                      ticks = FALSE,
                                                      direction = "vertical",
                                                      frame.colour = "black"))+
  geom_polygon(data=NZ, aes(x=long, y=lat, group=group), fill='transparent', col="black") +
  theme_bw() +
  theme(legend.position = "right", axis.text = element_blank(),
        text = element_text(size = 25)) +
  labs(x = NULL, y = NULL) +
  ggtitle("c) Δ colonisation time (years)")

comboplot <- ((p1 / p2) + p3) + plot_layout(ncol = 2, widths = c(1,2))

# writes new data to the PLOT_DIR
ggsave(filename = "Maori_growth_sensitivity.jpg",
       plot = comboplot,
       path = PLOT_DIR,
       height = 40, width = 20, units = "in", dpi = 300)


# split founders boxplots

metrics<- read.csv(file.path(SENS_DIR, "/split_founders/summary_metric_data.csv")) # reads existing data from the SENS_DIR
metrics$human_founding_split <- as.factor(metrics$human_founding_split)
metrics$X <- NULL

abundance.dat <- cbind(metrics[,c(7,11)])
abundance.dat$target <- "final.abundance"
names(abundance.dat)[names(abundance.dat) == 'final_abundance'] <- 'value'

occupancy.dat <- cbind(metrics[,c(10,11)])
occupancy.dat$target <- "occupancy"
names(occupancy.dat)[names(occupancy.dat) == 'occupancy_accuracy'] <- 'value'

metrics2 <- rbind(abundance.dat,occupancy.dat)

dummy2 <- data.frame(target = c("final.abundance", "final.abundance", "occupancy"), Z = c(150000, 100000, 1.0))

p1 <- ggplot(abundance.dat, aes(x=human_founding_split, y=value, fill=human_founding_split, alpha = 0.75)) + 
  geom_boxplot(colour = "black") +
  scale_fill_brewer(palette = "RdYlBu")+
  geom_hline(data = dummy2[c(1:2),], aes(yintercept = Z), linetype='dashed', colour = "darkred", size = 1)+
  labs(x = "number of founding events", y = element_blank()) +
  ggtitle("a) final abundance") +
  theme_bw() +
  theme(
    plot.title = element_text(size=25),
    axis.text = element_text(size = 15),
    axis.title = element_text(size = 15),
    strip.text.y = element_blank()
  ) +
  theme(legend.position = "none")


p2 <- ggplot(occupancy.dat, aes(x=human_founding_split, y=value, fill=human_founding_split, alpha = 0.75)) + 
  geom_boxplot(colour = "black") +
  scale_fill_brewer(palette = "RdYlBu")+
  geom_hline(data = dummy2[3,], aes(yintercept = Z), linetype='dashed', colour = "darkred", size = 1)+
  labs(x = "number of founding events", y = element_blank()) +
  ggtitle("b) occupancy") +
  theme_bw() +
  theme(
    plot.title = element_text(size=25),
    axis.text = element_text(size = 15),
    axis.title = element_text(size = 15),
    strip.text.y = element_blank()
  ) +
  theme(legend.position = "none")

# building ensembles and plotting settlement differences

# This process needs to be repeated for each of the four replicates of founding events. This can be done by chaning
# the value of 'splitnum' and repeating the validation process.

splitnum <- 1 # the number of founding events (1, 2, 5, or 10) will be used to select the best simulations for each scenario

selected_sims<-read.csv(paste0(SENS_DIR, "/split_founders/selected_sims_index_final_", splitnum, ".csv")) # reads existing data from the SENS_DIR
selected_sims$X <- NULL
selected_weights<-selected_sims$Weights

# This requires simulations constructed in script "04_Sensitivity_Analysis".These simulations can be established at line 1324
# by setting reset = 3 and running the simulation script.

SIMS_DIR <- paste0(DEMO_DIR, "/split_founders/sims")

# create settlement year raster stack
for(r in 1:nrow(selected_sims)){
  sample = paste0(selected_sims[r,1],".RDS")
  dat = readRDS(file.path(SIMS_DIR, sample))
  
  occupancy <- dat$human_density
  settlement <- vector(mode = "list", length = 532)
  for(i in 1:532){
    list<-occupancy[i,]
    settlement[i]<-850+(min(which(list > 0)))
  }
  
  unlist(settlement)
  
  rast <- region$raster_from_values(do.call(rbind, settlement))
  names(rast) <- selected_sims[r,1]
  
  if(r==1){sett.stack <- rast}else{sett.stack <- raster::stack(sett.stack,rast)}
}

writeRaster(sett.stack, file.path(PLOT_DIR, paste0("weighted_settlement_split", splitnum,".grd")), overwrite = TRUE)  # writes new data to the PLOT_DIR

# mapping settlement patterns - 2 events
sett.stack <- stack(file.path(PLOT_DIR, "weighted_settlement_split1.grd")) # reads new data from the PLOT_DIR
founder.sett.stack <- stack(file.path(PLOT_DIR, "weighted_settlement_split2.grd")) # reads new data from the PLOT_DIR

meanexp <- raster::calc(sett.stack, mean)
meanlog <- raster::calc(founder.sett.stack, mean)

deltalyr <- abs(meanexp - meanlog)

delta_df <- as.data.frame(deltalyr, xy=TRUE, long=TRUE)

p3 <- ggplot(data = delta_df[complete.cases(mean_df), ],
             aes(x = x, y = y, fill = value)) +
  geom_raster(alpha = 0.75) +
  viridis::scale_fill_viridis(option="plasma",direction = -1,
                              guide = guide_colourbar(title = "NA",
                                                      title.position = "right",
                                                      title.theme = element_text(size = 8, colour = "black", face='bold'),
                                                      title.hjust = 0.5,
                                                      title.vjust = 5,
                                                      label = TRUE,
                                                      label.position = "right",
                                                      barwidth = 0.5,
                                                      barheight = 15,
                                                      ticks = FALSE,
                                                      direction = "vertical",
                                                      frame.colour = "black"))+
  geom_polygon(data=NZ, aes(x=long, y=lat, group=group), fill='transparent', col="black") +
  theme_bw() +
  theme(legend.position = "right", axis.text = element_blank(),
        text = element_text(size = 25)) +
  labs(x = NULL, y = NULL) +
  ggtitle("c) Δ colonisation (2 events)")

# mapping settlement patterns - 5 events
sett.stack <- stack(file.path(PLOT_DIR, "weighted_settlement_split1.grd")) # reads new data from the PLOT_DIR
founder.sett.stack <- stack(file.path(PLOT_DIR, "weighted_settlement_split5.grd")) # reads new data from the PLOT_DIR

meanexp <- raster::calc(sett.stack, mean)
meanlog <- raster::calc(founder.sett.stack, mean)

deltalyr <- abs(meanexp - meanlog)

delta_df <- as.data.frame(deltalyr, xy=TRUE, long=TRUE)

p4 <- ggplot(data = delta_df[complete.cases(mean_df), ],
             aes(x = x, y = y, fill = value)) +
  geom_raster(alpha = 0.75) +
  viridis::scale_fill_viridis(option="plasma",direction = -1,
                              guide = guide_colourbar(title = "NA",
                                                      title.position = "right",
                                                      title.theme = element_text(size = 8, colour = "black", face='bold'),
                                                      title.hjust = 0.5,
                                                      title.vjust = 5,
                                                      label = TRUE,
                                                      label.position = "right",
                                                      barwidth = 0.5,
                                                      barheight = 15,
                                                      ticks = FALSE,
                                                      direction = "vertical",
                                                      frame.colour = "black"))+
  geom_polygon(data=NZ, aes(x=long, y=lat, group=group), fill='transparent', col="black") +
  theme_bw() +
  theme(legend.position = "right", axis.text = element_blank(),
        text = element_text(size = 25)) +
  labs(x = NULL, y = NULL) +
  ggtitle("c) Δ colonisation (5 events)")

# mapping settlement patterns - 10 events
sett.stack <- stack(file.path(PLOT_DIR, "weighted_settlement_split1.grd")) # reads new data from the PLOT_DIR
founder.sett.stack <- stack(file.path(PLOT_DIR, "weighted_settlement_split10.grd")) # reads new data from the PLOT_DIR

meanexp <- raster::calc(sett.stack, mean)
meanlog <- raster::calc(founder.sett.stack, mean)

deltalyr <- abs(meanexp - meanlog)

delta_df <- as.data.frame(deltalyr, xy=TRUE, long=TRUE)

p5 <- ggplot(data = delta_df[complete.cases(mean_df), ],
             aes(x = x, y = y, fill = value)) +
  geom_raster(alpha = 0.75) +
  viridis::scale_fill_viridis(option="plasma",direction = -1,
                              guide = guide_colourbar(title = "NA",
                                                      title.position = "right",
                                                      title.theme = element_text(size = 8, colour = "black", face='bold'),
                                                      title.hjust = 0.5,
                                                      title.vjust = 5,
                                                      label = TRUE,
                                                      label.position = "right",
                                                      barwidth = 0.5,
                                                      barheight = 15,
                                                      ticks = FALSE,
                                                      direction = "vertical",
                                                      frame.colour = "black"))+
  geom_polygon(data=NZ, aes(x=long, y=lat, group=group), fill='transparent', col="black") +
  theme_bw() +
  theme(legend.position = "right", axis.text = element_blank(),
        text = element_text(size = 25)) +
  labs(x = NULL, y = NULL) +
  ggtitle("c) Δ colonisation (10 events)")


comboplot <- p1 / p2 /(p3 + p4 + p5)

# writes new data to the PLOT_DIR
ggsave(filename = "Maori_settlement_sensitivity.jpg",
       plot = comboplot,
       path = PLOT_DIR,
       height = 40, width = 30, units = "in", dpi = 300)


#### Step 5: Animating Settlement Dynamic ####
maori<-stack(file.path(PLOT_DIR,"ensemble_human_abundance.grd"))  # reads new data from the PLOT_DIR 
maori<-maori[[c(381:951)]] #1230 - 1800

sum(raster::colSums(maori[[1]], na.rm = TRUE))
sum(raster::colSums(maori[[540]], na.rm = TRUE))

landsea <- raster(file.path(DATA_DIR, "NZ_landsea.grd"))

# import New Zealand coastline shapefile
NZ<-readOGR(file.path(DATA_DIR, "NZ_coast.shp"))
NZ <- spTransform(NZ, raster::crs(landsea))
NZ <- crop(NZ, raster::extent(landsea))
raster::plot(NZ)

# define time frame
min<-as.numeric(str_sub(unlist(names(maori)[[1]]),2))
max<-as.numeric(str_sub(unlist(names(maori)[[571]]),2))

# time is sorted into 20 year bins here because the habitat suitability rasters are stacked by 20 year layers
time <- sort(seq(min, max, 1), decreasing = F)

#### create dataframe for suitability trend ####
sp1 <- get("maori")
values(sp1)[values(sp1) == 0] <- NA
spplot(stack(sp1[[1]]))
aw_noz1 <- raster::mask(raster::area(sp1), sp1)
weighted_hs1 <- sp1 * aw_noz1
ts_hs1 <- raster::cellStats(weighted_hs1, sum, na.rm = TRUE) / raster::cellStats(aw_noz1, sum, na.rm = TRUE)
sd1 <- sapply(1:nlayers(sp1), function(x,...) sqrt(Hmisc::wtd.var(values(sp1[[x]]), weights = values(aw_noz1[[x]]), na.rm = TRUE)))

# adding range area trend
sp1[!is.na(sp1)] <- 1 ## everything else to 1
r_hs1 <- cellStats(sp1, sum) ## sum == count of cells that are suitable

## create data.table
ts_hs <- list()
ts_hs[[1]] <- data.table(ts_hs1)
ts_hs[[1]] <- ts_hs[[1]][, .(Suitability = ts_hs1)]
ts_hs[[1]][, Year := time]
ts_hs[[1]][, SD := sd1]
ts_hs[[1]][, Range := r_hs1]
ts_hs[[1]][, DB := "Maori"]
ts_hs <- rbindlist(ts_hs)
ts_hs

# remove files that won't be needed
rm(aw_noz1, aw_noz2, sp1, sp2, weighted_hs1, weighted_hs2, r_hs1, r_hs2, sd1, sd2, ts_hs1, ts_hs2); gc()

#### plotting both the suitability in space and the trends ####
# population trends
ANIM_DIR <- paste0(PLOT_DIR, "/animation/population") # writes new data to the DEMO_DIR

if (file.exists(ANIM_DIR)) {
  cat("The folder already exists")
} else {
  dir.create(ANIM_DIR)
}

ts_mean <- data.table(aggregate(. ~ Year, data = ts_hs[,c(3:6)], mean)) # averages across all selected sims
pb <- txtProgressBar(max = nlayers(maori), style = 3)
progress <- function(n) setTxtProgressBar(pb, n)
opts <- list(progress = progress)

seq = seq_len(nlayers(maori))
maori[[1]]


cls <- parallel::makeCluster(5)
doSNOW::registerDoSNOW(cls)

foreach(r = seq,
        .packages = c("raster", "ggplot2", "sf", "data.table", "patchwork"),
        .export = "maori",
        .inorder = TRUE, .verbose = FALSE,
        .options.snow = opts) %dopar% {
          if (file.exists(paste0(ANIM_DIR, paste0(sprintf('plot_%04d', time[r]), ".jpg")))) {
            return(NULL)
          } else {
            # convert habitat suitability rasters to data frames
            lay_stack <- stack(get("maori")[[r]])
            names(lay_stack) <- "Maori"
            stack_df <- as.data.frame(lay_stack, xy=TRUE, long=TRUE)
                }
          
          # plot habitat suitability and fossil locations
          p1 <- ggplot(data = stack_df,
                       aes(x = x, y = y, fill = log(value))) +
            geom_raster() +
            facet_wrap(~layer, ncol=1) +
            scale_fill_gradient2(low = "white", mid = "yellow",high = "red",
                                 midpoint = 3.5,
                                 limits = c(0,7),
                                 na.value = "transparent",
                                 guide = guide_colourbar(title = "NA",
                                                         title.position = "right",
                                                         title.theme = element_text(size = 8, colour = "black", face='bold'),
                                                         title.hjust = 0.5,
                                                         title.vjust = 5,
                                                         label = TRUE,
                                                         label.position = "right",
                                                         barwidth = 0.5,
                                                         barheight = 15,
                                                         ticks = FALSE,
                                                         direction = "vertical",
                                                         frame.colour = "black")) +
            geom_polygon(data=NZ, aes(x=long, y=lat, group=group), fill='transparent', col="black") +
            theme_bw() +
            theme(legend.position = "right", axis.text = element_blank()) +
            labs(x = NULL, y = NULL) +
            ggtitle(paste(time[r], "log population size"))
          
          # plot range area
          p4 <- ggplot() + geom_line(data = ts_hs[(Year-1) <= time[r]], aes(x=Year, y=Range*(20*20)), size=0.75, col = "lightblue") + 
            geom_line(data = ts_mean[(Year-1) <= time[r]], aes(x=Year, y=Range*(20*20)), size=1.5, col = "black") +
            xlab("Year AD") +
            ylab("Range (sq. km)") +
            theme_bw() +
            xlim(min(time), max(time)) +
            ylim(min(ts_hs$Range*(20*20), na.rm=T), max(ts_hs$Range*(20*20), na.rm = T)) +
            ggtitle("Range area")
          
          # save as jpg
          ggsave(filename = paste0(sprintf('plot_%04d', r), ".jpg"),
                 plot = (p1 + p4) + plot_layout(ncol = 2, widths = c(2,1.2)),
                 path = paste0(ANIM_DIR),
                 height = 10, width = 15, units = "in", dpi = 300)
        }


registerDoSEQ(); gc();
parallel::stopCluster(cls)


# model agreement

maori<-stack(file.path(MOD_DIR,"model_agreement.grd"))  # reads new data from the PLOT_DIR 
maori<-maori[[c(381:951)]] #1230 - 1800

sum(raster::colSums(maori[[1]], na.rm = TRUE))
sum(raster::colSums(maori[[540]], na.rm = TRUE))

landsea <- raster(file.path(DATA_DIR, "NZ_landsea.grd"))

# import New Zealand coastline shapefile
NZ<-readOGR(file.path(DATA_DIR, "NZ_coast.shp"))
NZ <- spTransform(NZ, raster::crs(landsea))
NZ <- crop(NZ, raster::extent(landsea))
raster::plot(NZ)

# define time frame
min<-as.numeric(str_sub(unlist(names(maori)[[1]]),2))
max<-as.numeric(str_sub(unlist(names(maori)[[571]]),2))

# time is sorted into 20 year bins here because the habitat suitability rasters are stacked by 20 year layers
time <- sort(seq(min, max, 1), decreasing = F)

#### create dataframe for suitability trend ####
sp1 <- get("maori")
values(sp1)[values(sp1) == 0] <- NA
spplot(stack(sp1[[1]]))
ts_hs1 <- raster::cellStats(sp1, mean, na.rm = TRUE)
sd1 <- raster::cellStats(sp1, sd, na.rm = TRUE)


# adding range area trend
sp1[!is.na(sp1)] <- 1 ## everything else to 1
r_hs1 <- cellStats(sp1, sum) ## sum == count of cells that are suitable

## create data.table
ts_hs <- list()
ts_hs[[1]] <- data.table(ts_hs1)
ts_hs[[1]] <- ts_hs[[1]][, .(Suitability = ts_hs1)]
ts_hs[[1]][, Year := time]
ts_hs[[1]][, SD := sd1]
ts_hs[[1]][, Range := r_hs1]
ts_hs[[1]][, DB := "Maori"]
ts_hs <- rbindlist(ts_hs)
ts_hs

# remove files that won't be needed
rm(aw_noz1, aw_noz2, sp1, sp2, weighted_hs1, weighted_hs2, r_hs1, r_hs2, sd1, sd2, ts_hs1, ts_hs2); gc()

ANIM_DIR <- paste0(PLOT_DIR, "/animation/model_agreement") # writes new data to the DEMO_DIR

if (file.exists(ANIM_DIR)) {
  cat("The folder already exists")
} else {
  dir.create(ANIM_DIR)
}

pb <- txtProgressBar(max = nlayers(maori), style = 3)
progress <- function(n) setTxtProgressBar(pb, n)
opts <- list(progress = progress)

seq = seq_len(nlayers(maori))
maori[[50]]


cls <- parallel::makeCluster(5)
doSNOW::registerDoSNOW(cls)

foreach(r = seq,
        .packages = c("raster", "ggplot2", "sf", "data.table", "patchwork"),
        .export = "maori",
        .inorder = TRUE, .verbose = FALSE,
        .options.snow = opts) %dopar% {
          if (file.exists(paste0(ANIM_DIR, paste0(sprintf('plot_%04d', time[r]), ".jpg")))) {
            return(NULL)
          } else {
            # convert habitat suitability rasters to data frames
            lay_stack <- stack(get("maori")[[r]])
            names(lay_stack) <- "Maori"
            values(lay_stack)[values(lay_stack) == 0] <- NA
            stack_df <- as.data.frame(lay_stack, xy=TRUE, long=TRUE)
          }
          
          # plot habitat suitability and fossil locations
          p1 <- ggplot(data = stack_df,
                       aes(x = x, y = y, fill = value)) +
            geom_raster() +
            facet_wrap(~layer, ncol=1) +
            scale_fill_gradient2(low = "white", mid = "yellow",high = "limegreen",
                                 midpoint = 0.5,
                                 limits = c(0,1),
                                 na.value = "transparent",
                                 guide = guide_colourbar(title = "NA",
                                                         title.position = "right",
                                                         title.theme = element_text(size = 8, colour = "black", face='bold'),
                                                         title.hjust = 0.5,
                                                         title.vjust = 5,
                                                         label = TRUE,
                                                         label.position = "right",
                                                         barwidth = 0.5,
                                                         barheight = 15,
                                                         ticks = FALSE,
                                                         direction = "vertical",
                                                         frame.colour = "black")) +
            geom_polygon(data=NZ, aes(x=long, y=lat, group=group), fill='transparent', col="black") +
            theme_bw() +
            theme(legend.position = "right", axis.text = element_blank()) +
            labs(x = NULL, y = NULL) +
            ggtitle(paste(time[r], "mapped simulation agreement"))
          
          
          
          # plot range area
          p4 <- ggplot() + 
            geom_line(data = ts_hs[(Year-1) <= time[r]], aes(x=Year, y=Suitability), size=1.5, col = "black") +
            xlab("Year AD") +
            ylab("Simulation agreement") +
            theme_bw() +
            xlim(min(time), max(time)) +
            ylim(0, 1) +
            ggtitle("Mean simluation agreement")
          
          # save as jpg
          ggsave(filename = paste0(sprintf('plot_%04d', r), ".jpg"),
                 plot = (p1 + p4) + plot_layout(ncol = 2, widths = c(2,1.2)),
                 path = paste0(ANIM_DIR),
                 height = 10, width = 15, units = "in", dpi = 300)
        }


registerDoSEQ(); gc();
parallel::stopCluster(cls)

# ANIMATING THE LAYERS THROUGH TIME ####
# ffmpeg runs through the command prompt or Windows Powershell
# Navigate to the directory containing all the images printed above:

# C:\Users\a1128091\Desktop\Hypervolume_plots\animation>

# enter this bash script:
#ffmpeg -r 10 -i plot_%04d.jpg -vf "pad=ceil(iw/2)*2:ceil(ih/2)*2" P1_D1comparisons.mp4

# 10 is the frame rate (10 frames a second)
# smaller numbers are slower

IMGS <- list.files(ANIM_DIR, pattern = ".jpg", full.names = TRUE)
new_files <- paste0(ANIM_DIR,"/plot_",formatC(rev(1:length(IMGS)), width=4, flag = "0"),".jpg")

file.copy(from = IMGS, to = new_files)
file.remove(IMGS)
